module.exports = function (grunt) {
    var webpack = require('webpack');
    var webpackConfig = require('./webpack.config.js');

    // Project configuration.
    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),
        lifecycle: {
            compile: [
                'webpack:build-min'
            ],
            test: [
                'karma:unit'
            ],
            report: [
                'karma:report', 'eslint'
            ],
            default: []
        },
        eslint: {
            options: {
                silent: true,
                outputFile: 'reports/eslint-checkstyle.html',
                format: 'html'
            },
            all: {
                src: [
                    'src/**/*.js',
                    'test/**/*.js'
                ]
            }
        },
        karma: {
            options: {
                configFile: 'karma.conf.js',
                browsers: (function () {
                    var userBrowser = grunt.option('browser');
                    if (userBrowser) {
                        return [userBrowser];
                    } else {
                        return ['chrome-small', 'Firefox'];
                    }
                })(),
                singleRun: !grunt.option('watch'),
                autoWatch: grunt.option('watch'),
                client: {
                    mocha: {
                        timeout: 50000 // increase global test timeout because of code instrumentation
                    }
                }
            },
            unit: {
                codeCoverage: false
            },
            report: {
                browsers: ['chrome-small'],
                codeCoverage: true
            }
        },
        webpack: {
            options: webpackConfig,
            'build': {},
            'build-min': {
                plugins: webpackConfig.plugins.concat(
                    new webpack.optimize.DedupePlugin(),
                    new webpack.optimize.UglifyJsPlugin()
                )
            },
            'build-dev': {
                devtool: 'sourcemap',
                debug: true
            }
        },
        'webpack-dev-server': {
            options: {
                webpack: webpackConfig,
                publicPath: '/' + webpackConfig.output.publicPath
            },
            start: {
                keepAlive: true,
                webpack: {
                    devtool: 'eval',
                    debug: true
                }
            }
        },
        watch: {
            webpack: {
                files: ['src/**/*'],
                tasks: ['webpack:build-dev']
            },
            tdd: {
                files: ['src/**/*', 'test/**/*'],
                tasks: ['webpack:build-dev', 'karma:unit']
            }
        }
    });
    require('load-grunt-tasks')(grunt);

    // Load the plugins
    grunt.loadNpmTasks('grunt-karma');
    grunt.loadNpmTasks('gruntify-eslint');

    // task(s).
    grunt.registerTask('dev', ['watch:tdd']);
};
