const path = require('path');

module.exports = function (config) {
    config.set({
        logLevel: config.LOG_INFO,

        // base path that will be used to resolve all patterns (eg. files, exclude)
        basePath: '',


        // frameworks to use
        // available frameworks: https://npmjs.org/browse/keyword/karma-adapter
        frameworks: ['mocha', 'chai', 'chai-sinon'],


        proxies: {},


        // list of files / patterns to load in the browser
        files: [
            {pattern: 'resources/**/*.*', included: false},

            'test_index.js'
        ],

        webpack: {
            isparta: function () {
                if (!config.codeCoverage) return {};

                return {
                    embedSource: true,
                    noAutoWrap: true,
                    // these babel options will be passed only to isparta and not to babel-loader
                    babel: {
                        presets: ['es2015']
                    }
                };
            }(),
            module: {
                loaders: function () {
                    if (!config.codeCoverage) {
                        return [{
                            test: /\.js$/,
                            include: [path.resolve('src/'), path.resolve('test/')],
                            loader: 'babel?presets[]=es2015!imports?this=>window'
                        }];
                    }
                    return [
                        {
                            test: /\.js$/,
                            include: [path.resolve('test/')],
                            loader: 'babel?presets[]=es2015!imports?this=>window'
                        },
                        {
                            test: /\.js$/,
                            include: path.resolve('src/'),
                            loader: 'isparta!imports?this=>window'
                        }
                    ]
                }()
            },
            devtool: 'inline-source-map'
        },

        preprocessors: {
            'test_index.js': ['webpack', 'sourcemap']
        },

        // list of files to exclude
        exclude: [],

        // test results reporter to use
        // possible values: 'dots', 'progress'
        // available reporters: https://npmjs.org/browse/keyword/karma-reporter
        reporters: ['mocha', 'junit', 'coverage'],

        junitReporter: {
            outputFile: 'reports/tests.xml',
            suite: ''
        },

        coverageReporter: {
            dir: 'reports/',
            subdir: '.',
            reporters: [
                {type: 'cobertura', subdir: 'coverage', file: 'cobertura.xml'},
                {type: 'html', subdir: 'coverage', file: 'cobertura.html'},
                {type: 'lcov', subdir: 'coverage'}
            ]
        },

        // web server port
        port: 9876,


        // enable / disable colors in the output (reporters and logs)
        colors: true,

        // enable / disable watching file and executing tests whenever any file changes
        autoWatch: true,

        browserDisconnectTolerance: 10,
        captureConsole: true,
        browserNoActivityTimeout: 100000,

        // start these browsers
        // available browser launchers: https://npmjs.org/browse/keyword/karma-launcher
        browsers: ['chrome-small', 'Firefox'],

        customLaunchers: {
            'chrome-small': {
                base: 'Chrome',
                flags: [
                    '--window-size=300,200'
                ]
            }
        },

        // Continuous Integration mode
        // if true, Karma captures browsers, runs the tests and exits
        singleRun: false
    })
};
