const _ = require('lodash'),
    distNameUtils = require('../utils/distNameUtils');

module.exports = Object.freeze({
    newAlarm: newAlarm,
    newHistoricalAlarm: newHistoricalAlarm,
    newChangeAlarm: newChangeAlarm,
    isAlarm: isAlarm,
    sortUsingObservationTime,
    sortUsingEventTime
});

const ALARM_L_CLASS_NAME = 'ALARM_L';

function newAlarm(distName, parameters) {
    const alarmInformation = parameters.alarmInformation;
    const alarm = {
        severity: alarmInformation.alarmSeverity,
        faultId: alarmInformation.faultId,
        number: alarmInformation.alarmNumber,
        distName: distName,
        btsObservationTime: alarmInformation.observationTime,
        eventType: alarmInformation.eventType,
        details: alarmInformation.alarmText.alarmDetail,
        notifier: parameters.unitInformation.alarmingResource,
        isEqualTo,
        isNew,
        hasSeverity
    };

    return alarm;

    function isEqualTo(otherAlarm) {
        return alarm.severity === otherAlarm.severity &&
            alarm.faultId === otherAlarm.faultId &&
            alarm.number === otherAlarm.number &&
            alarm.distName === otherAlarm.distName &&
            alarm.btsObservationTime === otherAlarm.btsObservationTime &&
            alarm.eventType === otherAlarm.eventType &&
            alarm.details === otherAlarm.details &&
            alarm.notifier === otherAlarm.notifier;
    }

    function isNew() {
        return !alarm.stateBefore;
    }

    function hasSeverity(severity) {
        return alarm.severity.toLowerCase() === severity.toLowerCase();
    }
}

function newHistoricalAlarm(distName, parameters, timestamp) {
    const alarm = newAlarm(distName, parameters);
    alarm.eventTime = timestamp;
    alarm.updateDisappearTime = updateDisappearTime;
    alarm.disappeared = disappeared;
    return alarm;

    function updateDisappearTime(timestamp) {
        alarm.disappearTime = timestamp;
        if (alarm.stateBefore) alarm.stateBefore.updateDisappearTime(timestamp);
    }

    function disappeared() {
        return !!alarm.disappearTime;
    }
}

function newChangeAlarm(changedAlarm, distName, parameters, timestamp) {
    const alarm = newHistoricalAlarm(distName, parameters, timestamp);
    alarm.stateBefore = changedAlarm;
    return alarm;
}

function isAlarm(distName) {
    return distNameUtils.getClassName(distName) === 'ALARM_L';
}


function sortUsingObservationTime(alarms) {
    alarms.sort(function (alarm1, alarm2) {
        return (alarm1.btsObservationTime - alarm2.btsObservationTime) ||
            distNameUtils.compareDistNames(alarm1.distName, alarm2.distName);
    });
}

function sortUsingEventTime(alarms) {
    alarms.sort(function (alarm1, alarm2) {
        return (alarm1.eventTime - alarm2.eventTime) ||
            distNameUtils.compareDistNames(alarm1.distName, alarm2.distName);
    });
}

