module.exports = Object.freeze({
    getDownlinkFrequency,
    getUplinkFrequency,
    getMnc,
    getMcc,
    getCellId
});

function getDownlinkFrequency(node) {
    const params = node.getDecodedParameters();
    if (isLNADJL(node)) {
        if (params.fDlEarfcn)
            return {
                name: 'fDlEarfcn',
                value: params.fDlEarfcn
            };
    } else if (isLNADJW(node)) {
        if (params.uTargetFreq)
            return {
                name: 'uTargetFreq',
                value: params.uTargetFreq
            };
    } else if (isLNCEL(node)) {
        if (params.earfcnDL)
            return {
                name: 'earfcnDL',
                value: params.earfcnDL
            };
    }
    return null;
}

function getUplinkFrequency(node) {
    const params = node.getDecodedParameters();
    if (isLNADJL(node)) {
        if (params.fUlEarfcn)
            return {
                name: 'fUlEarfcn',
                value: params.fUlEarfcn
            };
    } else if (isLNCEL(node)) {
        if (params.earfcnUL)
            return {
                name: 'earfcnUL',
                value: params.earfcnUL
            };
    }
    return null;
}

function getMnc(node) {
    const levelWithMncAndMcc = getCorrectLevelForMncAndMcc(node);
    if (levelWithMncAndMcc)
        return {
            name: 'mnc',
            value: levelWithMncAndMcc.mnc
        };
    return null;
}

function getMcc(node) {
    const levelWithMncAndMcc = getCorrectLevelForMncAndMcc(node);
    if (levelWithMncAndMcc)
        return {
            name: 'mcc',
            value: levelWithMncAndMcc.mcc
        };
    return null;
}

function getCorrectLevelForMncAndMcc(node) {
    const params = node.getDecodedParameters();
    if (isLNADJL(node))
        return params.ecgiPlmnId;
    else if (isLNADJW(node))
        return params.primaryPlmnId;
    else if (isLNADJG(node))
        return params;
    return null;
}

function getCellId(cellObject) {
    const params = cellObject.getDecodedParameters();
    if (isLNADJL(cellObject))
        return {
            name: 'phyCellId',
            shortName: 'PCI',
            value: params.phyCellId
        };
    else if (isLNADJW(cellObject))
        return {
            name: 'uTargetCid',
            shortName: 'CGI',
            value: params.uTargetCid
        };
    else if (isLNADJG(cellObject))
        return {
            name: 'gTargetCi',
            shortName: 'CI',
            value: params.gTargetCi
        };

    return null;
}

function isLNADJL(node) {
    return node.className === 'LNADJL' || node.className === 'LNADJL_A';
}

function isLNADJW(node) {
    return node.className === 'LNADJW' || node.className === 'LNADJW_A';
}

function isLNADJG(node) {
    return node.className === 'LNADJG' || node.className === 'LNADJG_A';
}

function isLNCEL(node) {
    return node.className === 'LNCEL' || node.className === 'LNCEL_A';
}
