module.exports = {
    addHandler: addHandler,
    handle: handle,

    // only for testing
    setNotifierInterval: setNotifierInterval
};

const logger = require('../logger.impl');
const intervalNotifier = require('./intervalNotifier');
const ADMIN_API_STATUSES = require('./adminApiStatuses');
const requestTypeHandlers = {};

const LOG = logger.getLogger('admin.core.remoteApi');
var intervalNotifierMs = 5000;
var msgRequestId;

function addHandler(requestType, handler) {
    requestTypeHandlers[requestType] = handler;
}

function handle(data, testServerSocket) {
    var decoded = decodeAndValidateMessage(data);
    if (!decoded) {
        var erroMsg;
        if (JsonValidator.error.dataPath !== '')
            erroMsg = JsonValidator.error.dataPath + ', ' + JsonValidator.error.message;
        else
            erroMsg = JsonValidator.error.message;
        send(testServerSocket, null, ADMIN_API_STATUSES.FAILED, erroMsg);
        return;
    }

    msgRequestId = decoded.requestId;

    const handler = requestTypeHandlers[decoded.requestType];
    if (!handler) {
        const errorMsg = 'No such request type';
        send(testServerSocket, msgRequestId, ADMIN_API_STATUSES.FAILED, errorMsg);
        LOG.error(errorMsg);
        return;
    }

    startIntervalNotifier(testServerSocket);
    handler(decoded.parameters, createCompletionListener(testServerSocket));
}

function decodeAndValidateMessage(data) {
    try {
        const decoded = JSON.parse(data);
        const validated = JsonValidator.validate(decoded, {
            type: 'object',
            id: 'message',
            properties: {
                requestId: {type: 'number'},
                requestType: {type: 'string'},
                parameters: {type: 'object'}
            },
            required: ['requestId', 'requestType', 'parameters']
        });
        return validated && decoded;
    } catch (e) {
        LOG.error('Invalid message: ' + data);
        return null;
    }
}

function createCompletionListener(testServerSocket) {
    return function (requestStatus, payload) {
        if (requestStatus === ADMIN_API_STATUSES.COMPLETED || requestStatus === ADMIN_API_STATUSES.FAILED)
            intervalNotifier.stop();
        else
            intervalNotifier.restart();

        send(testServerSocket, msgRequestId, requestStatus, payload);
    };
}

function startIntervalNotifier(testServerSocket) {
    intervalNotifier.start(function () {
        send(testServerSocket, msgRequestId, ADMIN_API_STATUSES.ONGOING, 'Waiting for response');
    }, intervalNotifierMs);
}

function send(socket, requestId, requestStatus, payload) {
    socket.send(JSON.stringify({
        requestId: requestId,
        requestStatus: requestStatus,
        payload: payload
    }));
}

function setNotifierInterval(intervalMs) {
    intervalNotifierMs = intervalMs;
}
