module.exports = connectionInitializer;

const connectionInitializerStepsFactory = require('./connectionInitializerSteps');


function connectionInitializer(onlineImTree, connectionService) {
    const connectionInitializerSteps = connectionInitializerStepsFactory(onlineImTree, connectionService);
    const steps = [];

    return {
        connectToBts,

        // visible for testing
        addNewStep,
        prepare,
        getSteps
    };

    function connectToBts() {
        addNewStep(connectionInitializerSteps.connectToBts, []);
        return {
            checkIfDnExists,
            prepare
        };
    }

    function checkIfDnExists(distName) {
        addNewStep(connectionInitializerSteps.checkIfDnExists, [distName]);
        return {
            prepare
        };
    }

    function addNewStep(callback, args) {
        steps.push(() => callback.apply(this, args));
    }

    function prepare() {
        return new Promise((resolve, reject) => {
            prepareRecursively(resolve, reject);
        });
    }

    function prepareRecursively(resolve, reject) {
        let promise = Promise.resolve();
        callNextStep();

        function callNextStep() {
            const step = steps.shift();
            if (!step) {
                resolve();
                return;
            }

            promise = promise.then(step)
                .then(callNextStep)
                .catch((reason) => {
                    steps.length = 0;
                    reject(reason);
                });
        }
    }

    function getSteps() {
        return steps;
    }
}
