module.exports = connectionInitializerSteps;

function connectionInitializerSteps(onlineImTree, connectionService) {
    return {
        connectToBts,
        checkIfDnExists
    };

    function connectToBts() {
        return new Promise(function (resolve, reject) {
            if (connectionService.isSignedIn()) {
                resolve();
            } else {
                const listener = {
                    signIn: function () {
                        connectionService.removeListener(this);
                        resolve();
                    },
                    error: function (error) {
                        reject(error);
                        connectionService.removeListener(this);
                    }
                };
                connectionService.addListener(listener);
                connectionService.toggleConnection();
            }
        });
    }

    function checkIfDnExists(distName) {
        return new Promise(function (resolve, reject) {
            onlineImTree.getImTree().then(function (completeImTree) {
                const object = completeImTree.findByDistName(distName);
                if (object) {
                    resolve();
                } else {
                    reject('No object with distname: ' + distName);
                }
            });
        });
    }
}
