module.exports = getNbrDataAction;

const validationLookup = require('../../../adjacency/adjacencyValidationLookup');

function getNbrDataAction({
    onlineImTree,
    adjacencyValidation
}) {
    return {
        execute
    };

    function getLoosEnds(technologyValidation) {
        return technologyValidation.relationStatuses.warnings.map(rel => {
            const lnrel = `${rel.className}-${rel.id}`;
            return {
                STATUS: 'loose end',
                LNREL: lnrel,
                LNADJL: '',
                CELL: ''
            };
        });
    }

    function getDuplicated(technologyValidation) {
        return technologyValidation.relationStatuses.errors.map(rel => {
            const lnrel = `${rel.className}-${rel.id}`;
            return {
                STATUS: 'duplicated relations',
                LNREL: lnrel,
                LNADJL: '',
                CELL: ''
            };
        });
    }

    function createNbrObjectLte(technologyValidation) {
        const unused = technologyValidation.cellStatuses.infos.map(cell => {
            const decodedParameters =  cell.getDecodedParameters();
            return {
                STATUS: 'unused',
                LNREL: '',
                LNADJL: `${cell.className}-${cell.id}`,
                CELL: {
                    ecgiAdjEnbId: decodedParameters.ecgiAdjEnbId,
                    ecgiLcrId: decodedParameters.ecgiLcrId,
                    mcc: decodedParameters.ecgiPlmnId.mcc,
                    mnc: decodedParameters.ecgiPlmnId.mnc,
                    sourceOfData: decodedParameters.sourceOfData,
                    phyCellId: `PCI-${decodedParameters.phyCellId}`,
                    fDlEarfcn: decodedParameters.fDlEarfcn,
                    fUlEarfcn: decodedParameters.fUlEarfcn
                }
            };
        });

        const correct = technologyValidation.cellStatuses.corrects.map(cell => {
            const decodedParameters =  cell.getDecodedParameters();
            const rel = validationLookup.findCorrectLnrel(cell, technologyValidation);
            return {
                STATUS: 'correct',
                LNREL: `${rel.className}-${rel.id}`,
                LNADJL: `${cell.className}-${cell.id}`,
                CELL: {
                    ecgiAdjEnbId: decodedParameters.ecgiAdjEnbId,
                    ecgiLcrId: decodedParameters.ecgiLcrId,
                    mcc: decodedParameters.ecgiPlmnId.mcc,
                    mnc: decodedParameters.ecgiPlmnId.mnc,
                    sourceOfData: decodedParameters.sourceOfData,
                    phyCellId: `PCI-${decodedParameters.phyCellId}`,
                    fDlEarfcn: decodedParameters.fDlEarfcn,
                    fUlEarfcn: decodedParameters.fUlEarfcn
                }
            };
        });

        const looseEndRel = getLoosEnds(technologyValidation);
        const duplicatedRel = getDuplicated(technologyValidation);

        const results = correct.concat(unused, looseEndRel, duplicatedRel);
        return results;
    }


    function createNbrObjectUtran(technologyValidation) {
        const unused = technologyValidation.cellStatuses.infos.map(cell => {
            const decodedParameters =  cell.getDecodedParameters();
            return {
                STATUS: 'unused',
                LNREL: '',
                LNADJL: `${cell.className}-${cell.id}`,
                CELL: {
                    uTargetCid: decodedParameters.uTargetCid,
                    uTargetFreq: decodedParameters.uTargetFreq,
                    uTargetLac: decodedParameters.uTargetLac,
                    uTargetRac: decodedParameters.uTargetRac,
                    uTargetScFdd: decodedParameters.uTargetScFdd,
                    mcc: decodedParameters.primaryPlmnId.mcc,
                    mnc: decodedParameters.primaryPlmnId.mnc
                }
            };
        });

        const correct = technologyValidation.cellStatuses.corrects.map(cell => {
            const decodedParameters =  cell.getDecodedParameters();
            const rel = validationLookup.findCorrectLnrel(cell, technologyValidation);
            return {
                STATUS: 'correct',
                LNREL: `${rel.className}-${rel.id}`,
                LNADJL: `${cell.className}-${cell.id}`,
                CELL: {
                    uTargetCid: decodedParameters.uTargetCid,
                    uTargetFreq: decodedParameters.uTargetFreq,
                    uTargetLac: decodedParameters.uTargetLac,
                    uTargetRac: decodedParameters.uTargetRac,
                    uTargetScFdd: decodedParameters.uTargetScFdd,
                    mcc: decodedParameters.primaryPlmnId.mcc,
                    mnc: decodedParameters.primaryPlmnId.mnc
                }
            };
        });

        const looseEndRel = getLoosEnds(technologyValidation);
        const duplicatedRel = getDuplicated(technologyValidation);

        const results = correct.concat(unused, looseEndRel, duplicatedRel);
        return results;
    }

    function createNbrObjectLteGeran(technologyValidation) {
        const unused = technologyValidation.cellStatuses.infos.map(cell => {
            const decodedParameters =  cell.getDecodedParameters();
            return {
                STATUS: 'unused',
                LNREL: '',
                LNADJL: `${cell.className}-${cell.id}`,
                CELL: {
                    gTargetCi: decodedParameters.gTargetCi,
                    gTargetLac: decodedParameters.gTargetLac,
                    gTargetRac: decodedParameters.gTargetRac,
                    mcc: decodedParameters.mcc,
                    mnc: decodedParameters.mnc
                }
            };
        });

        const correct = technologyValidation.cellStatuses.corrects.map(cell => {
            const decodedParameters =  cell.getDecodedParameters();
            const rel = validationLookup.findCorrectLnrel(cell, technologyValidation);
            return {
                STATUS: 'correct',
                LNREL: `${rel.className}-${rel.id}`,
                LNADJL: `${cell.className}-${cell.id}`,
                CELL: {
                    gTargetCi: decodedParameters.gTargetCi,
                    gTargetLac: decodedParameters.gTargetLac,
                    gTargetRac: decodedParameters.gTargetRac,
                    mcc: decodedParameters.mcc,
                    mnc: decodedParameters.mnc
                }
            };
        });

        const looseEndRel = getLoosEnds(technologyValidation);
        const duplicatedRel = getDuplicated(technologyValidation);

        const results = correct.concat(unused, looseEndRel, duplicatedRel);
        return results;
    }

    function createNbrObjectForCell(cell, validationData) {
        const nbrObjectBuilders = Object.create({
            'LTE': createNbrObjectLte,
            'UTRAN': createNbrObjectUtran,
            'GERAN': createNbrObjectLteGeran
        });
        const lncellWithValidation = validationLookup.findLncel(cell.localCell, validationData);
        const nbrObject = {};
        lncellWithValidation.technologyValidations.forEach(technologyValidation => {

            nbrObject[technologyValidation.technology] = nbrObjectBuilders[technologyValidation.technology](technologyValidation);
            return;
        });
        return nbrObject;
    }

    function filterCells(cellName) {
        return cell => `${cell.localCell.className}-${cell.localCell.id}` === cellName;
    }

    function execute(msgData, resolve, reject) {
        onlineImTree.getImTree()
            .then((imTree) => {
                const validationData = adjacencyValidation.validateAdjacencies(imTree);
                const filteredCells = validationData.localCellValidations
                    .filter((msgData.parameters && msgData.parameters.cellName) ? filterCells(msgData.parameters.cellName) : () => true);
                if (filteredCells.length === 0) {
                    reject('No local cells with any relations were found.');
                } else {
                    const results = {};
                    filteredCells.forEach(cell => {
                        const cellName = `${cell.localCell.className}-${cell.localCell.id}`;
                        results[cellName] = createNbrObjectForCell(cell, validationData);
                    });

                    resolve(results);
                }
            })
            .catch(reason => reject(reason.message));
    }
}
