module.exports = infoModelDispatcher;

const ADMIN_API_STATUSES = require('../adminApiStatuses');
const adminApi = require('../adminApi');
const adminMessageValidator = require('../adminMessageValidator');
const logger = require('../../logger.impl');
const LOG = logger.getLogger('admin.core.remoteApi.infoModel');

function infoModelDispatcher({
    sfpMonitoring = {},
    parametersGetter = {},
    lnbtsState = {},
    abnormalities = {},
    adjacencies = {},
    timings = {},
    baseband = {},
    stateInfo = {}
    }) {

    const EXECUTORS = {
        sfp: sfpMonitoring.execute,
        getParameters: parametersGetter.execute,
        lnbtsState: lnbtsState.execute,
        abnormalities: abnormalities.execute,
        adjacencies: adjacencies.execute,
        timings: timings.execute,
        baseband: baseband.execute,
        stateInfo: stateInfo.execute
    };

    adminApi.addHandler('infoModel', dispatch);

    return {
        // visible for testing
        dispatch
    };

    function dispatch(msgData, completionListener) {
        const validationResult = adminMessageValidator.validate(msgData, getProcedureSchema());
        if (!validationResult.status) {
            completionListener(ADMIN_API_STATUSES.FAILED, validationResult.message);
            return;
        }

        const remoteExecutor = EXECUTORS[msgData.name];
        if (!remoteExecutor) {
            completionListener(ADMIN_API_STATUSES.FAILED, `No "${msgData.name}" infoModel getter in testing API`);
            return;
        }

        remoteExecutor(msgData, completionListener)
            .catch(reason => LOG.info(`Operation failed: ${reason}`));
    }

    function getProcedureSchema() {
        return {
            type: 'object',
            id: 'infoModel',
            properties: {
                name: {type: 'string'}
            },
            required: ['name']
        };
    }

}
