module.exports = lnbtsState;

const Promise = require('bluebird');
const ADMIN_API_STATUSES = require('../adminApiStatuses');

function lnbtsState({
    btsInfoService,
    onlineImTree,
    connectionService,
    connectionInitializer = require('../connectionInitializer')(onlineImTree, connectionService)
    }) {
    return {
        execute
    };

    function execute(msgData, completionListener) {
        return connectionInitializer.connectToBts()
            .prepare()
            .then(getLnbtsState)
            .then(result => {
                completionListener(ADMIN_API_STATUSES.COMPLETED, result);
            })
            .catch(reason => {
                completionListener(ADMIN_API_STATUSES.FAILED, reason);
                return Promise.reject(reason);
            });
    }

    function getLnbtsState() {
        return new Promise((resolve, reject) => {
            let actionEnded = false;
            const unwatcher = btsInfoService.watch(btsInfo => {
                actionEnded = true;
                if (!btsInfo.state) reject('Unable to find lnbts in imTree');
                else resolve(btsInfo.state);
                unwatch();
            });
            unwatch();

            function unwatch() {
                if (actionEnded && unwatcher) unwatcher.unwatch();
            }
        });
    }
}

