module.exports = parametersGetter;

const Promise = require('bluebird');
const ADMIN_API_STATUSES = require('../adminApiStatuses');
const adminMessageValidator = require('../adminMessageValidator');
const objects = require('../../utils/objects');

function parametersGetter({
    onlineImTree,
    connectionService,
    connectionInitializer = require('../connectionInitializer')(onlineImTree, connectionService)
    }) {

    return Object.freeze({
        execute
    });

    function execute(msgData, completionListener) {
        const validationResult = adminMessageValidator.validate(msgData, getProcedureSchema());
        if (!validationResult.status) {
            completionListener(ADMIN_API_STATUSES.FAILED, validationResult.message);
            return Promise.reject();
        }
        const distName = msgData.parameters.distName;
        const pattern = new RegExp(msgData.parameters.pattern);
        return prepareAndRun(getParams.bind(null, distName, pattern), distName, completionListener);
    }

    function prepareAndRun(executor, distName, completionListener) {
        return connectionInitializer.connectToBts()
            .checkIfDnExists(distName)
            .prepare()
            .then(() => executor())
            .then(data => {
                completionListener(ADMIN_API_STATUSES.COMPLETED, data);
            })
            .catch(reason => {
                completionListener(ADMIN_API_STATUSES.FAILED, reason);
                return Promise.reject(reason);
            });
    }

    function getParams(distName, pattern) {
        return onlineImTree.getImTree().then((completeImTree) => {
            const node = completeImTree.findByDistName(distName);
            const parameters = node.getDecodedParameters();
            return Promise.resolve(objects.filterObject(parameters, pattern));
        });
    }

    function getProcedureSchema() {
        return {
            type: 'object',
            id: 'procedure',
            properties: {
                name: {type: 'string'},
                parameters: {
                    type: 'object',
                    id: 'parameters',
                    properties: {
                        distName: {type: 'string'},
                        pattern: {type: 'string'}
                    },
                    required: ['distName']
                }
            },
            required: ['name', 'parameters']
        };
    }
}
