module.exports = timings;

const Promise = require('bluebird');
const ADMIN_API_STATUSES = require('../adminApiStatuses');
const adminMessageValidator = require('../adminMessageValidator');

function timings({
    onlineImTree,
    connectionService,
    timingServiceForOnlineImTree,
    connectionInitializer = require('../connectionInitializer')(onlineImTree, connectionService)
    }) {

    return {
        execute
    };

    function execute(msgData, completionListener) {
        const validationResult = adminMessageValidator.validate(msgData, schemaRouter[msgData.parameters.procedure]);
        if (!validationResult.status) {
            completionListener(ADMIN_API_STATUSES.FAILED, validationResult.message);
            return Promise.reject(validationResult.message);
        } else
            return connectionInitializer
                .connectToBts()
                .prepare()
                .then(() => {
                    timingServiceForOnlineImTree.newInstance(onlineImTree).then((timingsApi) => {
                        completionListener(ADMIN_API_STATUSES.COMPLETED, timingsApi[msgData.parameters.procedure](msgData.parameters.param || ''));
                    });
                })
                .catch(reason => {
                    completionListener(ADMIN_API_STATUSES.FAILED, reason);
                    return Promise.reject(reason);
                });
    }
}

const withParamSchema = {
    type: 'object',
    id: 'infoModel',
    properties: {
        name: {
            type: 'string',
            pattern: 'timings'
        },
        parameters: {
            type: 'object',
            properties: {
                procedure: {
                    type: 'string',
                    enum: ['findTimingsForProcedureName', 'findTimingsForDistName']
                },
                param: {
                    type: 'string'
                }
            },
            required: ['procedure', 'param']
        }
    },
    required: ['name', 'parameters']
};

const withoutParamSchema = {
    type: 'object',
    id: 'infoModel',
    properties: {
        name: {
            type: 'string',
            pattern: 'timings'
        },
        parameters: {
            type: 'object',
            properties: {
                procedure: {
                    type: 'string',
                    enum: ['findAllTimings', 'getAllProcedureNames']
                }
            },
            additionalProperties: false,
            required: ['procedure']
        }
    },
    required: ['name', 'parameters']
};

const schemaRouter = {
    findAllTimings: withoutParamSchema,
    findTimingsForProcedureName: withParamSchema,
    findTimingsForDistName: withParamSchema,
    getAllProcedureNames: withoutParamSchema
};
