module.exports = {
    adminApiSnapshotInitializer,
    initializeProceduresHandler,
    initializeInfoModelHandler
};

const zipFactory = require('../procedures/snapshot/zipFactory');
const snapshotListenerFactory = require('../procedures/snapshot/snapshotListenerFactory');
const snapshot = require('../procedures/snapshot/snapshot');
const distNameProcedures = require('../adminApi/procedures/distNameProcedures');
const imSnapshotProcedure = require('../adminApi/procedures/imSnapshotProcedure');
const snapshotProcedure = require('../adminApi/procedures/snapshotProcedure');
const commissioningProcedure = require('../adminApi/procedures/commissioningProcedure');
const commissioningExecutor = require('../procedures/commissioning/commissioningExecutor');
const resetProcedure = require('../adminApi/procedures/resetProcedure');
const proceduresDispatcher = require('../adminApi/procedures/proceduresDispatcher');
const sfpMonitoring = require('../adminApi/infoModel/sfpMonitoring');
const lnbtsState = require('../adminApi/infoModel/lnbtsState');
const parametersGetter = require('../adminApi/infoModel/parametersGetter');
const abnormalities = require('../adminApi/infoModel/abnormalities');
const adjacencies = require('../adminApi/infoModel/adjacencies/adjacencies');
const timings = require('../adminApi/infoModel/timings');
const timingServiceForOnlineImTree = require('../timings/timingServiceForOnlineImTree');
const baseband = require('../adminApi/infoModel/baseband');
const infoModelDispatcher = require('../adminApi/infoModel/infoModelDispatcher');
const stateInfo = require('../adminApi/infoModel/stateInfo');
const swUpdateProcedure = require('../adminApi/procedures/swUpdateProcedure');
const otherProcedure = require('../adminApi/procedures/otherProcedure');
const snapshotFileSaver = require('../adminApi/procedures/snapshotFileSaver');
const applicationInfo = require('../applicationInfo/applicationInfo');
const adjacencyValidation = require('../adjacency/adjacencyValidation');

function adminApiSnapshotInitializer(
    snapshotExecutor,
    filesInfoFactory,
    snapshotStatusHandler,
    procedureIdFactory,
    procedureTrackerFactory,
    fileTransferFactory,
    fileDownloaderFactory,
    onlineImTree) {

    const remoteZipFactory = zipFactory(snapshotFileSaver);
    const remoteSnapshotListenerFactory = snapshotListenerFactory(
        fileTransferFactory,
        remoteZipFactory,
        fileDownloaderFactory,
        onlineImTree
    );
    return snapshot(
        snapshotExecutor,
        remoteSnapshotListenerFactory,
        filesInfoFactory,
        snapshotStatusHandler,
        procedureIdFactory,
        procedureTrackerFactory
    );
}

function initializeProceduresHandler(
    btsServiceProcedure,
    procedureExecutor,
    onlineImTree,
    connectionService,
    procedureIdFactory,
    ims2Downloader,
    adminApiSnapshot,
    imAuth,
    softwareZipDispatcherFactory,
    swUpdateExecutorFactory,
    imTracker) {

    const distNameProceduresInst = distNameProcedures({
        btsServiceProcedure: btsServiceProcedure,
        procedureExecutor: procedureExecutor,
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        procedureIdFactory: procedureIdFactory
    });

    const imSnapshotProcedureInst = imSnapshotProcedure({
        ims2Downloader: ims2Downloader,
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        procedureIdFactory: procedureIdFactory
    });

    const snapshotProcedureInst = snapshotProcedure({
        snapshot: adminApiSnapshot,
        snapshotFileSaver: snapshotFileSaver,
        connectionService: connectionService,
        onlineImTree: onlineImTree
    });

    const commissioningProcedureInst = commissioningProcedure.newInstance({
        btsServiceProcedure: btsServiceProcedure,
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        commissioningExecutor: commissioningExecutor
            .newInstance(procedureExecutor, imAuth, applicationInfo, btsServiceProcedure),
        procedureExecutor: procedureExecutor,
        procedureIdFactory: procedureIdFactory
    });

    const resetProcedureInst = resetProcedure.newInstance({
        btsServiceProcedure: btsServiceProcedure,
        procedureExecutor: procedureExecutor,
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        procedureIdFactory: procedureIdFactory
    });

    const swUpdateProcedureInst = swUpdateProcedure({
        softwareZipDispatcherFactory: softwareZipDispatcherFactory,
        swUpdateExecutorFactory: swUpdateExecutorFactory,
        onlineImTree: onlineImTree,
        imTracker: imTracker,
        connectionService: connectionService
    });

    const otherProcedureInst = otherProcedure.newInstance({
        btsServiceProcedure: btsServiceProcedure,
        procedureExecutor: procedureExecutor,
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        procedureIdFactory: procedureIdFactory
    });

    proceduresDispatcher({
        distNameProcedures: distNameProceduresInst,
        imSnapshotProcedure: imSnapshotProcedureInst,
        snapshotProcedure: snapshotProcedureInst,
        commissioningProcedure: commissioningProcedureInst,
        resetProcedure: resetProcedureInst,
        swUpdateProcedure: swUpdateProcedureInst,
        otherProcedure: otherProcedureInst
    });
}

function initializeInfoModelHandler(
    alarmsMonitor,
    onlineImTree,
    connectionService,
    btsInfoService,
    dspsService) {
    const sfpMonitoringInst = sfpMonitoring.newInstance({
        onlineImTree: onlineImTree,
        connectionService: connectionService
    });
    const lnbtsStateInst = lnbtsState({
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        btsInfoService: btsInfoService
    });

    const parametersGetterInst = parametersGetter({
        onlineImTree: onlineImTree,
        connectionService: connectionService
    });

    const abnormalitiesInst = abnormalities({
        alarmsMonitor: alarmsMonitor,
        onlineImTree: onlineImTree,
        connectionService: connectionService
    });

    const adjacenciesInst = adjacencies({
        onlineImTree: onlineImTree,
        adjacencyValidation: adjacencyValidation,
        connectionService: connectionService
    });

    const timingsInst = timings({
        onlineImTree: onlineImTree,
        connectionService: connectionService,
        timingServiceForOnlineImTree: timingServiceForOnlineImTree
    });

    const basebandInst = baseband({
        dspsService: dspsService,
        onlineImTree: onlineImTree,
        connectionService: connectionService
    });

    const stateInfoInst = stateInfo({
        onlineImTree: onlineImTree,
        connectionService: connectionService
    });

    infoModelDispatcher({
        sfpMonitoring: sfpMonitoringInst,
        lnbtsState: lnbtsStateInst,
        parametersGetter: parametersGetterInst,
        abnormalities: abnormalitiesInst,
        adjacencies: adjacenciesInst,
        timings: timingsInst,
        baseband: basebandInst,
        stateInfo: stateInfoInst
    });
}
