const defaultProcedureFactory = require('./defaultProcedureFactory');
module.exports = {
    newInstance
};

const DOWNLOAD_PLAN_SCHEMA = {
    type: 'object',
    id: 'procedure',
    properties: {
        name: {type: 'string'},
        parameters: {
            type: 'object',
            id: 'parameters',
            properties: {
                planId: {type: 'string'},
                btsId: {type: 'string'},
                scf: {
                    type: 'array',
                    items: {
                        allOf: [{
                            type: 'number'
                        }]
                    }

                }
            },
            required: ['planId', 'btsId', 'scf']
        }
    },
    required: ['name', 'parameters']
};
const ACTIVATE_PLAN_SCHEMA = {
    type: 'object',
    id: 'procedure',
    properties: {
        name: {type: 'string'},
        parameters: {
            type: 'object',
            id: 'parameters',
            properties: {
                planId: {type: 'string'}
            },
            required: ['planId']
        }
    },
    required: ['name', 'parameters']
};

function newInstance({
    btsServiceProcedure,
    commissioningExecutor,
    onlineImTree,
    connectionService,
    procedureIdFactory = require('../../procedures/procedureIdFactory')({})
    }) {
    const procedureFactory = defaultProcedureFactory.newInstance({
        btsServiceProcedure,
        onlineImTree,
        connectionService,
        procedureIdFactory
    });

    return {
        executeDownloadPlan: procedureFactory.create(DOWNLOAD_PLAN_SCHEMA, commissioningExecutor.executeDownloadPlan),
        executeActivatePlan: procedureFactory.create(ACTIVATE_PLAN_SCHEMA, commissioningExecutor.executeActivatePlan)
    };
}
