const Promise = require('bluebird');
const ADMIN_API_STATUSES = require('../adminApiStatuses');
const adminMessageValidator = require('../adminMessageValidator');
const connectionInitializerFactory = require('../connectionInitializer');
const remoteProcedureListenerFactory = require('./remoteProcedureListener');

module.exports = {
    newInstance
};

function newInstance({
    btsServiceProcedure,
    onlineImTree,
    connectionService,
    procedureIdFactory = require('../../procedures/procedureIdFactory')({}),
    connectionInitializer = connectionInitializerFactory(onlineImTree, connectionService),
    remoteProcedureListener = remoteProcedureListenerFactory(btsServiceProcedure)
    }) {
    return {
        create
    };
    function create(procedureSchema, executor) {

        return execute;

        function execute(msgData, completionListener) {
            const validationResult = adminMessageValidator.validate(msgData, procedureSchema);
            if (!validationResult.status) {
                completionListener(ADMIN_API_STATUSES.FAILED, validationResult.message);
                return Promise.reject('Invalid schema');
            }
            return perform(msgData, completionListener);
        }

        function perform(msgData, completionListener) {
            return connectionInitializer.connectToBts()
                .prepare()
                .then(() => {
                    const procedureId = procedureIdFactory.getId().id;
                    listenUntilCompleted(completionListener, procedureId);
                    const parameters = msgData.parameters || {};
                    parameters.clientId = procedureId;
                    return executor(parameters);
                })
                .catch((reason) => {
                    completionListener(ADMIN_API_STATUSES.FAILED, reason);
                    return Promise.reject(reason);
                });
        }

        function listenUntilCompleted(completionListener, procedureId) {
            remoteProcedureListener.create(procedureId, completionListener, onlineImTree, connectionService);
            remoteProcedureListener.register();
        }
    }
}
