module.exports = imSnapshotProcedure;

const Promise = require('bluebird');
const ADMIN_API_STATUSES = require('../adminApiStatuses');

function imSnapshotProcedure({
    ims2Downloader,
    onlineImTree,
    connectionService,
    procedureIdFactory = require('../../procedures/procedureIdFactory')({}),
    connectionInitializer = require('../connectionInitializer')(onlineImTree, connectionService)
    }) {

    return {
        execute
    };

    function execute(msgData, completionListener) {
        const procedureId = procedureIdFactory.getId().id;
        return prepareAndRun(procedureId, completionListener);
    }

    function prepareAndRun(procedureId, completionListener) {
        return connectionInitializer.connectToBts()
            .prepare()
            .then(() => ims2Downloader.downloadDecompressed(procedureId))
            .then((result) => {
                const base64File = new Buffer(result.fileData, 'binary').toString('base64');
                completionListener(ADMIN_API_STATUSES.COMPLETED, base64File);
            })
            .catch((reason) => {
                completionListener(ADMIN_API_STATUSES.FAILED, reason);
                return Promise.reject(reason);
            });
    }

}
