const defaultProcedureFactory = require('./defaultProcedureFactory');

module.exports = {
    newInstance
};

const SW_ROLLBACK_SCHEMA = {
    type: 'object',
    id: 'procedure',
    properties: {
        name: {
            type: 'string',
            pattern: '^swRollback$'
        }
    },
    required: ['name']
};
const CHANGE_ETHERNET_PORT_SECURITY_SCHEMA = {
    type: 'object',
    id: 'procedure',
    properties: {
        name: {
            type: 'string',
            pattern: '^changeEthernetPortSecurity$'
        },
        parameters: {
            type: 'object',
            id: 'parameters',
            properties: {
                enableEthernetPortSec: {type: 'boolean'}
            },
            required: ['enableEthernetPortSec']
        }
    },
    required: ['name', 'parameters']
};
function newInstance({
    btsServiceProcedure,
    onlineImTree,
    connectionService,
    procedureExecutor,
    procedureIdFactory
    }) {

    const procedureFactory = defaultProcedureFactory.newInstance({
        btsServiceProcedure,
        onlineImTree,
        connectionService,
        procedureIdFactory
    });

    return {
        executeSwRollback: procedureFactory.create(SW_ROLLBACK_SCHEMA, procedureExecutor.executeSwRollback),
        executeChangeEthernetPortSecurity: procedureFactory.create(CHANGE_ETHERNET_PORT_SECURITY_SCHEMA, procedureExecutor.executeChangeEthernetPortSecurity)
    };
}
