module.exports = remoteProcedureListener;

const BTS_SERVICE_PROCEDURE_STATUS = require('../../procedures/btsServiceProcedureStatus');
const ADMIN_API_STATUSES = require('../adminApiStatuses');

var procedureListener;

function remoteProcedureListener(btsServiceProcedure) {
    return {
        create: create,
        register: register,
        unregister: unregister
    };

    function create(procedureId, completionListener) {
        procedureListener = {
            update: function (event) {
                event.objects.forEach(function (object) {
                    if (object.parameters.clientId !== procedureId) return;

                    const status = object.parameters.status;
                    completionListener(ADMIN_API_STATUSES.getRequestStatus(status), status);

                    if (BTS_SERVICE_PROCEDURE_STATUS.isFinal(status))
                        unregister();
                });
            }
        };
        return procedureListener;
    }

    function register() {
        btsServiceProcedure.registerListener(procedureListener);
    }

    function unregister() {
        btsServiceProcedure.removeListener(procedureListener);
    }
}
