module.exports = remoteResetListener;

const ADMIN_API_STATUSES = require('../adminApiStatuses');
const siteAvailabilityResponderFactory = require('../../procedures/availabilityResponders/siteAvailabilityResponder');
const imTrackerFactory = require('../../imTracker/imTracker');
const ResetRequestTracker = require('../../procedures/resetRequestTracker');

let procedureListener;

function remoteResetListener(btsServiceProcedure) {
    return {
        create: create,
        register: register,
        unregister: unregister
    };


    function create(procedureId, completionListener, onlineImTree, connectionService) {
        const imTracker = imTrackerFactory({onlineImTree: onlineImTree});
        const siteAvailabilityResponder = siteAvailabilityResponderFactory(imTracker, ResetRequestTracker.newInstance({imTracker: imTracker}));
        procedureListener = {
            update: function (event) {
                event.objects.forEach((object) => {
                    if (object.parameters.clientId !== procedureId) return;

                    const status = object.parameters.status;
                    let testStatus = ADMIN_API_STATUSES.getRequestStatus(status);
                    if (testStatus === ADMIN_API_STATUSES.COMPLETED)
                        testStatus = ADMIN_API_STATUSES.ONGOING;

                    completionListener(testStatus, status);

                    var connectedListener = {
                        connected: function () {
                            if (!siteAvailabilityResponder.isResetting()) {
                                completionListener(ADMIN_API_STATUSES.COMPLETED, 'Completed');
                                unregister();
                                connectionService.removeListener(connectedListener);
                            }
                        }
                    };
                    connectionService.addListener(connectedListener);

                });
            }
        };
        return procedureListener;
    }

    function register() {
        btsServiceProcedure.registerListener(procedureListener);
    }

    function unregister() {
        btsServiceProcedure.removeListener(procedureListener);
    }
}
