const defaultProcedureFactory = require('./defaultProcedureFactory');

module.exports = {
    newInstance
};

const RESET_SCHEMA = {
    type: 'object',
    id: 'procedure',
    properties: {
        name: {
            type: 'string'
        }
    },
    required: ['name']
};

const CONFIGURATION_RESET_SCHEMA = {
    type: 'object',
    id: 'procedure',
    properties: {
        name: {
            type: 'string',
            pattern: '^configurationReset$'
        },
        parameters: {
            type: 'object',
            id: 'parameters',
            properties: {
                removeSiteId: {type: 'boolean'},
                removeSecurityCredentials: {type: 'boolean'}
            },
            required: ['removeSiteId', 'removeSecurityCredentials']
        }
    },
    required: ['name', 'parameters']
};
function newInstance({
    btsServiceProcedure,
    onlineImTree,
    connectionService,
    procedureExecutor,
    procedureIdFactory,
    remoteResetListener = require('./remoteResetListener')(btsServiceProcedure)
    }) {

    const LNBTS_DISTNAME = '/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1';

    const procedureFactory = defaultProcedureFactory.newInstance({
        btsServiceProcedure,
        onlineImTree,
        connectionService,
        procedureIdFactory,
        remoteProcedureListener: remoteResetListener
    }), noop = () => null;


    return {
        executeBtsReset: procedureFactory.create(RESET_SCHEMA, createResetExecutor('BTS')),
        executeSiteReset: procedureFactory.create(RESET_SCHEMA, createResetExecutor('Site')),
        executeResetToTestDedicated: procedureFactory.create(RESET_SCHEMA, procedureExecutor.executeResetToTestDedicated),
        executeConfigurationReset: procedureFactory.create(CONFIGURATION_RESET_SCHEMA, procedureExecutor.executeConfigurationReset)
    };

    function createResetExecutor(coverage) {
        return (parameters) => {
            parameters.coverage = coverage;
            return procedureExecutor.executeReset('', LNBTS_DISTNAME, noop, parameters);
        };
    }
}

