module.exports = snapshotProcedure;

const SNAPSHOT_STATES = require('../../procedures/snapshot/snapshotStates');
const ADMIN_API_STATUSES = require('../adminApiStatuses');
const snapshotFileSaverFactory = require('./snapshotFileSaver');

function snapshotProcedure({
    snapshot,
    onlineImTree,
    connectionService,
    snapshotFileSaver = snapshotFileSaverFactory(),
    connectionInitializer = require('../connectionInitializer')(onlineImTree, connectionService)
    }) {

    var activeStatusListener;
    var activeCompletionListener;

    return {
        execute: execute
    };

    function fileSaverListener(status, message) {
        activeStatusListener.unwatch();
        activeStatusListener = null;

        activeCompletionListener(status, message);
    }

    function statusListener(status) {
        if (status === SNAPSHOT_STATES.DISCONNECTED
            || status === SNAPSHOT_STATES.REJECTED
            || status === SNAPSHOT_STATES.CANCELLED
            || status === SNAPSHOT_STATES.DISABLED
            || status === SNAPSHOT_STATES.INTERRUPTED
            || status === SNAPSHOT_STATES.AVAILABLE_PARTIALLY)
            snapshotFileSaver.failure(status.text);

        if (SNAPSHOT_STATES.CANCELLING === status)
            snapshotFileSaver.setAborted();
    }

    function execute(msgData, completionListener) {
        return connectionInitializer.connectToBts()
            .prepare()
            .then(() => {
                if (snapshot.isSnapshotPossible(msgData.parameters.force)) {
                    return snapshot.startSnapshot(msgData.parameters.coverage)
                        .then( () => {
                            activeCompletionListener = completionListener;
                            activeStatusListener = snapshot.addStatusListener(statusListener);
                            snapshotFileSaver.setListener(fileSaverListener);
                        });
                }
                else {
                    completionListener(ADMIN_API_STATUSES.FAILED, SNAPSHOT_STATES.REJECTED.text);
                    return Promise.reject(SNAPSHOT_STATES.REJECTED);
                }
            })
            .catch(reason => {
                completionListener(ADMIN_API_STATUSES.FAILED, reason);
                return Promise.reject(reason);
            });
    }
}
