module.exports = {
    process: process,
    processUntilSuccess: processUntilSuccess
};

function process(array, maxProcessingTimeMs, processItemFunction, onDoneFunction, timeoutFunction = setTimeout) {
    var that = array;

    function internal(offset) {
        var i = offset;
        var start = Date.now();
        for (; i < that.length; i++) {
            processItemFunction(that[i]);

            var time = Date.now() - start;
            if (time > maxProcessingTimeMs) {
                timeoutFunction(function () {
                    i++;
                    internal(i);
                });
                break;
            }
        }
        if (onDoneFunction && i === that.length) {
            onDoneFunction();
        }
    }

    internal(0);
}

function processUntilSuccess(array, maxProcessingTimeMs, processItemFunction, onDoneFunction, timeoutFunction = setTimeout) {
    var that = array;

    function internal(offset) {
        var success = false;

        var i = offset;
        var start = Date.now();
        for (; i < that.length && !success; i++) {
            success = processItemFunction(that[i]) === true;

            var time = Date.now() - start;
            if (time > maxProcessingTimeMs && !success) {
                timeoutFunction(function () {
                    i++;
                    internal(i);
                });
                break;
            }
        }
        if (onDoneFunction && (success || i === that.length)) {
            onDoneFunction();
        }
    }

    internal(0);
}
