module.exports = {
    getByName
};

const _ = require('lodash');

const PROFILES = {};
PROFILES.FULL_ACCESS = {
    name: 'full',
    defaultRouteId: 'info-model',
    accessAllRouteIds: true,
    excludedFunctionality: []
};
PROFILES.OPERATOR = {
    name: 'operator',
    defaultRouteId: 'site-runtime',
    excludedRouteIds: [
        'info-model',
        'process-analysis-overview',
        'process-analysis-editor',
        'procedures-editor',
        'timings',
        'kpi-editor'
    ],
    excludedFunctionality: [
        'SHOW_PARAMETERS',
        'SHOW_OPTIF_PARAMETERS',
        'SHOW_PARAMETER_HISTORY',
        'DOWNLOAD_OFFLINE_VERSION',
        'SETTINGS_UPDATE_INTERVAL',
        'OTHER_ABNORMALITIES',
        'SHOW_TIMELINE_DETAILS',
        'SHOW_SITE_PARAMETERS',
        'STATE_INFO_CHART',
        'SOFTWARE_DETAILS',
        'KPI_GRID_DISTNAME'
    ]
};
PROFILES.OFFLINE = {
    name: 'offline',
    defaultRouteId: 'site-runtime',
    excludedRouteIds: PROFILES.OPERATOR.excludedRouteIds,
    excludedFunctionality: PROFILES.OPERATOR.excludedFunctionality.concat(['CONNECT_TO_BTS', 'USER_ACCOUNT'])
};
PROFILES.DEVELOPER = _.assign({}, PROFILES.FULL_ACCESS, { name: 'developer' });
PROFILES.TESTER = _.assign({}, PROFILES.FULL_ACCESS, { name: 'tester' });
PROFILES.READONLY = _.assign({},
    PROFILES.FULL_ACCESS, { excludedFunctionality: ['CONNECT_TO_BTS', 'USER_ACCOUNT', 'DOWNLOAD_OFFLINE_VERSION'] }, { name: 'readonly' }
);

function getByName(profileName) {
    const profile = PROFILES[profileName.toUpperCase()];
    if (profile) return _.cloneDeep(profile);

    return null;
}
