module.exports = {
    newInstance: newInstance
};

function newInstance(capacity) {
    const entries = new Map();

    var size = 0;
    var head = null;
    var tail = null;

    return {
        set: set,
        get: get,
        clear: clear
    };

    function set(key, value) {
        var entry = entries.get(key);
        if (!entry) {
            addNew(key, value);
        } else {
            entry.value = value;

            refresh(entry);
        }
    }

    function get(key, produce) {
        var entry = entries.get(key);

        if (entry)
            refresh(entry);
        else if (produce)
            entry = addNew(key, produce(key));


        return entry ? entry.value : null;
    }

    function clear() {
        size = 0;
        entries.clear();
        head = null;
        tail = null;
    }

    function addNew(key, value) {
        var entry = {
            key: key,
            value: value
        };

        entries.set(key, entry);

        if (!head) {
            head = entry;
            tail = entry;
        } else {
            updateHead(entry);
        }

        size++;

        if (size > capacity)
            removeTail();

        return entry;
    }

    function refresh(entry) {
        if (head === entry) return;

        if (tail === entry)
            tail = entry.newer;

        const older = entry.older;
        const newer = entry.newer;

        if (older)
            older.newer = newer;

        if (newer)
            newer.older = older;

        updateHead(entry);
    }

    function updateHead(entry) {
        head.newer = entry;
        entry.older = head;
        entry.newer = null;
        head = entry;
    }

    function removeTail() {
        entries.delete(tail.key);

        const newTail = tail.newer;
        newTail.older = null;
        tail = newTail;

        size--;
    }
}

