const ProtoByteBuffer = require('protobufjs').ByteBuffer;
const createBtsClient = require('./btsClient');

module.exports = {newInstance};

function newInstance({btsClient = createBtsClient()} = {}) {
    return {
        buildObjectInfo,
        buildImiObjectInfo,
        uuidBytesToHex,
        hexToUuidBytes
    };

    function buildObjectInfo(imiObjectInfo) {
        const imiObjectInfoMsg = btsClient.getSicInstance().objectInfo();
        imiObjectInfoMsg.setProto(imiObjectInfo);

        return {
            uuid: uuidBytesToHex(imiObjectInfoMsg.getObjectId()),
            objectType: imiObjectInfoMsg.getObjectType(),
            instanceId: imiObjectInfoMsg.getInstanceId(),
            parentUuid: uuidBytesToHex(imiObjectInfoMsg.getParentId())
        };
    }

    function buildImiObjectInfo(objectInfo) {
        const sicInstance = btsClient.getSicInstance();
        const imiObjectInfo = sicInstance.objectInfo();
        imiObjectInfo.setObjectId(hexToUuidBytes(objectInfo.uuid));
        imiObjectInfo.setObjectType(objectInfo.objectType);
        imiObjectInfo.setInstanceId(objectInfo.instanceId);
        imiObjectInfo.setParentId(hexToUuidBytes(objectInfo.parentUuid));
        return imiObjectInfo.getProto();
    }

    function uuidBytesToHex(uuidBytes) {
        return uuidBytes && uuidBytes.toHex();
    }

    function hexToUuidBytes(hex) {
        return hex && ProtoByteBuffer.fromHex(hex);
    }
}
