module.exports = connectionService;

const applicationInfo = require('../applicationInfo/applicationInfo');

function connectionService({updateService, signInHandlerService, imAuth, currentSettings}) {
    return {
        toggleConnection: toggleConnection,
        addListener: signInHandlerService.addListener,
        removeListener: signInHandlerService.removeListener,
        isSignedIn: signInHandlerService.isSignedIn
    };

    function toggleConnection() {
        if (signInHandlerService.isSignedIn() || signInHandlerService.isSignInInProgress())
            signInHandlerService.signOut();
        else
            updateService
                .clearHistory()
                .then(() => {
                    signInHandlerService.signIn(getSignInData());
                });
    }

    function getSignInData() {
        const releaseInfo = applicationInfo.getReleaseInfo();
        return {
            userName: imAuth.getUserName(),
            token: imAuth.getToken(),
            authMethodId: imAuth.getAuthMethod().id,
            host: imAuth.getHost(),
            port: imAuth.getPort(),
            isSSL: imAuth.isSecure(),
            adminVersion: releaseInfo.branch,
            clientId: releaseInfo.hash,
            updateInterval: currentSettings.updateInterval()
        };
    }
}
