const protoDecoder = require('./protoDecoder');
const LOG = require('./logger.impl').getLogger('admin.core.decoderService');
const utils = require('./utils/distNameUtils');

module.exports = {
    newInstance: newInstance
};

function newInstance(metaService, objectIdMapper) {
    return decoderService(metaService, protoDecoder, objectIdMapper);
}

function decoderService(metaService, protoDecoder, objectIdMapper) {
    var protoDecoderInstance = protoDecoder(objectIdMapper);

    return {
        decodeByClassName: decodeByClassName,
        decodeByDistName: decodeByDistName,
        getTypeInfo: getTypeInfo
    };

    function decodeByClassName(metaUid, payload, className) {
        var typeInfo = metaService.getTypeInfo(metaUid, className);
        return protoDecoderInstance.decode(typeInfo, payload);
    }

    function decodeByDistName(metaUid, payload, distName) {
        try {
            return decodeByClassName(metaUid, payload, getClassName(distName));
        } catch (e) {
            LOG.error('Failed to decode object "' + utils.getName(distName) + '"');
            throw e;
        }
    }

    function getClassName(distName) {
        var distNameLastPart = distName.split('/').slice(-1)[0];
        return distNameLastPart.substring(0, distNameLastPart.indexOf('-'));
    }

    function getTypeInfo(metaUid, className) {
        return metaService.getTypeInfo(metaUid, className);
    }
}



