module.exports = {
    newEvent: function () {
        var watchers = new Set();
        var firstAddedAction;
        var lastRemovedAction;

        return {
            watch: watch,
            unwatch: unwatch,
            notify: notify,
            onFirstAdded: onFirstAdded,
            onLastRemoved: onLastRemoved,
            getWatchersCount: getWatchersCount
        };

        function unwatch(watcher) {
            if (!watchers.has(watcher)) throw new Error('Watcher was not added: ' + watcher.toString());
            watchers.delete(watcher);
            if (!watchers.size && lastRemovedAction) lastRemovedAction();
        }

        function watch(watcher) {
            watchers.add(watcher);
            if (watchers.size === 1 && firstAddedAction) firstAddedAction();

            return {
                unwatch: function () {
                    unwatch(watcher);
                }
            };
        }

        function notify(event) {
            var promises = [];
            watchers.forEach(function (watcher) {
                promises.push(watcher(event));
            });
            return Promise.all(promises);
        }

        function onFirstAdded(watcher) {
            firstAddedAction = watcher;
        }

        function onLastRemoved(watcher) {
            lastRemovedAction = watcher;
        }

        function getWatchersCount() {
            return watchers.size;
        }
    }
};



