const btsClient = require('../communication/btsClient');
const applicationInfo = require('../applicationInfo/applicationInfo');

const OPTIONAL_PARAMS = {
    receiver: 0,
    sender: 0
};

const SESSION_TYPE_PLAIN = 1;


module.exports = function FileTransferConnection(authService, applicationInfoCustom, sessionType) {

    FileTransferConnection.prototype.connect = connect;
    FileTransferConnection.prototype.disconnect = disconnect;
    FileTransferConnection.prototype.sendMessage = sendMessage;
    FileTransferConnection.prototype.setWebSocketClient = setWebSocketClient;

    const that = this;
    that.authService = authService;
    that.applicationInfo = applicationInfoCustom || applicationInfo;
    that.sessionType = sessionType || SESSION_TYPE_PLAIN;
    that.webSocketClient = btsClient();
    that.connectListener;

    function connect() {
        return new Promise((resolve, reject) => {
            try {
                that.connectListener = {
                    connected: () => {
                        loginUser();
                    },
                    connectionError: () => {
                        removeConnectionListener();
                        reject('Connection error');
                    },
                    loggedIn: () => {
                        removeConnectionListener();
                        resolve();
                    },
                    loginError: () => {
                        disconnect();
                        reject('Login error');
                    }
                };
                that.webSocketClient.addListener(that.connectListener);
                that.webSocketClient.connect(that.authService.getHost(), that.authService.getPort(), that.authService.isSecure());
            } catch (error) {
                reject(error);
            }
        });
    }

    function disconnect() {
        removeConnectionListener();
        that.webSocketClient.disconnect();
    }

    function setWebSocketClient(client) {
        that.webSocketClient = client;
    }

    function removeConnectionListener() {
        that.webSocketClient.removeListener(that.connectListener);
    }

    function loginUser() {
        const releaseInfo = that.applicationInfo.getReleaseInfo();
        that.webSocketClient.login({
            userName: that.authService.getUserName(),
            token: that.authService.getToken(),
            authMethodId: that.authService.getAuthMethod().id,
            sessionType: that.sessionType,
            clientVersion: releaseInfo.branch,
            clientId: releaseInfo.hash
        });
    }

    function sendMessage(msg) {
        that.webSocketClient.send(msg, OPTIONAL_PARAMS);
    }
};
