const sic = require('../sic');
const FileTransferConnection = require('./fileTransferConnection');

const FILE_REQ_MSG = 0x6000;
const SESSION_TYPE_SW_UPDATE = 3;


FileUploadService.prototype = Object.create(FileTransferConnection.prototype);

function FileUploadService(authService, fileDispatcher, applicationInfo) {
    FileUploadService.prototype.startService = startService;
    FileUploadService.prototype.stopService = stopService;
    FileTransferConnection.call(this, authService, applicationInfo, SESSION_TYPE_SW_UPDATE);

    const that = this;
    that.fileDispatcher = fileDispatcher;
    that.sicInstance = sic();
    that.requestWatcher;

    function stopService() {
        removeListeners();
        that.disconnect();
    }

    function startService() {
        return new Promise((resolve, reject) => {
            that.requestWatcher = registerFileRequestListener();
            that.connect()
                .then(resolve)
                .catch(reject);
        });
    }

    function registerFileRequestListener() {
        const FILE_NOT_FOUND_CODE = 1;
        return that.webSocketClient.addMessageListener(FILE_REQ_MSG, requestReceived);

        function requestReceived(message) {
            const requestId = message.getRequestId();
            const uri = removeDuplicatedShashes(decodeURIComponent(message.getUri()));
            const file = that.fileDispatcher.getFile(uri);
            if (!file) {
                const fileErrorMessage = that.sicInstance.imgwFileServiceErrorResp()
                    .setRequestId(requestId)
                    .setErrorCode(FILE_NOT_FOUND_CODE)
                    .setDescription(`File: ${uri} is not present in a given location.`);
                that.sendMessage(fileErrorMessage);
                return;
            }
            const fileContentMessage = that.sicInstance.imgwFileServiceFileContentResp()
                .setRequestId(requestId)
                .setContent(file);
            that.sendMessage(fileContentMessage);
        }

        function removeDuplicatedShashes(uri) {
            const slashes = new RegExp('/+', 'g');
            return uri.replace(slashes, '/');
        }
    }

    function removeListeners() {
        that.requestWatcher.unwatch();
    }
}

module.exports = FileUploadService;
