const IMI_CONSTANTS = require('../imi/imiConstants');
const ProtoBuf = require('protobufjs');
const gatewayProtocol = require('./gateway_protocol');
const gatewayDefinitions = require('./gatewayDefinitions');
const sha1CheckSum = require('../fileTransfer/sha1CheckSumService');
const ByteBuffer = require('bytebuffer');
const _ = require('lodash');


module.exports = {
    createMessages: createMessages,
    createProtocolMessages: createProtocolMessages
};

function encodeProto(proto) {
    const buffer = proto.encode();
    return buffer.toArrayBuffer();
}

function decodeProto(protoMessage, protoBuffer) {
    return protoMessage.decode(protoBuffer);
}

function createProtocolMessages() {
    const protocolMessages = ProtoBuf.loadJson(_.cloneDeep(gatewayProtocol)).build('nokia.im.gateway');

    return {
        imgwGetProtocolVersionReq: imgwGetProtocolVersionReq,
        imgwGetProtocolVersionResp: imgwGetProtocolVersionResp
    };

    function imgwGetProtocolVersionReq() {
        var buffer = new ArrayBuffer(0);
        var theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.GET_PROTOCOL_VERSION_REQ_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwGetProtocolVersionReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwGetProtocolVersionResp() {
        const ProtoMessage = protocolMessages.GetProtocolVersionResp;

        var proto = new ProtoMessage({
            'version': 2
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.GET_PROTOCOL_VERSION_RSP_ID;
        };
        theMsg.getVersion = function () {
            return proto.version;
        };
        theMsg.setVersion = function (value) {
            proto.version = value;
            return theMsg;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwGetProtocolVersionResp { \n id: 0x${theMsg.getId().toString(16)} version: ${theMsg.getVersion()} \n}`;
        };
        return theMsg;
    }
}

function createMessages(gatewayVersion) {
    const theDefinition = gatewayDefinitions.get(gatewayVersion);
    if (!theDefinition) throw new Error(`Invalid gateway version: ${gatewayVersion}`);
    const gatewayMessages = ProtoBuf.loadJson(_.cloneDeep(theDefinition)).build(`nokia.im.gateway.v${gatewayVersion}`);

    return {
        imgwGetMetaReq: imgwGetMetaReq,
        imgwGetMetaResp: imgwGetMetaResp,
        imgwLoginReq: imgwLoginReq,
        imgwLoginSuccessResp: imgwLoginSuccessResp,
        imgwLoginErrorResp: imgwLoginErrorResp,
        imgwFileServiceGetReq: imgwFileServiceGetReq,
        imgwFileServiceFileContentResp: imgwFileServiceFileContentResp,
        imgwFileServiceErrorResp: imgwFileServiceErrorResp,
        imgwDiagSessionSetupReq: imgwDiagSessionSetupReq,
        imgwDiagSessionAcceptResp: imgwDiagSessionAcceptResp,
        imgwDiagSessionRejectResp: imgwDiagSessionRejectResp,
        imgwDiagSessionData: imgwDiagSessionData,
        imgwDiagSessionEnd: imgwDiagSessionEnd,
        imgwDetectEndiannessReq: imgwDetectEndiannessReq,
        imgwDetectEndiannessResp: imgwDetectEndiannessResp,
        imgwGetGatewayInformationReq: imgwGetGatewayInformationReq,
        imgwGetGatewayInformationResp: imgwGetGatewayInformationResp,
        imgwSessionsListReq,
        imgwSessionsListResp,
        imgwSession
    };

    function imgwGetGatewayInformationReq() {
        var buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.GET_GATEWAY_INFORMATION_REQ_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwGetGatewayInformationReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwGetGatewayInformationResp() {
        const ProtoMessage = gatewayMessages.GetGatewayInformationResp;

        var proto = new ProtoMessage({
            'endianness': 1,
            'embeddedAdminVersion': 'default',
            'embeddedAdminId': '1234',
            'imiVersionMajor': 1,
            'imiVersionMinor': 2,
            'imServiceSicad': 3,
            'imProtocolRelease': '3',
            'imProtocolVersionMajor': 3,
            'imProtocolVersionMinor': 2
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.GET_GATEWAY_INFORMATION_RESP_ID;
        };
        theMsg.setEndianness = function (endianness) {
            proto.endianness = endianness;
            return theMsg;
        };
        theMsg.getEndianness = function () {
            return proto.endianness;
        };
        theMsg.setEmbeddedAdminVersion = function (embeddedAdminVersion) {
            proto.embeddedAdminVersion = embeddedAdminVersion;
            return theMsg;
        };
        theMsg.getEmbeddedAdminVersion = function () {
            return proto.embeddedAdminVersion;
        };
        theMsg.setEmbeddedAdminId = function (embeddedAdminId) {
            proto.embeddedAdminId = embeddedAdminId;
            return theMsg;
        };
        theMsg.getEmbeddedAdminId = function () {
            return proto.embeddedAdminId;
        };
        theMsg.setImiVersionMajor = function (imiVersionMajor) {
            proto.imiVersionMajor = imiVersionMajor;
            return theMsg;
        };
        theMsg.getImiVersionMajor = function () {
            return proto.imiVersionMajor;
        };
        theMsg.setImiVersionMinor = function (imiVersionMinor) {
            proto.imiVersionMinor = imiVersionMinor;
            return theMsg;
        };
        theMsg.getImiVersionMinor = function () {
            return proto.imiVersionMinor;
        };
        theMsg.setInfoModelSicAddress = function (imServiceSicad) {
            proto.imServiceSicad = imServiceSicad;
            return theMsg;
        };
        theMsg.getInfoModelSicAddress = function () {
            return proto.imServiceSicad;
        };
        theMsg.setImProtocolRelease = function (imProtocolRelease) {
            proto.imProtocolRelease = imProtocolRelease;
            return theMsg;
        };
        theMsg.getImProtocolRelease = function () {
            return proto.imProtocolRelease;
        };
        theMsg.setImProtocolVersionMajor = function (imProtocolVersionMajor) {
            proto.imProtocolVersionMajor = imProtocolVersionMajor;
            return theMsg;
        };
        theMsg.getImProtocolVersionMajor = function () {
            return proto.imProtocolVersionMajor;
        };
        theMsg.setImProtocolVersionMinor = function (imProtocolVersionMinor) {
            proto.imProtocolVersionMinor = imProtocolVersionMinor;
            return theMsg;
        };
        theMsg.getImProtocolVersionMinor = function () {
            return proto.imProtocolVersionMinor;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwGetGatewayInformationResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwGetMetaReq() {
        var buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.GET_META_REQ_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwGetMetaReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwGetMetaResp() {
        const ProtoMessage = gatewayMessages.FileServiceFileContentResponse;

        var proto = new ProtoMessage({
            'requestId': 0,
            'content': new ArrayBuffer(0),
            'hash': new ArrayBuffer(0)
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.GET_META_RSP_ID;
        };
        theMsg.setRequestId = function (requestId) {
            proto.requestId = requestId;
            return theMsg;
        };
        theMsg.getRequestId = function () {
            return proto.requestId;
        };
        theMsg.setMeta = function (content) {
            const contentUint8Array = new Uint8Array(content);
            proto.content = ByteBuffer.wrap(contentUint8Array);
            const checkSum = sha1CheckSum.calculateCheckSum(contentUint8Array);
            setHashFromUint8Array(checkSum);
            return theMsg;
        };
        theMsg.getMeta = function () {
            const content = new Uint8Array(proto.content.toArrayBuffer());
            if (!sha1CheckSum.isCheckSumCorrect(content, this.getHash())) {
                throw 'Meta checkSum incorrect. Meta wasn\'t loaded.';
            }
            return content;
        };
        theMsg.setHash = function (hash) {
            setHashFromUint8Array(new Uint8Array(hash));
            return theMsg;
        };
        theMsg.getHash = function () {
            return new Uint8Array(proto.hash.toArrayBuffer());
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwGetMetaResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        function setHashFromUint8Array(uint8Array) {
            proto.hash = ByteBuffer.wrap(uint8Array);
        }

        return theMsg;
    }

    function imgwLoginReq() {
        const ProtoMessage = gatewayMessages.LoginRequest;

        var proto = new ProtoMessage({
            'login': '',
            'authMethod': 1,
            'token': '',
            'sessionType': 2,
            'clientVersion': 'default',
            'clientId': '1234',
            'updateInterval': 5
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.LOGIN_REQ_ID;
        };
        theMsg.setLogin = function (login) {
            proto.login = login;
            return theMsg;
        };
        theMsg.getLogin = function () {
            return proto.login;
        };
        theMsg.setAuthMethod = function (authMethod) {
            proto.authMethod = authMethod;
            return theMsg;
        };
        theMsg.getAuthMethod = function () {
            return proto.authMethod;
        };
        theMsg.setToken = function (token) {
            proto.token = token;
            return theMsg;
        };
        theMsg.getToken = function () {
            return proto.token;
        };
        theMsg.setSessionType = function (sessionType) {
            proto.sessionType = sessionType;
            return theMsg;
        };
        theMsg.getSessionType = function () {
            return proto.sessionType;
        };
        theMsg.setClientVersion = function (clientVersion) {
            proto.clientVersion = clientVersion;
            return theMsg;
        };
        theMsg.getClientVersion = function () {
            return proto.clientVersion;
        };
        theMsg.setClientId = function (clientId) {
            proto.clientId = clientId;
            return theMsg;
        };
        theMsg.getClientId = function () {
            return proto.clientId;
        };
        theMsg.setUpdateInterval = function (updateInterval) {
            proto.updateInterval = updateInterval;
            return theMsg;
        };
        theMsg.getUpdateInterval = function () {
            return proto.updateInterval;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwLoginReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwLoginSuccessResp() {
        const ProtoMessage = gatewayMessages.LoginSuccessResponse;

        var proto = new ProtoMessage({
            'profile': '',
            'username': '',
            'failedLoginNo': 0,
            'lastSuccess': ''
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.LOGIN_SUCCESS_RSP_ID;
        };
        theMsg.setProfile = function (profile) {
            proto.profile = profile;
            return theMsg;
        };
        theMsg.getProfile = function () {
            return proto.profile;
        };
        theMsg.setUserName = function (userName) {
            proto.username = userName;
            return theMsg;
        };
        theMsg.getUserName = function () {
            return proto.username;
        };
        theMsg.setFailedLoginNo = function (failedLoginNo) {
            proto.failedLoginNo = failedLoginNo;
            return theMsg;
        };
        theMsg.getFailedLoginNo = function () {
            return proto.failedLoginNo;
        };
        theMsg.setLastSuccess = function (lastSuccess) {
            proto.lastSuccess = lastSuccess;
            return theMsg;
        };
        theMsg.getLastSuccess = function () {
            return proto.lastSuccess;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwLoginSuccessResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwLoginErrorResp() {
        const ProtoMessage = gatewayMessages.LoginErrorResponse;

        var proto = new ProtoMessage({
            'errorCode': 0,
            'description': ''
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.LOGIN_ERROR_RSP_ID;
        };
        theMsg.setErrorCode = function (errorCode) {
            proto.errorCode = errorCode;
            return theMsg;
        };
        theMsg.getErrorCode = function () {
            return proto.errorCode;
        };
        theMsg.setDescription = function (description) {
            proto.description = description;
            return theMsg;
        };
        theMsg.getDescription = function () {
            return proto.description;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwLoginErrorResp { \n id: 0x${theMsg.getId().toString(16)} '\n}`;
        };
        return theMsg;
    }

    function imgwFileServiceGetReq() {
        const ProtoMessage = gatewayMessages.FileServiceGetRequest;

        var proto = new ProtoMessage({
            'requestId': 0,
            'uri': ''
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.FILE_SERVICE_GET_REQ_ID;
        };
        theMsg.setRequestId = function (requestId) {
            proto.requestId = requestId;
            return theMsg;
        };
        theMsg.getRequestId = function () {
            return proto.requestId;
        };
        theMsg.setUri = function (uri) {
            proto.uri = uri;
            return theMsg;
        };
        theMsg.getUri = function () {
            return proto.uri;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwFileServiceGetReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwFileServiceFileContentResp() {
        const ProtoMessage = gatewayMessages.FileServiceFileContentResponse;

        var proto = new ProtoMessage({
            'requestId': 0,
            'content': new ArrayBuffer(0),
            'hash': new ArrayBuffer(0)
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.FILE_SERVICE_FILE_CONTENT_RSP_ID;
        };
        theMsg.setRequestId = function (requestId) {
            proto.requestId = requestId;
            return theMsg;
        };
        theMsg.getRequestId = function () {
            return proto.requestId;
        };
        theMsg.setContent = function (content) {
            const contentUint8Array = new Uint8Array(content);
            proto.content = ByteBuffer.wrap(contentUint8Array);
            const checkSum = sha1CheckSum.calculateCheckSum(contentUint8Array);
            setHashFromUint8Array(checkSum);
            return theMsg;
        };
        theMsg.getContent = function () {
            const content = new Uint8Array(proto.content.toArrayBuffer());
            if (!sha1CheckSum.isCheckSumCorrect(content, this.getHash())) {
                throw 'Downloaded file checkSum is incorrect.';
            }
            return content;
        };
        theMsg.setHash = function (hash) {
            setHashFromUint8Array(new Uint8Array(hash));
            return theMsg;
        };
        theMsg.getHash = function () {
            return new Uint8Array(proto.hash.toArrayBuffer());
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwFileServiceFileContentResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        function setHashFromUint8Array(uint8Array) {
            proto.hash = ByteBuffer.wrap(uint8Array);
        }

        return theMsg;
    }

    function imgwFileServiceErrorResp() {
        const ProtoMessage = gatewayMessages.FileServiceErrorResponse;

        var proto = new ProtoMessage({
            'requestId': 0,
            'errorCode': 1,
            'description': 'description'
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.FILE_SERVICE_ERROR_RSP_ID;
        };
        theMsg.setRequestId = function (requestId) {
            proto.requestId = requestId;
            return theMsg;
        };
        theMsg.getRequestId = function () {
            return proto.requestId;
        };
        theMsg.setErrorCode = function (errorCode) {
            proto.errorCode = errorCode;
            return theMsg;
        };
        theMsg.getErrorCode = function () {
            return proto.errorCode;
        };
        theMsg.setDescription = function (description) {
            proto.description = description;
            return theMsg;
        };
        theMsg.getDescription = function () {
            return proto.description;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwFileServiceErrorResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwSwServerStarted() {
        var buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.SW_SERVER_STARTED_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwSwServerStarted { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDiagSessionSetupReq() {
        const ProtoMessage = gatewayMessages.DiagnosticSessionSetupRequest;

        var proto = new ProtoMessage({
            'command': 1,
            'parameters': ''
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DIAG_SESSION_SETUP_REQ_ID;
        };
        theMsg.getCommand = function () {
            return proto.command;
        };
        theMsg.setCommand = function (command) {
            proto.command = command;
            return theMsg;
        };
        theMsg.getParameters = function () {
            return proto.parameters;
        };
        theMsg.setParameters = function (parameters) {
            proto.parameters = parameters;
            return theMsg;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDiagSessionSetupReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDiagSessionAcceptResp() {
        var buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DIAG_SESSION_ACCEPT_RSP_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDiagSessionAcceptResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDiagSessionRejectResp() {
        const ProtoMessage = gatewayMessages.DiagnosticSessionRejectResponse;

        var proto = new ProtoMessage({
            'reason': ''
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DIAG_SESSION_REJECT_RSP_ID;
        };
        theMsg.getReason = function () {
            return proto.reason;
        };
        theMsg.setReason = function (reason) {
            proto.reason = reason;
            return theMsg;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDiagSessionRejectResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDiagSessionData() {
        const ProtoMessage = gatewayMessages.DiagnosticSessionData;

        var proto = new ProtoMessage({
            'line': ''
        });

        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DIAG_SESSION_DATA_ID;
        };
        theMsg.getLine = function () {
            return proto.line;
        };
        theMsg.setLine = function (line) {
            proto.line = line;
            return theMsg;
        };
        theMsg.getBuffer = function () {
            return encodeProto(proto);
        };
        theMsg.setBuffer = function (value) {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDiagSessionData { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDiagSessionEnd() {
        var buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DIAG_SESSION_END_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDiagSessionEnd { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDetectEndiannessReq() {
        var buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DETECT_ENDIANNESS_REQ_ID;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (data) {
            buffer = data;
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDetectEndiannessReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        };
        return theMsg;
    }

    function imgwDetectEndiannessResp() {
        const theMsg = {};
        var buffer = new ArrayBuffer(0), endianness;

        theMsg.getId = function () {
            return IMI_CONSTANTS.IMGW.DETECT_ENDIANNESS_RESP_ID;
        };
        theMsg.getEndianness = function () {
            return endianness;
        };
        theMsg.setEndianness = function (value) {
            endianness = value;
            return theMsg;
        };
        theMsg.getBuffer = function () {
            return buffer;
        };
        theMsg.setBuffer = function (value) {
            buffer = value;
            endianness = getEndiannessFlag(buffer);
            return theMsg;
        };
        theMsg.toString = function () {
            return `imgwDetectEndiannessResp { \n id: 0x${theMsg.getId().toString(16)}, endianness: ${theMsg.getEndianness()} \n}`;
        };

        return theMsg;

        function getEndiannessFlag(buffer) {
            var view = new Uint8Array(buffer);
            return view[view.length - 1];
        }
    }

    function imgwSessionsListReq() {
        let buffer = new ArrayBuffer(0);
        const theMsg = {};
        theMsg.getId = () => IMI_CONSTANTS.IMGW.SESSIONS_LIST_REQ;
        theMsg.getBuffer = () => buffer;
        theMsg.setBuffer = (value) => {
            buffer = value;
            return theMsg;
        };
        theMsg.toString = () => `imgwSessionsListReq { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        return theMsg;
    }

    function imgwSessionsListResp() {
        const ProtoMessage = gatewayMessages.SessionsListResp;
        let proto = new ProtoMessage({sessions: []});
        const theMsg = {};
        theMsg.getId = () => IMI_CONSTANTS.IMGW.SESSIONS_LIST_RESP;
        theMsg.getSessions = () => proto.sessions || [];
        theMsg.setSessions = (value) => {
            proto.sessions = value;
            return theMsg;
        };
        theMsg.getBuffer = () => encodeProto(proto);
        theMsg.setBuffer = (value) => {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = () => `imgwSessionsListResp { \n id: 0x${theMsg.getId().toString(16)} \n}`;
        return theMsg;
    }

    function imgwSession() {
        const ProtoMessage = gatewayMessages.Session;
        let proto = new ProtoMessage({});
        const theMsg = {};
        theMsg.getUserLogin = () => proto.userLogin;
        theMsg.setUserLogin = (value) => {
            proto.userLogin = value;
        };
        theMsg.getProfileName = () => proto.profileName;
        theMsg.setProfileName = (value) => {
            proto.profileName = value;
        };
        theMsg.getClientVersion = () => proto.clientVersion;
        theMsg.setClientVersion = (value) => {
            proto.clientVersion = value;
        };
        theMsg.getConnectionType = () => proto.connectionType;
        theMsg.setConnectionType = (value) => {
            proto.connectionType = value;
        };
        theMsg.getSessionType = () => proto.sessionType;
        theMsg.setSessionType = (value) => {
            proto.sessionType = value;
        };
        theMsg.getAge = () => proto.age;
        theMsg.setAge = (value) => {
            proto.age = value;
        };
        theMsg.getIp = () => proto.ip;
        theMsg.setIp = (value) => {
            proto.ip = value;
        };
        theMsg.getSessionId = () => proto.sessionId;
        theMsg.setSessionId = (value) => {
            proto.sessionId = value;
        };
        theMsg.getBuffer = () => encodeProto(proto);
        theMsg.setBuffer = (value) => {
            proto = decodeProto(ProtoMessage, value);
            return theMsg;
        };
        theMsg.toString = () => `imgwSession
{
 userLogin: ${theMsg.getUserLogin()},
 profileName: ${theMsg.getProfileName()},
 clientVersion: ${theMsg.getClientVersion()},
 connectionType: ${theMsg.getConnectionType()},
 sessionType: ${theMsg.getSessionType()},
 age: ${theMsg.getAge()},
 ip: ${theMsg.getIp()},
 sessionId: ${theMsg.getSessionId()}
}`;
        return theMsg;
    }
}

