const _ = require('lodash');

module.exports = {
    createChangeEntry: createChangeEntry,
    createEmptyUpdate: createEmptyUpdate,
    createResetEntry: createResetEntry,

    assertIsEmptyChangeEntry: assertIsEmptyChangeEntry,
    assertContainsUpdate: assertContainsUpdate,
    assertContainsOnly: assertContainsOnly,
    assertContainsOnlyUpdates: assertContainsOnlyUpdates
};

function createChangeEntry(id, updatedNames, removedNames) {
    if (!Array.isArray(updatedNames))
        return createChangeEntry(id, [updatedNames]);

    const entry = {
        id: id,
        metaUid: '1',
        timestamp: Date.now(),
        numOfChanges: 0,
        change: {
            removed: removedNames || [],
            updated: []
        }
    };
    _.forEach(updatedNames, function (distName) {
        entry.change.updated.push(createEmptyUpdate(distName));
    });
    entry.numOfChanges = updatedNames.length;
    return entry;
}

function createEmptyUpdate(distName) {
    return {
        distName: distName,
        payload: new ArrayBuffer(0)
    };
}

function createResetEntry(id, timestamp) {
    const entry = {
        id: id,
        reset: true
    };
    if (timestamp)
        entry.timestamp = timestamp;

    return entry;
}

function assertIsEmptyChangeEntry(entry) {
    assert.equal(entry.numOfChanges, 0);
    assert.lengthOf(entry.change.updated, 0);
    assert.lengthOf(entry.change.removed, 0);
}

function assertContainsUpdate(entry, expectedDistName) {
    assert.isTrue(!!_.find(entry.change.updated, function (update) {
        return update.distName === expectedDistName;
    }));
}

function assertContainsOnly(entry, expected) {
    var updates = expected.updates;
    var removes = expected.removes;

    assertContainsOnlyUpdates(entry, updates);
    assertContainsOnlyRemoves(entry, removes);

    if (updates && removes) {
        assert.equal(entry.numOfChanges, updates.length + removes.length);
    }
}

function assertContainsOnlyUpdates(entry, distNameUpdates) {
    if (!distNameUpdates) return;

    assert.lengthOf(entry.change.updated, distNameUpdates.length);
    assert.isTrue(_.all(entry.change.updated, function (update) {
        return _.contains(distNameUpdates, update.distName);
    }));
}

function assertContainsOnlyRemoves(entry, distNameRemoves) {
    if (!distNameRemoves) return;

    assert.lengthOf(entry.change.removed, distNameRemoves.length);
    assert.deepEqual(distNameRemoves, entry.change.removed);
}
