module.exports = ObjectListenersContainer;


ObjectListenersContainer.prototype.addListener = function (listener) {
    this.listeners.push(listener);
    if (shouldTrackParameters(listener))
        this.metadata.numberOfParametersListeners++;
    if (shouldTrackChildren(listener))
        this.metadata.numberOfChildListeners++;
};

ObjectListenersContainer.prototype.removeListener = function (listenerToRemove) {
    _.remove(this.listeners, listenerToRemove);
    if (shouldTrackParameters(listenerToRemove))
        this.metadata.numberOfParametersListeners--;
    if (shouldTrackChildren(listenerToRemove))
        this.metadata.numberOfChildListeners--;
};

ObjectListenersContainer.prototype.hasParametersListeners = function () {
    return this.metadata.numberOfParametersListeners > 0;
};

ObjectListenersContainer.prototype.hasChildListeners = function () {
    return this.metadata.numberOfChildListeners > 0;
};

ObjectListenersContainer.prototype.getListeners = function () {
    return this.listeners;
};

ObjectListenersContainer.prototype.hasExistedObjects = function () {
    return this.metadata.existedObjects.size;
};

ObjectListenersContainer.prototype.addExistedObject = function (distName, parameters) {
    parameters = parameters || {};
    this.metadata.existedObjects.set(distName, parameters)
};

ObjectListenersContainer.prototype.removeExistedObject = function (distName) {
    this.metadata.existedObjects.delete(distName);
};

ObjectListenersContainer.prototype.getExistedDistNames = function () {
    var distNames = [];
    this.metadata.existedObjects.forEach(function (parameters, distName) {
        distNames.push(distName);
    });
    return distNames;
};

ObjectListenersContainer.prototype.getExistedObjects = function () {
    return this.metadata.existedObjects;
};

ObjectListenersContainer.prototype.clearObjects = function () {
    return this.metadata.existedObjects.clear();
};

ObjectListenersContainer.prototype.children = function () {
    if (!arguments.length) return this.metadata.children;
    this.metadata.children = arguments[0];
};

function ObjectListenersContainer() {
    this.listeners = [];
    this.metadata = {
        numberOfParametersListeners: 0,
        numberOfChildListeners: 0,
        children: [],
        existedObjects: new Map()
    };
}

function shouldTrackParameters(listener) {
    return typeof listener.callbacks.onParameterChanged !== 'undefined' || typeof listener.callbacks.onParametersChanged !== 'undefined';
}

function shouldTrackChildren(listener) {
    return typeof listener.callbacks.onChildAdded !== 'undefined';
}
