const events = require('../../events');
module.exports = pmCountersTracker;

function pmCountersTracker({
    pmSnapshot,
    imTracker,
    onlineImTree
}) {
    const updateEvent = events.newEvent();
    const PERFMON_FILE_CLASSNAME = 'PERFMON_FILE_L';
    let pmCounters = {
        values: {},
        startTimes: []
    };
    let pmLocations = [];
    const listener = {
        onListenerRegistered: onListenerRegistered,
        onObjectAppeared: onObjectAppeared
    };

    imTracker.trackByClassName(PERFMON_FILE_CLASSNAME, listener);

    return {
        watch: updateEvent.watch,
        // for tests only
        downloadPmSnapshot
    };

    function downloadPmSnapshot() {
        return pmSnapshot.downloadCompressed(pmLocations)
            .then( (response) => {
                pmCounters =  pmSnapshot.formatPmCounters(response, pmCounters);
                updateEvent.notify(pmCounters);
            }).then(() => {
                pmLocations = [];
            });
    }

    function onListenerRegistered(event) {
        event.objects.forEach( (obj) => {
            if (obj.parameters.uri)
                pmLocations.push(obj.parameters.uri);
        });
        if (pmLocations)
            downloadPmSnapshot();
    }

    function onObjectAppeared(event) {
        onlineImTree.getImTree()
            .then((imTree) => {
                event.distNames.forEach( (distName) => {
                    const imTreeNode = imTree.findByDistName(distName);
                    const filePath = imTreeNode.getDecodedParameters().uri;
                    if (filePath)
                        pmLocations.push(filePath);
                });
                return pmLocations;
            }).then((pmLocationsArr) => {
                if (pmLocationsArr)
                    downloadPmSnapshot();
            });
    }
}
