const xml2json = require('./xml2json');
const loggers = require('./logger.impl');
const unzipper = require('./unzipper');
const arrays = require('./arrays');
const DOMParser = require('xmldom').DOMParser;

const log = loggers.getLogger('admin.core.metaReader');

module.exports = {
    read: read,
    versions: Object.freeze({
        V1: 'pdmeta',
        V2: 'infoModel'
    })
};

const MAX_PROCESS_TIME_MS = 50;

function read(buffer, callback) {
    const converter = createXml2JsonConverter();
    const instance = unzipper.newInstance(buffer);
    const onEachFile = (filename, fileContent, index, total)=> {
        log.debug('processing file[' + index + '/' + total + ']: ' + filename);
        if (fileContent.length > 0) {
            var xmlContent = new DOMParser().parseFromString(fileContent, 'text/xml');
            const metaVersion = xmlContent.firstChild.nodeName;
            const xmlNodes = xmlContent.getElementsByTagName('managedObject');
            converter.addXmlNodes(xmlNodes, metaVersion);
        }
    };
    const onDone = ()=> {
        converter.toJson(callback);
    };
    instance.unzipAll(onEachFile, onDone);
}

var createXml2JsonConverter = function () {
    var entries = [];

    function addXmlNodes(nodes, metaVersion) {
        entries.push({
            metaVersion: metaVersion,
            nodes: nodes
        });
    }

    function toJson(callback) {
        var convertedEntries = [];

        var entry2Json = function (entry) {
            var objects = [];
            for (var i = 0; i < (entry.nodes || []).length; i++) {
                var jsonObject = xml2json.toJson(entry.nodes[i]);
                objects.push(jsonObject[1]);
            }

            convertedEntries.push({
                metaVersion: entry.metaVersion,
                objects: objects
            })
        };

        var onComplete = function () {
            callback(convertedEntries);
        };

        arrays.process(entries, MAX_PROCESS_TIME_MS, entry2Json, onComplete);
    }

    return {
        addXmlNodes: addXmlNodes,
        toJson: toJson
    };
};
