module.exports = {
    filterObject: filterObject
};

const _ = require('lodash');

/**
 * Filter object comparing keys with a pattern.
 * @param {object} object - Input object.
 * @param {object} pattern - Regexp pattern.
 * @param {number} depth - Traversing level (-1 as infinite).
 * @returns {object} - Filtered object.
 */
function filterObject(object, pattern = new RegExp(''), depth = -1) {
    return _.transform(object, (result, value, key) => {
        let isMatched = pattern.test(key);
        const isNested = _.isPlainObject(value) && (depth > 0 || depth === -1);
        if (isNested) {
            value = filterObject(value, pattern, depth === -1 ? -1 : depth - 1);
            isMatched = !_.isEmpty(value);
        }
        if (isMatched) result[key] = value;
    }, {});
}
