module.exports = btsServiceProcedure;

const _ = require('lodash');

function btsServiceProcedure(imTracker, procedureIdFactory) {
    const BTS_SERVICE_PROCEDURE_CLASSNAME = 'BTS_SERVICE_PROCEDURE',
        imListener = {
            onParametersChanged: checkObjects,
            onListenerRegistered: checkObjects,
            onObjectRemoved: onObjectRemoved
        }, newClientId = procedureIdFactory.getClientId(),
        clientIdPredicate = createClientIdPredicate(newClientId),
        listeners = [],
        objectsMap = new Map();
    var promise = Promise.resolve();

    return {
        registerListener: registerListener,
        removeListener: removeListener
    };

    function registerListener(listener) {
        promise = promise
            .then(startTracking)
            .then(addListener);

        return promise;

        function addListener() {
            listeners.push(listener);

            if (objectsMap.size)
                listener.update({
                    eventType: 'onParametersChanged',
                    objects: createObjects()
                });
        }

        function startTracking() {
            if (_.isEmpty(listeners))
                return imTracker.trackByClassName(BTS_SERVICE_PROCEDURE_CLASSNAME, imListener);
            return Promise.resolve();
        }

        function createObjects() {
            const objects = [];
            objectsMap.forEach(function (value, key) {
                objects.push({distName: key, parameters: value});
            });
            return objects;
        }
    }

    function removeListener(listener) {
        promise.then(function () {
            _.remove(listeners, function (item) {
                return item === listener;
            });
            if (!listeners.length)
                imTracker.stopTracking(imListener, BTS_SERVICE_PROCEDURE_CLASSNAME);
        });
    }

    function checkObjects(event) {
        event.objects = _.filter(event.objects, clientIdPredicate);
        _.forEach(event.objects, function (object) {
            objectsMap.set(object.distName, object.parameters);
        });
        if (!_.isEmpty(event.objects))
            notifyListeners(event);
    }

    function onObjectRemoved(event) {
        _.forEach(event.distNames, function (distName) {
            objectsMap.delete(distName);
        });
    }

    function notifyListeners(event) {
        _.forEach(listeners, function (listener) {
            listener.update(event);
        });
    }

    function createClientIdPredicate(clientId) {
        return function (object) {
            var parameters = object.parameters;
            return !!parameters && !!parameters.clientId && (procedureIdFactory.decodeClientId(parameters.clientId) === clientId);
        };
    }
}
