module.exports = ChangeBlockingStateReqTracker;
const loggers = require('../logger.impl');
const imTrackerFunction = require('../imTracker/imTracker');
const ChangeBlockingStateReq = require('./changeBlockingStateReq');
const LOG = loggers.getLogger('admin.core.procedures.changeBlockingStateReqTracker');

const CHANGE_BLOCKING_STATE_REQ_CLASS_NAME = 'CHANGE_BLOCKING_STATE_REQ';

ChangeBlockingStateReqTracker.prototype.isRequestExecuting = isRequestExecuting;

function ChangeBlockingStateReqTracker({
    imTracker = imTrackerFunction()
    }) {
    const that = this;
    that.requestObjects = [];
    that.listener = {
        onListenerRegistered: onListenerRegistered,
        onObjectRemoved: onObjectRemoved,
        onParameterChanged: onParameterChanged
    };
    imTracker.trackByClassName(CHANGE_BLOCKING_STATE_REQ_CLASS_NAME, this.listener, ['objectDN', 'operationStatus', 'operationType']);

    function onListenerRegistered(event) {
        const objects = event.objects;
        if (!objects) return;

        objects.forEach(object => that.requestObjects.push(new ChangeBlockingStateReq(object.distName, object.parameters)));
    }

    function onObjectRemoved(event) {
        const distNames = event.distNames;
        if (!distNames) return;

        distNames.forEach(distName => {
            const objectIndex = that.requestObjects.findIndex(object => object.hasDistName(distName));
            if (objectIndex > -1)
                that.requestObjects.splice(objectIndex, 1);
        });
    }

    function onParameterChanged(event) {
        const objects = event.objects;
        if (!objects) return;

        objects.forEach(object => {
                const parameters = object.parameters;
                const newRequest = new ChangeBlockingStateReq(object.distName, parameters);
                const existingObject = that.requestObjects.find(request => request.isEqual(newRequest));

                if (existingObject) {
                    existingObject.setOperationStatus(parameters.operationStatus);
                } else {
                    that.requestObjects.push(newRequest);
                }
            }
        );
    }
}

function isRequestExecuting(operationType, objectDN) {
    const object = this.requestObjects.find(object => object.isForOperation(operationType, objectDN));
    return !!object;
}
