const Promise = require('bluebird');
const FileUploadService = require('../../fileTransfer/fileUploadService');
const btsServiceDefaultListenerFactory = require('../btsServiceDefaultListener');

module.exports = {
    newInstance
};

function newInstance(procedureExecutor, imAuth, applicationInfo, btsServiceProcedure) {
    return {
        executeDownloadPlan,
        executeActivatePlan,
        executeCommissioning
    };

    function createFileName(baseStationId, planId) {
        return `CDNL_BTSA_${baseStationId}_${planId}.xml`;
    }

    function executeCommissioning(parameters) {
        return executeDownloadPlan(parameters)
            .then(executeActivatePlan(parameters));
    }

    function executeActivatePlan(parameters) {
        const clientId = parameters.clientId;

        return procedureExecutor.executeActivatePlan(parameters)
            .then(() => createTrackProcedurePromise(clientId))
            .then(Promise.resolve)
            .catch(Promise.reject);
    }

    function executeDownloadPlan(parameters) {
        const planId = parameters.planId;
        const clientId = parameters.clientId;
        const btsId = parameters.btsId;

        const uri = `http://localhost:12344/${createFileName(btsId, planId)}`;
        const service = new FileUploadService(imAuth, {
            getFile: () => parameters.scf
        }, applicationInfo);

        return service.startService()
            .then(() => procedureExecutor.executeDownloadPlan({uri, planId, clientId}))
            .then(() => createTrackProcedurePromise(clientId))
            .then(service.stopService)
            .catch((error) => {
                service.stopService();
                return Promise.reject(error);
            });
    }

    function createTrackProcedurePromise(clientId) {
        return new Promise(trackProcedure);

        function trackProcedure(resolve, reject) {
            const procedureListener = btsServiceDefaultListenerFactory.newInstance(clientId, onSuccess, onFail);
            btsServiceProcedure.registerListener(procedureListener);

            function onSuccess() {
                btsServiceProcedure.removeListener(procedureListener);
                resolve();
            }

            function onFail(status) {
                btsServiceProcedure.removeListener(procedureListener);
                reject(status);
            }
        }
    }
}
