module.exports = procedureExecutionCleaner;

const _ = require('lodash');
const loggers = require('../logger.impl.js');
const PROCEDURE_STATUS = require('./btsServiceProcedureStatus');

function procedureExecutionCleaner({btsServiceProcedure, operationsService}) {
    const LOG = loggers.getLogger('imc.view.procedures.executionCleaner');
    const OPERATION_NAME = ['oper_delete', 'delete'];

    return {
        enable: enable,
        //visible for testing
        update: update
    };

    function enable() {
        btsServiceProcedure.registerListener({
            update: update
        });
    }

    function update(event) {
        _.forEach(event.objects, deleteObjectsIfNeeded);

        function deleteObjectsIfNeeded(object) {
            if (!PROCEDURE_STATUS.isFinal(object.parameters.status)) return;

            const btsServiceProcedureDistName = object.distName;
            operationsService.executeOperation(OPERATION_NAME, btsServiceProcedureDistName)
                .then(() => {
                    LOG.info(`Sending "delete" operation on ${btsServiceProcedureDistName} succeeded!`);
                })
                .catch((error) => {
                    LOG.warn(`Sending "delete" operation on ${btsServiceProcedureDistName} failed! ${(error && error.toString())}`);
                });
        }
    }
}
