module.exports = procedureIdFactory;

function procedureIdFactory(localStorage) {

    const PROCEDURE_ID_BYTES = Math.pow(2, 6);
    const CLIENT_ID_BYTES = Math.pow(2, 18);
    const MAX_PROCEDURE_ID = PROCEDURE_ID_BYTES - 1;
    const CLIENT_ID = restoreClientId();

    return {
        getId: getId,
        decodeClientId: decodeClientId,
        decodeProcedureId: decodeProcedureId,
        getClientId: getClientId,
        //visible for tests
        encodeId: encodeId
    };

    function restoreClientId() {
        if (!localStorage.clientId) {
            localStorage.clientId = createClientId();
        }
        return localStorage.clientId;

        function createClientId() {
            return Math.floor(Math.random() * CLIENT_ID_BYTES) + CLIENT_ID_BYTES;
        }
    }

    function getId() {
        const procedureId = getProcedureId();
        return {
            id: encodeId(CLIENT_ID, procedureId),
            clientId: CLIENT_ID,
            procedureId: procedureId
        };

        function getProcedureId() {
            localStorage.procedureId = shouldProcedureIdBeReset() ? 0 : localStorage.procedureId + 1;
            return localStorage.procedureId;
        }

        function shouldProcedureIdBeReset() {
            const procedureId = localStorage.procedureId;
            return _.isNull(procedureId) || _.isUndefined(procedureId) || localStorage.procedureId >= MAX_PROCEDURE_ID;
        }
    }

    function decodeClientId(id) {
        return Math.floor(id / PROCEDURE_ID_BYTES);
    }

    function decodeProcedureId(id) {
        return id % PROCEDURE_ID_BYTES;
    }

    function getClientId() {
        return CLIENT_ID;
    }

    function encodeId(clientId, procedureId) {
        return clientId * PROCEDURE_ID_BYTES + procedureId;
    }
}

