module.exports = {newInstance};

const loggers = require('../logger.impl');
const imTrackerFunction = require('../imTracker/imTracker');
const ResetRequest = require('./resetRequest');

const LOG = loggers.getLogger('admin.core.procedures.resetRequestTracker');

function newInstance({imTracker = imTrackerFunction()} = {}) {
    const requestObjects = [];
    const listener = {
        onListenerRegistered: onListenerRegistered,
        onObjectRemoved: onObjectRemoved,
        onParameterChanged: onParameterChanged
    };
    const filterLnbtsPredicate = object => object.parameters.objectDN === ResetRequest.SITE_DIST_NAME;
    imTracker.trackByClassName(ResetRequest.RESET_REQUEST_CLASS_NAME, listener, ResetRequest.REQUIRED_PARAMETERS);

    return {isResetExecuting};

    function isResetExecuting() {
        return requestObjects.length > 0;
    }

    function onListenerRegistered(event) {
        const objects = event.objects || [];
        objects
            .filter(filterLnbtsPredicate)
            .forEach(object => requestObjects.push(ResetRequest.newInstance(object.distName, object.parameters)));
    }

    function onObjectRemoved(event) {
        const distNames = event.distNames || [];
        distNames.forEach(distName => {
            const objectIndex = requestObjects.findIndex(request => request.hasDistName(distName));
            if (objectIndex >= 0)
                requestObjects.splice(objectIndex, 1);
        });
    }

    function onParameterChanged(event) {
        const objects = event.objects || [];
        objects.filter(filterLnbtsPredicate)
            .forEach(object => {
                const parameters = object.parameters;
                const newRequest = ResetRequest.newInstance(object.distName, parameters);
                const objectIndex = requestObjects.findIndex(request => request.isEqual(newRequest));
                if (objectIndex < 0)
                    requestObjects.push(newRequest);
            });
    }
}
