const events = require('../../events');
const _ = require('lodash');

function filesInfoFactory() {

    const FILE_INFO_EVENT = events.newEvent();
    var filesInfo = [];

    return {
        createFileInfo: createFileInfo
    };

    function createFileInfo() {
        return {
            reset: reset,
            addFileInfoListener: addFileInfoListener,
            push: push,
            notify: notify,
            isSuccessful: isSuccessful,
            isEmpty: isEmpty
        };
    }

    function reset() {
        filesInfo = [];
    }

    function addFileInfoListener(watcher) {
        const unwatcher = FILE_INFO_EVENT.watch(watcher);
        FILE_INFO_EVENT.notify(_.cloneDeep(filesInfo));
        return unwatcher;
    }

    function push(fileInfo) {
        filesInfo.push(fileInfo);
    }

    function notify() {
        FILE_INFO_EVENT.notify(filesInfo);
    }

    function isSuccessful() {
        if (isEmpty()) return false;

        return !filesInfo.some(hasError);

        function hasError(element) {
            return element.status === 'status-error';
        }
    }

    function isEmpty() {
        return filesInfo.length === 0;
    }
}

module.exports = filesInfoFactory;
