const fileNameFactory = require('../../utils/fileNameFactory');
const loggers = require('../../logger.impl');
const LOG = loggers.getLogger('admin.core.snapshot');

function snapshotListenerFactory(fileTransfer, zipFactory, fileDownloaderFactory, onlineImTree) {
    return {
        activateFilesDownload: activateFilesDownload,
        //visible for test
        createSnapshotListener: createSnapshotListener
    };

    function activateFilesDownload(snapshotDistName, filesInfoService, interruptSnapshot) {
        return fileTransfer.connect()
            .then(registerSnapshotListener);

        function registerSnapshotListener() {
            const snapshotListener = createSnapshotListener(snapshotDistName, filesInfoService, interruptSnapshot);
            onlineImTree.watchUpdates(snapshotListener.onUpdate)
                .then(function (unwatcher) {
                    snapshotListener.setUnwatcher(unwatcher);
                });
        }
    }

    function createSnapshotListener(snapshotDistName, filesInfoService, interruptSnapshot) {
        var consumedFilesDistNames = [];
        var updatesUnwatcher;
        var lastImTree;
        const zip = zipFactory.newZip();
        const snapshotListener = {
            onUpdate: onUpdate,
            //for tests only
            onSnapshotLRemoved: onSnapshotLRemoved,
            setUnwatcher: function (unwatcher) {
                updatesUnwatcher = unwatcher;
            }
        };
        return snapshotListener;

        function onUpdate(updateEvent) {
            try {
                if (updateEvent.reset) return;

                lastImTree = updateEvent.completeImTree;
                const treeNode = lastImTree.findByDistName(snapshotDistName);
                if (!treeNode) {
                    onSnapshotLRemoved();
                    return;
                }
                const diagnosticFiles = treeNode.getChildren();
                if (diagnosticFiles.length === 0) return;

                diagnosticFiles.forEach(function (file) {
                    if (consumedFilesDistNames.indexOf(file.distName) === -1) {
                        consumedFilesDistNames.push(file.distName);
                        downloadFile(file);
                    }
                });
            } catch (error) {
                LOG.error(error);
            }

            function downloadFile(file) {
                const fileDownloader = fileDownloaderFactory.createFileDownloader(file.distName, zip, filesInfoService, interruptSnapshot);
                fileDownloader.downloadFile(file.getDecodedParameters());
            }
        }

        function onSnapshotLRemoved() {
            updatesUnwatcher.unwatch();
            consumedFilesDistNames = [];
            if (!filesInfoService.isEmpty()) {
                const fileName = fileNameFactory.createName(lastImTree, 'Snapshot', 'zip');
                zip.setName(fileName);
                zip.saveZip();
            }
            fileTransfer.disconnect();
        }
    }
}

module.exports = snapshotListenerFactory;
