//const loggers = require('../../logger.impl');
const distNameUtils = require('../../utils/distNameUtils');

function snapshotTRAgent(imTracker) {

    const SNAPSHOT_TRAGENT_CLASS = 'TRAGENT_L';
    const PROCEDURE_LISTENER = {
        onListenerRegistered,
        onParameterChanged,
        onObjectRemoved
    };
    let activeElements = [];
    imTracker.trackByClassName(SNAPSHOT_TRAGENT_CLASS, PROCEDURE_LISTENER, ['usageState']);

    return {
        isSnapshotTRAgentActive: isSnapshotTRAgentActive,
        getActiveTRAgentsParents: getActiveTRAgentsParents,
        //needed for testing
        parameterChanged: onParameterChanged,
        objectRemoved: onObjectRemoved
    };

    function getActiveTRAgentsParents() {
        let trAgentParentsArray = [];
        activeElements.forEach((distName) => {
            trAgentParentsArray.push( distNameUtils.getParentClassName(distName) );
        });
        return trAgentParentsArray;
    }

    function removeInactiveElement(distName) {
        const objectIndex = activeElements.indexOf(distName);
        if (objectIndex > -1) activeElements.splice(objectIndex, 1);
    }

    function addRemoveTRAgents(event) {
        const objects = event.objects;
        if (!objects) return;

        objects.forEach((object) => {
            const parameters = object.parameters;
            const distName = object.distName;
            if (parameters.usageState === 'Active')
                activeElements.push(distName);
            else
                removeInactiveElement(distName);
        });
    }

    function onListenerRegistered(event) {
        activeElements = [];
        addRemoveTRAgents(event);
    }

    function onParameterChanged(event) {
        addRemoveTRAgents(event);
    }

    function onObjectRemoved(event) {
        if (!event.distNames) return;

        event.distNames.forEach((distName) => {
            removeInactiveElement(distName);
        });
    }

    function isSnapshotTRAgentActive() {
        return (activeElements.length > 0);
    }
}

module.exports = snapshotTRAgent;
