const Unzip = require('zlibjs/bin/unzip.min').Zlib.Unzip;

const LOG = require('../../../logger.impl').getLogger('admin.core.softwareZipDispatcherFactory');
const targetbdParser = require('./targetbdParser');

module.exports = softwareZipDispatcherFactory;

function softwareZipDispatcherFactory( ) {
    var dispatcher;

    return {
        fromZipFile: fromZipFile,
        getDispatcher: getDispatcher,
        isDispatcherCreated: isDispatcherCreated
    };

    function fromZipFile(buffer) {
        dispatcher = createDispatcher();

        return this;

        function createDispatcher() {
            const zipFile = new Unzip(new Uint8Array(buffer));
            const parser = targetbdParser.newInstance(zipFile);

            return {
                getFile: getFile,
                isZipLoaded: isZipLoaded,
                getTargetBd: getTargetBd
            };

            function getFile(uri) {
                const URI_PREFIX = 'bts_sw';
                const completeUri = URI_PREFIX + uri;
                if (!!zipFile) {
                    const file = zipFile.decompress(completeUri);

                    if (file)
                        return file.slice(0).buffer;
                }
                LOG.error('File: ' + completeUri + ' does not exist.');
                return null;
            }

            function getTargetBd() {
                return parser.isTargetBd() && parser.getTargetBd();
            }

            function isZipLoaded() {
                return !!zipFile && parser.isTargetBd();
            }
        }
    }

    function getDispatcher() {
        return dispatcher;
    }

    function isDispatcherCreated() {
        return !!dispatcher;
    }
}
