const LOG = require('../../../logger.impl').getLogger('admin.core.targetbdParser');
const xml2json = require('../../../xml2json');

module.exports.newInstance = targetbdParser;

const TARGET_BD_END_TAG_PHRASE = '>';
const TARGET_BD_START_TAG_PHRASE = '<targetBD';
const TARGET_BD_CLOSING_TAG = '</targetBD>';
const TARGET_BD_DESCRIPTOR = 'targetBD';
const TARGET_BD_PROPERTIES = {
    BUILD_DESCRIPTOR: '@buildDescriptorId',
    SOFTWARE_RELEASE_DATE: '@softwareReleaseDate',
    SOFTWARE_RELEASE_VERSION: '@softwareReleaseVersion'
};

const TARGET_BD_FILE_REGEX = /TargetBD.*\.xml/;

function targetbdParser(zipFile) {
    const zippedTargetBd = getZippedTargetBd();
    const targetBdInfo = parseTargetBd(getTargetBdFile(zippedTargetBd));

    return {
        getTargetBd: getTargetBd,
        isTargetBd: isTargetBd
    };

    function uint8ArrayToString(array) {
        return array.reduce(function (string, charCode) {
            return string + String.fromCharCode(charCode);
        }, '');
    }

    function getTargetBdFile(src) {
        const file = zipFile.decompress(src);
        return uint8ArrayToString(file);
    }

    function parseTargetBd(targetBd) {
        const tag = getTargetBdTag();
        const tagAsJson = xml2json.toJson(tag);
        const targetBdObject = tagAsJson[TARGET_BD_DESCRIPTOR];

        const buildDescriptorId = targetBdObject[TARGET_BD_PROPERTIES.BUILD_DESCRIPTOR],
            softwareReleaseDate = targetBdObject[TARGET_BD_PROPERTIES.SOFTWARE_RELEASE_DATE],
            softwareReleaseVersion = targetBdObject[TARGET_BD_PROPERTIES.SOFTWARE_RELEASE_VERSION];


        if (isTargetBdCorrectlyParsed()) {
            LOG.error('TargetBD has not been correctly parsed.');
            return null;
        }

        return {
            buildDescriptorId: buildDescriptorId,
            softwareReleaseDate: softwareReleaseDate,
            softwareReleaseVersion: softwareReleaseVersion
        };

        function getTargetBdTag() {
            const start = targetBd.indexOf(TARGET_BD_START_TAG_PHRASE);
            const end = targetBd.indexOf(TARGET_BD_END_TAG_PHRASE, start);
            return (start !== -1 && end !== -1) ? targetBd.substring(start, end + 1) + TARGET_BD_CLOSING_TAG : '';
        }

        function isTargetBdCorrectlyParsed() {
            return !buildDescriptorId || !softwareReleaseDate || !softwareReleaseVersion;
        }
    }

    function getZippedTargetBd() {
        const files = zipFile.getFilenames().filter(function (name) {
            return name.match(TARGET_BD_FILE_REGEX);
        });
        if (files.length === 1) return files[0];

        LOG.error('TargetBD has not been correctly found.');
        return null;
    }

    function getTargetBd() {
        return {
            name: zippedTargetBd.split('/')[1],
            content: zippedTargetBd,
            properties: targetBdInfo
        };
    }

    function isTargetBd() {
        return !!zippedTargetBd && !!targetBdInfo;
    }
}
