const LOG = require('../../../logger.impl').getLogger('admin.core.swListDownloader');

function swListDownloader(webdav) {
    var hostname = '', port = '', protocol = '';
    var connectionAvailability = false;

    return {
        getSoftwareList: getSoftwareList,
        setSoftwareServer: setSoftwareServer,
        getHostname: getHostname,
        isConnectionAvailable: isConnectionAvailable
    };

    function getSoftwareList(address) {
        var endpointAddress = address || createSoftwareListEndpointAddress();

        return webdav
            .propfind(endpointAddress, 'infinity')
            .then(function (data) {
                connectionAvailability = true;
                return Promise.resolve(data);
            })
            .catch(function (error) {
                connectionAvailability = false;
                LOG.error('Release server: ' + endpointAddress + ' does not respond.');
                return Promise.reject(error);
            });
    }

    function isConnectionAvailable() {
        return connectionAvailability;
    }

    function setSoftwareServer(config) {
        hostname = config.hostname || '';
        port = config.port || '';
        protocol = config.protocol || '';
    }

    function getHostname() {
        return hostname;
    }

    function createSoftwareListEndpointAddress() {
        return protocol + '//' + hostname + ':' + port;
    }
}

module.exports = swListDownloader;
