module.exports = {newInstance};

const events = require('../../events');
const imTrackerCreator = require('../../imTracker/imTracker');
const resetRequestTracker = require('../resetRequestTracker');

function newInstance({imTracker = imTrackerCreator(), resetRequestTracker = resetRequestTracker()} = {}) {
    const SW_UPDATE_L_DISTNAME = '/MRBTS-1/RAT-1/BTS_L-1/FUNC_L-1/SW_UPDATE_L-1';
    const SW_UPDATE_STATE_PARAM = 'stateInfo.swUpdateState';
    const SW_UPDATE_STATE_BUSY = 'Busy';
    const SW_UPDATE_EVENT = events.newEvent();
    const swUpdateLListener = {
        onListenerRegistered: onParameterChanged,
        onParametersChanged: onParameterChanged
    };

    var swUpdateState;

    imTracker.trackByDistName(SW_UPDATE_L_DISTNAME, swUpdateLListener, SW_UPDATE_STATE_PARAM);

    return {
        isAvailable: isAvailable,
        watch: SW_UPDATE_EVENT.watch,
        unwatch: SW_UPDATE_EVENT.unwatch,
        //for tests only
        onParameterChanged: onParameterChanged
    };

    function onParameterChanged(event) {
        const newSwUpdateState = _.get(event.parameters, SW_UPDATE_STATE_PARAM);
        if (newSwUpdateState === swUpdateState) return;

        swUpdateState = newSwUpdateState;
        SW_UPDATE_EVENT.notify(swUpdateState);
    }

    function isAvailable() {
        return swUpdateState && swUpdateState !== SW_UPDATE_STATE_BUSY && !resetRequestTracker.isResetExecuting();
    }
}
