const _ = require('lodash'),
    log = require('./logger.impl').getLogger('admin.core.sequentialExecutor');

module.exports = {newInstance: newInstance};

function newInstance() {

    var queue = [];

    function promisifyFunction(thisArg, fn) {
        return function () {
            var args = arguments;
            var deferredPromise = Promise.pending();

            var task = function () {
                try {
                    var fnResult = fn.apply(thisArg, args);
                    if (isPromise(fnResult)) redirectPromiseResult(fnResult, deferredPromise);
                    else deferredPromise.fulfill(fnResult);
                } catch (e) {
                    deferredPromise.reject(e);
                }
                return deferredPromise.promise;
            };

            push(task);
            return deferredPromise.promise;
        };
    }

    function isPromise(object) {
        return object instanceof Promise;
    }

    function redirectPromiseResult(promise, deferredPromise) {
        promise.then(function (result) {
            deferredPromise.fulfill(result);
        }).catch(function (error) {
            deferredPromise.reject(error);
        });
    }

    function push(item) {
        queue.push(item);
        if (queue.length === 1) {
            schedule();
        }
    }

    function schedule() {
        const head = queue[0];
        head().then(scheduleLoop).catch(function (e) {
            log.error(e);
            return scheduleLoop();
        });
    }

    function scheduleLoop() {
        queue.shift();
        if (queue.length > 0) {
            schedule();
        }
    }

    return {
        promisifyFunction: promisifyFunction
    };
}
