const events = require('../events');
const sessionsResolver = require('./sessionsResolver');
module.exports = {
    newInstance
};
function newInstance(btsClient) {
    let currentSessionList = [];
    const sessionsEvent = createSessionsEvent();
    const btsClientListener = {
        sessionsList: (sessionList) => {
            currentSessionList = sessionsResolver.resolve(sessionList);
            sessionsEvent.notify(currentSessionList);
        }
    };

    return {
        watch,
        getList,
        requestNewList
    };

    function createSessionsEvent() {
        const event = events.newEvent();
        event.onFirstAdded(() => {
            btsClient.addListener(btsClientListener);
        });
        event.onLastRemoved(() => {
            btsClient.removeListener(btsClientListener);
        });
        return event;
    }

    function watch(watcher) {
        watcher(currentSessionList);
        return sessionsEvent.watch(watcher);
    }

    function getList() {
        return currentSessionList;
    }

    function requestNewList(timeout = 5000) {
        return new Promise((resolve, reject) => {
            let isFinished;
            const watcher = sessionsEvent.watch((sessionsList) => {
                if (isFinished) return;

                watcher.unwatch();
                resolve(sessionsList);
                isFinished = true;
            });
            btsClient.sendSessionsListRequest();
            setTimeout(() => {
                if (isFinished) return;

                watcher.unwatch();
                reject('Timeout exceeded');
                isFinished = true;
            }, timeout);
        });
    }
}
