module.exports = currentSettingsService;

const _ = require('lodash');
const defaultSettings = require('./defaultSettings');

function currentSettingsService({localStorage}) {
    let settings = localStorage.settings;
    if (!settings) restore();

    return {
        hideNotSupportedBrowserWarning: hideNotSupportedBrowserWarning,
        updateInterval: updateInterval,
        updateScreenLockTimeout: updateScreenLockTimeout,
        equals: equals,
        get: get
    };

    function hideNotSupportedBrowserWarning(isWarningDisabled) {
        return _.isUndefined(isWarningDisabled)
            ? settings.hideNotSupportedBrowserWarning
            : (settings.hideNotSupportedBrowserWarning = isWarningDisabled);
    }

    function updateInterval(newUpdateInterval) {
        return _.isUndefined(newUpdateInterval)
            ? settings.updateInterval
            : (settings.updateInterval = newUpdateInterval);
    }

    function updateScreenLockTimeout(newScreenLockTimeout) {
        return _.isUndefined(newScreenLockTimeout)
            ? settings.screenLockTimeout
            : (settings.screenLockTimeout = newScreenLockTimeout);

    }

    function equals(otherSettings) {
        return _.isEqual(settings, otherSettings);
    }

    function get() {
        return settings;
    }

    function restore() {
        localStorage.settings = {
            updateInterval: defaultSettings.updateInterval(),
            screenLockTimeout: defaultSettings.updateScreenLockTimeout(),
            hideNotSupportedBrowserWarning: defaultSettings.hideNotSupportedBrowserWarning()
        };
        settings = localStorage.settings;
        return settings;
    }
}
