const sfpConnections = require('./sfpConnections');
const events = require('../events');

module.exports =  { newInstance };

function newInstance(updateService) {

    let imTree;
    const event = events.newEvent();

    const imTreeListener = {
        completeImTree: function (tree) {
            imTree = tree;
            notifyImTreeChanged();
        }
    };

    event.onFirstAdded(() => updateService.addImTreeListener(imTreeListener));
    event.onLastRemoved(() => updateService.removeImTreeListener(imTreeListener));

    return {
        createConnections,
        STATUSES: sfpConnections.STATUSES,
        watchSfpConnections: event.watch,
        unwatchSfpConnections: event.unwatch,
        getFaultDetails,
        getModuleDetails
    };

    function createConnections() {
        return sfpConnections.create(imTree);
    }

    function getFaultDetails(distName) {
        return sfpConnections.getFaultDetails(distName, imTree);
    }

    function getModuleDetails(distName) {
        return sfpConnections.getModuleDetails(distName, imTree);
    }

    function notifyImTreeChanged() {
        event.notify(createConnections());
    }
}
