module.exports = {
    assignSecondaryCellsToEachCellMappedToRuntime,
    assignSecondaryCellsToEachCell,
    collectInterCarrierAggregationCells
};

const _ = require('lodash'),
    cellMapsService = require('../cells/siteCellMapsService');

function assignSecondaryCellsToEachCell(cells, btsId, imTree, cellMaps) {
    var maps = cellMaps || cellMapsService.create(imTree);
    assignSecondaryCells(cells, btsId, {
        cellDistNameToLncelANode: maps.lcellADistNameToLncelANodeMap,
        lcrIdToCellDistName: maps.lcrIdToLcellADistNameMap
    });
}

function assignSecondaryCellsToEachCellMappedToRuntime(cells, btsId, imTree, cellMaps) {
    var maps = cellMaps || cellMapsService.create(imTree);
    assignSecondaryCells(cells, btsId, {
        cellDistNameToLncelANode: maps.lcellDistNameToLncelANodeMap,
        lcrIdToCellDistName: maps.lcrIdToLcellDistNameMap
    });
}

function assignSecondaryCells(cells, btsId, maps) {
    _.each(cells, function (cell) {
        var lncelANode = maps.cellDistNameToLncelANode.get(cell.distName);
        var carelNodes = lncelANode ? lncelANode.findChildrenByClassName('CAREL_A') : [];

        var secondaryCells = [];
        _.each(carelNodes, function (carelNode) {

            var lcrId = carelNode.getDecodedParameters().lcrId;
            var cellSource = carelNode.getDecodedParameters().lnBtsId;

            var cellDistName;
            if (cellSource === btsId)
                cellDistName = maps.lcrIdToCellDistName.get(lcrId);
            else
                cellDistName = '/LNBTS-' + cellSource + '/LCELL-' + lcrId;


            secondaryCells.push({
                cellDn: cellDistName,
                cellId: lcrId,
                btsId: cellSource
            });
        });
        cell.secondaryCells = secondaryCells;
    });
}

function collectInterCarrierAggregationCells(btsId, cells) {
    return _(cells)
        .map(function (cell) {
            return cell.secondaryCells;
        })
        .flatten()
        .filter(function (secondaryCell) {
            return secondaryCell.btsId !== btsId;
        })
        .uniq('cellDn')
        .sortBy('cellDn')
        .value();
}

