const CPU_LOAD_DIVIDER = 65535;

module.exports = {
    CPU_LOAD_DIVIDER: CPU_LOAD_DIVIDER,
    createMeasurements: createMeasurements,
    createCpuUsage: createCpuUsage,
    createRamUsage: createRamUsage,
    createDiskUsage: createDiskUsage
};

const _ = require('lodash');

const CPU_NOT_MEASURED = -1;

const hardwareClasses = {
    FILE_SYSTEM: 'FILE_SYSTEM_L',
    STORAGE: 'STORAGE_L',
    MEMORY: 'MEMORY_L',
    CCU: 'CCU_L',
    CPU: 'CPU_L',
    SOC: 'SOC_L'
};

function createMeasurements(moduleNode) {
    var measurements = _([])
        .concat(createCpuUsage(moduleNode))
        .concat(createRamUsage(moduleNode))
        .concat(createDiskUsage(moduleNode))
        .value();

    if (measurements.length > 0)
        return measurements;
    return null;
}

function createCpuUsage(moduleNode) {
    const cpuNode = getCpuNode(moduleNode);
    if (!cpuNode)  return [];

    var parameters = cpuNode.getDecodedParameters();
    if (!parameters.cpuLoad || parameters.cpuLoad === CPU_NOT_MEASURED) return [];

    var value = parameters.cpuLoad / CPU_LOAD_DIVIDER;
    return [{name: 'CPU', value: value.round()}];
}

function createRamUsage(moduleNode) {
    const cpuNode = getCpuNode(moduleNode);
    if (!cpuNode)  return [];

    const memoryNode = cpuNode.findChildByClassName(hardwareClasses.MEMORY);
    if (!memoryNode)  return [];

    var parameters = memoryNode.getDecodedParameters();
    var value = parameters.usedCapacity / parameters.totalCapacity * 100;
    return [{name: 'RAM', value: value.round()}];
}

function getCpuNode(parentNode) {
    var node = parentNode;
    var ccuNode = node.findChildByClassName(hardwareClasses.CCU);
    if (ccuNode)
        node = ccuNode;

    const socNode = node.findChildByClassName(hardwareClasses.SOC);
    if (socNode)
        node = socNode;

    return node.findChildByClassName(hardwareClasses.CPU);
}

function createDiskUsage(moduleNode) {
    const ccuNode = moduleNode.findChildByClassName(hardwareClasses.CCU);
    const storageNode = ccuNode && ccuNode.findChildByClassName(hardwareClasses.STORAGE);
    if (!storageNode) return [];

    const fileSystemNodes = storageNode.findChildrenByClassName(hardwareClasses.FILE_SYSTEM);
    return createDiskMeasurements(fileSystemNodes);

    function createDiskMeasurements(nodes) {
        const measurements = nodes.map(function (node) {
            const parameters = node.getDecodedParameters();
            const value = parameters.usedCapacity / parameters.totalCapacity * 100;
            return {name: parameters.mountPoint, value: value.round()};
        });
        return _.sortBy(measurements, 'name');
    }
}


