module.exports = {
    createModules
};

const imCompatibilityService = require('../../utils/imCompatibilityService'),
    hardwareClasses = require('../../siteCommon/hardwareClassesService'),
    siteConcentratorsFactory = require('./siteConcentratorsFactory');

const moduleProducers = {};
let genericModuleProducer;

function createModules(imTree, moduleFactories) {
    initModuleProducers(moduleFactories);

    const modules = {
        pwrmods: [],
        cabinets: [],
        trmods: [],
        smods: [],
        extSmods: [],
        bbmods: [],
        rmods: [],
        mhas: [],
        rets: [],
        raes: [],
        phyants: []
    };

    const eqm = imCompatibilityService.findEqmLNode(imTree);
    if (!eqm) return modules;

    modules.cabinets = createCabinets(eqm, imTree);
    modules.pwrmods = createModulesByClassName(eqm, hardwareClasses.MODULES.PWRMOD, imTree);
    modules.trmods = createModulesByClassName(eqm, hardwareClasses.MODULES.TRMOD, imTree);
    modules.smods = createModulesByClassName(eqm, hardwareClasses.MODULES.SMOD, imTree);
    modules.extSmods = createModulesByClassName(eqm, hardwareClasses.MODULES.SMOD_EXT, imTree);
    modules.bbmods = createModulesByClassName(eqm, hardwareClasses.MODULES.BBMOD, imTree);
    modules.rmods = createModulesByClassName(eqm, hardwareClasses.MODULES.RMOD, imTree);
    modules.mhas = createModulesByClassName(eqm, hardwareClasses.MODULES.MHA, imTree);
    modules.rets = createModulesByClassName(eqm, hardwareClasses.MODULES.RET, imTree);
    modules.phyants = createModulesByClassName(eqm, hardwareClasses.MODULES.PHYANT, imTree);
    modules.raes = createModulesByClassName(eqm, hardwareClasses.MODULES.RAE, imTree);
    return modules;
}

function createCabinets(eqm, imTree) {
    const cabinetNodes = eqm.findChildrenByClassName(hardwareClasses.MODULES.CABINET);
    return cabinetNodes.map((cabinetNode) => {
        const cabinet = createModule(cabinetNode, imTree);
        cabinet.smods = createModulesByClassName(cabinetNode, hardwareClasses.MODULES.SMOD, imTree);
        cabinet.bbmods = createModulesByClassName(cabinetNode, hardwareClasses.MODULES.BBMOD, imTree);
        return cabinet;
    });
}

function initModuleProducers(moduleFactories) {
    if (!moduleFactories) return;

    moduleProducers[hardwareClasses.MODULES.DSP] = moduleFactories.dsp;
    moduleProducers[hardwareClasses.MODULES.MHA] = moduleFactories.mha;
    moduleProducers[hardwareClasses.MODULES.RMOD] = moduleFactories.rmod;
    moduleProducers[hardwareClasses.MODULES.TRU] = moduleFactories.tru;
    moduleProducers[hardwareClasses.MODULES.BBMOD] = moduleFactories.bbmod;
    moduleProducers[hardwareClasses.MODULES.RET] = moduleFactories.ret;
    moduleProducers[hardwareClasses.MODULES.RAE] = moduleFactories.rae;
    moduleProducers[hardwareClasses.MODULES.CABINET] = moduleFactories.cabinet;
    genericModuleProducer = moduleFactories.generic;
}

function createModulesByClassName(parentNode, className, imTree) {
    const nodes = parentNode.findChildrenByClassName(className);
    return nodes.map((moduleNode) => {
        const module = createModule(moduleNode, imTree);
        module.children = module.children.concat(createModulesByClassName(moduleNode, hardwareClasses.MODULES.TRU, imTree));
        module.children = module.children.concat(createModulesByClassName(moduleNode, hardwareClasses.MODULES.BBU, imTree));
        module.children = module.children.concat(createModulesByClassName(moduleNode, hardwareClasses.MODULES.PDU, imTree));
        return module;
    });
}

function createModule(moduleNode, imTree) {
    const moduleProducer = moduleProducers[moduleNode.className];
    const module = moduleProducer ? moduleProducer(moduleNode, imTree) : genericModuleProducer(moduleNode);
    module.concentrators = siteConcentratorsFactory.create(moduleNode);
    return module;
}

