module.exports = {
    createCell,
    createChannel,
    createBackEndConnector,
    createFrontEndConnector,
    createLink,
    createModule,
    createQfspNode,
    createPowerGroupWithDsp
};


function createCell(imTree, lnbts, cellNumber, parameters) {
    var lncelNode = imTree.addNode(lnbts, 'LNCEL', cellNumber);
    lncelNode.parameters = parameters || {};
    var cellNode = imTree.addNode(lnbts, 'LCELL', cellNumber);
    cellNode.parameters = parameters || {};
    lncelNode.parameters.LCELL_DN = cellNode.distName;
    var subCellNode = imTree.addNode(cellNode, 'SUBCELL', 0);
    subCellNode.parameters = {};
    return cellNode;
}

function createChannel(imTree, lnbts, parentCellNumber, channelNumber, parameters) {
    var subCellNode = imTree.findByDistName(lnbts.distName + '/LCELL-' + parentCellNumber + '/SUBCELL-0');
    var channelNode = imTree.addNode(subCellNode, 'CHANNEL', channelNumber);
    channelNode.parameters = parameters || {};
    return channelNode;
}


function createBackEndConnector(imTree, parentNode, connectorNumber) {
    return createConnector(imTree, parentNode, connectorNumber, 'backend');
}

function createFrontEndConnector(imTree, parentNode, connectorNumber) {
    return createConnector(imTree, parentNode, connectorNumber, 'frontend');
}

function createConnector(imTree, parentNode, connectorNumber, position) {
    var node = imTree.addNode(parentNode, 'CONNECTOR_L', connectorNumber);
    node.parameters = {
        connectorLabel: '' + connectorNumber,
        position: position,
        stateInfo: 'stateInfo'
    };
    return node;
}

function createLink(imTree, hwtopL, linkNumber, source, target) {
    var node = imTree.addNode(hwtopL, 'CABLNK_L', linkNumber);
    node.parameters = {
        sourceDistName: source.distName,
        destinationDistName: target.distName
    };
    return node;
}

function createModule(imTree, eqmL, className, id, parameters, parentNode) {
    parentNode = parentNode || eqmL;
    var moduleNode = imTree.addNode(parentNode, className, id);
    moduleNode.parameters = parameters || {};
    return moduleNode;
}

function createQfspNode(imTree, module, id) {
    const node = imTree.addNode(module, 'QSFP_L', id);
    node.parameters = {
        physicalLabel: 'BBEXT-' + id,
        stateInfo: 'stateInfo'
    };
    return node;
}

function createPowerGroupWithDsp(parentNode, powerGroupId, socId) {
    const powerGroup = createModule('POWER_GROUP_L', powerGroupId, {}, parentNode);
    const soc = createModule('SOC_L', socId, {}, powerGroup);
    createModule('DSP_L', 1, {}, soc);
    return powerGroup;
}

