module.exports = {
    create: createHardwareGraph
};

const dn = require('../../utils/distNameUtils');

function createDistNameClassHelper(hardwareClasses) {
    return {
        isConnectorDN: function (distName) {
            return hardwareClasses.CONNECTOR === dn.getClassName(distName);
        },

        isChannelDN: function (distName) {
            return hardwareClasses.CHANNEL === dn.getClassName(distName);
        }
    };
}

function createHardwareGraph(hardware, links, hardwareClasses) {
    var dnClass = createDistNameClassHelper(hardwareClasses);
    var graph = {};

    if (!links)
        return graph;

    links.forEach(function (link) {
        var sourceDN = link.source.distName;
        var targetDN = link.target.distName;

        var moduleSourceDN = sourceDN;
        var moduleTargetDN = targetDN;

        var sourceConnectorDN, targetConnectorDN;

        if (dnClass.isConnectorDN(sourceDN)) {
            sourceConnectorDN = sourceDN;
            moduleSourceDN = dn.getParent(sourceDN);
        }
        if (dnClass.isConnectorDN(targetDN)) {
            targetConnectorDN = targetDN;
            moduleTargetDN = dn.getParent(targetDN);
        }
        if (dnClass.isChannelDN(sourceDN)) {
            var sourceSubCell = dn.getParent(sourceDN);
            var sourceCell = dn.getParent(sourceSubCell);
            moduleSourceDN = sourceCell;
        }
        if (dnClass.isChannelDN(targetDN)) {
            var targetSubCell = dn.getParent(targetDN);
            var targetCell = dn.getParent(targetSubCell);
            moduleTargetDN = targetCell;
        }

        graph[moduleSourceDN] = graph[moduleSourceDN] || [];
        graph[moduleSourceDN].push({
            module: hardware.mapByDistName[moduleTargetDN]
        });

        graph[moduleTargetDN] = graph[moduleTargetDN] || [];
        graph[moduleTargetDN].push({
            module: hardware.mapByDistName[moduleSourceDN]
        });

        if (targetConnectorDN)
            graph[moduleSourceDN].push({
                module: hardware.mapByDistName[targetConnectorDN]
            });

        if (sourceConnectorDN)
            graph[moduleTargetDN].push({
                module: hardware.mapByDistName[sourceConnectorDN]
            });

    });

    graph.adj = function (distName) {
        return graph[distName] || [];
    };
    return graph;
}


