var SORTER = {
    BY_LABEL: function (a, b) {
        return a.label.localeCompare(b.label);
    },
    BY_NUMBER: function (a, b) {
        return a.number - b.number;
    },
    BY_POSITION: function (a, b) {
        return (a.position || LAST) - (b.position || LAST);
    }
};

module.exports = {
    SORTER,
    computePosition,
    sortHardware
};

const hardwareGraph = require('./siteHardwareGraphService'),
    _ = require('lodash');

var LAST = 1000000000;

function parsePosition(position) {
    if (position && !_.isNaN(position))
        return parseInt(position, 10);
    return null;
}

function computeChildrenPosition(module, graph) {
    if (module.children && module.children.length) {
        computePosition(module.children, graph);
        return _(module.children).map('position').sum() / module.children.length;
    }
    return null;
}

function computePositionFromAdj(adjVertexes) {
    var positionFromAdj = 0, positionFromAdjFound = 0;
    _.forEach(adjVertexes, function (adjVertex) {
        var adjModule = adjVertex.module;
        if (adjModule) {
            var adjPosition = parsePosition(adjModule.position);
            if (adjPosition) {
                positionFromAdj += adjPosition;
                positionFromAdjFound++;
            }
        }
    });
    if (positionFromAdjFound > 0) {
        positionFromAdj /= positionFromAdjFound;
        return positionFromAdj;
    }
    return null;
}

function computePosition(modules, graph) {
    _.forEach(modules, function (module, index) {
        module.position = index + 1;
        var adjVertexes = graph.adj(module.distName);
        var positionFromAdj = computePositionFromAdj(adjVertexes);

        if (positionFromAdj) {
            module.position += positionFromAdj;
        } else {
            var childrenPosition = computeChildrenPosition(module, graph);
            if (childrenPosition)
                module.position += childrenPosition;
            else
                module.position += LAST;
        }
    });
}

function computeRmodsPosition(modules) {
    _.forEach(modules, function (module, index) {
        module.position = (index + 1) * 1000;
        _.forEach(module.rightConnectors, function (connector, connectorIndex) {
            connector.position = (connectorIndex + 1) * 100;
        });
    });
}

function sortWithChildren(modules, sortStrategy) {
    modules.sort(sortStrategy);
    _.forEach(modules, function (module) {
        if (module.children)
            module.children.sort(sortStrategy);
    });
}

function sortHardware(hardware, hardwareClasses) {
    hardware.trmods.sort(SORTER.BY_NUMBER);
    hardware.smods.sort(SORTER.BY_NUMBER);
    hardware.extSmods.sort(SORTER.BY_NUMBER);
    hardware.bbmods.sort(SORTER.BY_NUMBER);
    hardware.rmods.sort(SORTER.BY_NUMBER);
    computeRmodsPosition(hardware.rmods);
    var graph = hardwareGraph.create(hardware, hardware.links, hardwareClasses);

    computePosition(hardware.mhas, graph);
    sortWithChildren(hardware.mhas, SORTER.BY_POSITION);

    computePosition(hardware.rets, graph);
    hardware.rets.sort(SORTER.BY_POSITION);

    computePosition(hardware.raes, graph);
    hardware.raes.sort(SORTER.BY_POSITION);

    computePosition(hardware.phyants, graph);
    hardware.phyants.sort(SORTER.BY_POSITION);

    var logGraph = hardwareGraph.create(hardware, hardware.logLinks, hardwareClasses);
    hardware.cells.sort(SORTER.BY_NUMBER);
    computePosition(hardware.cells, logGraph);
    hardware.cells.sort(SORTER.BY_POSITION);
}

return {
    SORTER: SORTER,
    computePosition: computePosition,
    sortHardware: sortHardware
};

