const hardwareFactory = require('./siteHardware/hardwareFactory'),
    siteModuleFactory = require('./factories/siteModuleFactory'),
    siteModulesFactory = require('./factories/siteModulesFactory'),
    lnmmesFactory = require('./factories/lnmmesFactory'),
    linksFactory = require('./factories/linksFactory'),
    cellBuilder = require('./factories/siteCellFactory'),
    controllingRetLinksFactory = require('./factories/controllingRetLinksFactory'),
    alarmsFactory = require('./factories/alarmsFactory');

module.exports = {
    create: createHardware,
    createModules,
    createLnmmes: lnmmesFactory.create,
    createLinks: linksFactory.create,
    createCellsAndLogLinks: cellBuilder.createCellsAndLogLinks,
    findAlarms: alarmsFactory.create,
    createControllingRetLinks: controllingRetLinksFactory.create
};

const modulesFactories = {
    mha: siteModuleFactory.createMhaModule,
    rmod: siteModuleFactory.createRmodModule,
    tru: siteModuleFactory.createModuleWithCommissioningStatus,
    ret: siteModuleFactory.createRetModule,
    rae: siteModuleFactory.createRaeModule,
    bbmod: siteModuleFactory.createModuleWithCommissioningStatus,
    generic: siteModuleFactory.createGenericModule,
    cabinet: siteModuleFactory.createCabinet
};

function createModules(imTree) {
    return siteModulesFactory.createModules(imTree, modulesFactories);
}

function createHardware(imTree) {
    return hardwareFactory.create(imTree, modulesFactories);
}
