module.exports = {
    getDiodeClass,
    getCellClass,
    mergeStateInfos
};

const _ = require('lodash');

function mergeOperationalState(stateInfos) {
    if (_.all(stateInfos, {operationalState: 'Enabled'}))
        return 'Enabled';
    if (_.some(stateInfos, {operationalState: 'Disabled'}))
        return 'Disabled';

    return null;
}

function mergeAvailabilityStatus(stateInfos) {
    if (_.all(stateInfos, {availabilityStatus: 'Online'}))
        return 'Online';

    if (_.some(stateInfos, {availabilityStatus: 'Failed'}))
        return 'Failed';

    if (_.some(stateInfos, {availabilityStatus: 'Offline'}))
        return 'Offline';

    if (_.some(stateInfos, {availabilityStatus: 'NotInstalled'}))
        return 'NotInstalled';

    if (_.some(stateInfos, {availabilityStatus: 'Degraded'}))
        return 'Degraded';

    return null;
}

function mergeUsageState(stateInfos) {
    if (_.all(stateInfos, {usageState: 'Idle'}))
        return 'Idle';

    if (_.some(stateInfos, {usageState: 'Busy'}))
        return 'Busy';

    if (_.some(stateInfos, {usageState: 'Active'}))
        return 'Active';


    return null;
}

function mergeStateInfos(stateInfos) {
    return {
        operationalState: mergeOperationalState(stateInfos),
        availabilityStatus: mergeAvailabilityStatus(stateInfos),
        usageState: mergeUsageState(stateInfos)
    };
}

function getDiodeClass(stateInfo) {
    var operational = stateInfo.operationalState;
    var availability = stateInfo.availabilityStatus;
    var usage = stateInfo.usageState;

    if (operational === 'Enabled') {

        if (availability === 'Failed') return 'red-diode';
        if (availability === 'Offline') return 'gray-diode';
        if (availability === 'NotInstalled') return 'gray-diode';

        if (availability === 'Online' && usage === 'Busy') return 'blinking-green-diode';
        if (availability === 'Online' && usage === 'Active') return 'fast-blinking-green-diode';
        if ((!availability || availability === 'Online') && usage === 'Idle') return 'green-diode';

        if (availability === 'Degraded' && usage === 'Busy') return 'blinking-red-diode';
        if (availability === 'Degraded' && usage === 'Active') return 'blinking-yellow-diode';
        if (availability === 'Degraded' && usage === 'Idle') return 'yellow-diode';
    }

    if (operational === 'Disabled') return 'gray-diode';

    return 'unknown-diode';
}

function getCellClass(stateInfo) {
    var operational = stateInfo.operationalState;
    var availability = stateInfo.availabilityStatus;

    if (operational === 'Enabled' && availability === 'Failed') return 'red-cell';
    if (operational === 'Enabled' && availability === 'Offline') return 'gray-cell';
    if (operational === 'Enabled' && availability === 'NotInstalled') return 'gray-cell';
    if (operational === 'Enabled' && availability === 'Online') return 'green-cell';
    if (operational === 'Enabled' && availability === 'Degraded') return 'yellow-cell';
    if (operational === 'Disabled') return 'gray-cell';

    return '';
}




