const NUMBER_PRECISION = 3;

function toSeconds(milliseconds) {
    return parseInt(milliseconds, 10) / 1000;
}
function removeRootPrefix(distName) {
    return distName.replace('/MRBTS-1/RAT-1/EQM_L-1/', '').replace('/MRBTS-1/RAT-1/', '');
}
function createTiming(timingsNode) {

    const parameters = timingsNode.getDecodedParameters();
    const startInSeconds = toSeconds(parameters.startTick);
    const durationInSeconds = toSeconds(parameters.duration);

    return {
        parentName: removeRootPrefix(timingsNode.parent.distName),
        offset: startInSeconds,
        width: durationInSeconds,
        details: {
            distName: timingsNode.distName,
            procedureName: parameters.procedureName,
            start: `${startInSeconds.toFixed(NUMBER_PRECISION)}s`,
            duration: `${durationInSeconds.toFixed(NUMBER_PRECISION)}s`,
            end: `${(startInSeconds + durationInSeconds).toFixed(NUMBER_PRECISION)}s`
        }
    };
}

function findTimings(predicate, imTree) {
    return imTree.findByClassName('TIMINGS_L').filter(predicate).map(createTiming);
}

function getRegex(pattern) {
    return new RegExp(`^.*${pattern}.*$`, 'i');
}

const alwaysTruePred = () => () => true;

const fullPredicate = pattern => timingNode => {
    const regex = getRegex(pattern);
    return pattern.length === 0 || regex.test(timingNode.distName) || regex.test(timingNode.getDecodedParameters().procedureName);
};

const getTimingsByDistNamePred = distName => timingNode =>
    distName.length === 0 || getRegex(distName).test(timingNode.distName);

const getTimingsByProcedureName = procedureName => timingNode =>
    procedureName.length === 0 || getRegex(procedureName).test(timingNode.getDecodedParameters().procedureName);

module.exports = {
    findTimings,
    fullPredicate,
    getTimingsByProcedureName,
    getTimingsByDistNamePred,
    alwaysTruePred
};
