const {findTimings, fullPredicate} = require('./timingFinder');
const events = require('../events');

function newInstance(updateService) {
    const event = events.newEvent();
    function addListener(listener) {
        listener.listenerObj = {
            completeImTree(imTree) {
                listener(imTree);
            }
        };
        event.watch(listener);
        updateService.addImTreeListener(listener.listenerObj);
    }
    function removeListener(listener) {
        event.unwatch(listener);
        updateService.removeImTreeListener(listener.listenerObj);
    }
    function findTimingsForPattern(pattern, imTree) {
        return findTimings(fullPredicate(pattern), imTree);
    }
    return {
        addListener,
        removeListener,
        findTimingsForPattern
    };
}

module.exports = {
    newInstance
};
