const _ = require('lodash');
const {findTimings, alwaysTruePred, getTimingsByProcedureName, getTimingsByDistNamePred} = require('./timingFinder');

function newInstance(onlineImTree) {
    return onlineImTree.getImTree().then((imTree) => {
        function findAllTimings() {
            return findTimings(alwaysTruePred(), imTree);
        }
        function findTimingsForProcedureName(procedureName) {
            return findTimings(getTimingsByProcedureName(procedureName), imTree);
        }
        function findTimingsForDistName(distName) {
            return findTimings(getTimingsByDistNamePred(distName), imTree);
        }
        function getAllProcedureNames() {
            return _.uniq(_.pluck(findAllTimings(), 'details.procedureName'));
        }
        return {
            findAllTimings,
            findTimingsForProcedureName,
            findTimingsForDistName,
            getAllProcedureNames
        };
    });
}

module.exports = {
    newInstance
};
