const Unzip = require('zlibjs/bin/unzip.min').Zlib.Unzip;

const TextDecoder = this.TextDecoder;

module.exports = {
    newInstance: newInstance
};

function newInstance(arrayBuffer) {
    const unzip = new Unzip(new Uint8Array(arrayBuffer));

    return {
        unzipAll,
        unzipByFileName,
        getFileNames
    };

    /**
     * onEachFile gets fileContent as Uint8Array or String(default)
     * @param onEachFile callback function (filename, fileContent, fileIndex, totalFiles)
     * @param onDone callback function invoked when unzip is done
     * @param opt Options fileContentAsUint8=[true|false]
     */
    function unzipAll(onEachFile, onDone, opt = {}) {
        var fileNames = getFileNames();
        fileNames.forEach(unzipFile);
        if (onDone) onDone();

        function unzipFile(filename, index) {
            const uint8 = unzip.decompress(filename);
            var fileContent = uint8;
            if (!opt.fileContentAsUint8) fileContent = decodeToString(uint8);

            onEachFile(filename, fileContent, index + 1, fileNames.length);
        }
    }

    function getFileNames() {
        return unzip.getFilenames() || [];
    }

    /**
     * returns Uint8Array or String(default)
     * @param filename filename to unzip
     * @param opt Options fileContentAsUint8=[true|false]
     */
    function unzipByFileName(filename, opt = {}) {
        const uint8 = unzip.decompress(filename);
        if (opt.fileContentAsUint8) return uint8;

        return decodeToString(uint8);
    }
}

function decodeToString(uint8) {
    if (TextDecoder) return new TextDecoder().decode(uint8);

    var array = [];
    for (var i = 0; i < uint8.length; i++) {
        array.push(String.fromCharCode(uint8[i]));
    }
    return array.join('');
}
