const events = require('../events');
const imCompatibilityService = require('./imCompatibilityService');
const _ = require('lodash');

module.exports = {newInstance};

function newInstance(updateService) {
    let btsInfo = null;
    const event = createEvent();
    const imTreeListener = createImTreeListener();

    return {
        watch
    };

    function watch(watcher) {
        const unwatcher = event.watch(watcher);
        if (btsInfo !== null) watcher(btsInfo);
        return unwatcher;
    }

    function createEvent() {
        const newEvent = events.newEvent();
        newEvent.onFirstAdded(() => {
            updateService.addImTreeListener(imTreeListener);
        });
        newEvent.onLastRemoved(() => {
            resetBtsInfo();
            updateService.removeImTreeListener(imTreeListener);
        });
        return newEvent;
    }

    function createImTreeListener() {
        return {
            completeImTree: function (tree) {
                const newBtsInfo = {
                    id: getId(tree),
                    state: getState(tree)
                };
                if (_.isEqual(btsInfo, newBtsInfo)) return;

                btsInfo = newBtsInfo;
                event.notify(btsInfo);
            }
        };
    }

    function getId(tree) {
        const aNode = imCompatibilityService.findMrbtsANode(tree);

        if (!aNode) return null;
        return aNode.getDecodedParameters().mrbtsId;
    }

    function getState(tree) {
        const lnbtsNode = imCompatibilityService.findLnbtsNode(tree);
        if (!lnbtsNode)  return null;

        return lnbtsNode.getDecodedParameters().stateInfo.proceduralState;
    }

    function resetBtsInfo() {
        btsInfo = null;
    }
}
