const logger = require('../logger.impl');
const moment = require('moment/moment');
const imCompatibilityService = require('./imCompatibilityService');
const log = logger.getLogger('admin.core.utils.fileNameFactory');

module.exports = Object.freeze({
    createName: createName
});

function createName(imtree, prefix, extension, predefinedDate = Date.now()) {
    const btsId = getBtsId();
    const swName = getSwName();
    const btsName = getBtsName();
    const date = getDate();

    return (prefix ? prefix : '') +
        (btsId ? ((prefix ? '_' : '') + ('MRBTS-' + btsId)) : '') +
        (swName ? ('_' + swName) : '') +
        (btsName ? ('_' + btsName) : '') +
        '_' + date +
        '.' + extension;

    function getBtsId() {
        if (!imtree) return null;
        const mrbtsA = imCompatibilityService.findMrbtsANode(imtree);
        if (!mrbtsA) {
            log.warn('MRBTS_A doesn\'t exist!');
            return null;
        }
        return mrbtsA.getDecodedParameters().mrbtsId;
    }

    function getSwName() {
        if (!imtree) return null;
        const btsConf = imCompatibilityService.findBtsConfNode(imtree);
        if (!btsConf) {
            log.warn('BTS_CONF doesn\'t exist!');
            return null;
        }
        return btsConf.getDecodedParameters().activeSoftwareVersion;
    }

    function getBtsName() {
        if (!imtree) return null;

        const lnbtsA = imCompatibilityService.findLnbtsANode(imtree);
        if (!lnbtsA) {
            log.warn('LNBTS_A doesn\'t exist!');
            return null;
        }
        return lnbtsA.getDecodedParameters().btsName;
    }

    function getDate() {
        return moment(predefinedDate).format('YYYYMMDD-HHmm');
    }
}


