const events = require('../events');

module.exports = entriesValidator;

function entriesValidator(updateService) {
    const timestampDecreasingEvent = events.newEvent();
    const validations = {
        timestampDecreasing: []
    };

    startWatchingEntries();

    return {
        watchOnTimestampDecreasing
    };

    function watchOnTimestampDecreasing(watcher) {
        validations.timestampDecreasing.forEach((entryId) => watcher(newTimestampDecreasingNotification(entryId)));
        return timestampDecreasingEvent.watch(watcher);
    }

    function startWatchingEntries() {
        updateService.addHistoryListener({
            newEntry: validateEntry,
            newEntries: entries => entries.forEach(entry => validateEntry(entry)),
            clearEntries: clearTimestampDecreasing
        });
    }

    function clearTimestampDecreasing() {
        validations.timestampDecreasing = [];
        timestampDecreasingEvent.notify(clearTimestampDecreasingNotification());
    }

    function validateEntry(entry) {
        if (entry.id === 0) return;

        var entryBefore = updateService.getHistoryEntry(entry.id - 1);
        if (entryBefore.timestamp > entry.timestamp) {
            validations.timestampDecreasing.push(entry.id);
            timestampDecreasingEvent.notify(newTimestampDecreasingNotification(entry.id));
        }
    }

    function newTimestampDecreasingNotification(entryId) {
        return {
            type: 'new',
            entryId
        };
    }

    function clearTimestampDecreasingNotification() {
        return {
            type: 'clear'
        };
    }

}
