const Promise = require('bluebird');
const httpConstants = require('./httpConstants.js');
const loggers = require('./logger.impl');
const log = loggers.getLogger('admin.core.WebDav');

const ALLPROPS_XML_REQUEST = '<?xml version="1.0" encoding="utf-8" ?><D:propfind xmlns:D="DAV:"><D:prop/></D:propfind>';

module.exports = createWebDav;

function createWebDav(XMLHttpRequest){
    return {
        propfind: propfind
    };

    /*
     url: shared file systems resource
     depth: depth of file structure (
        "0" - only current directory path (without files)
        "1" - current directory path and files within it
        "infinite" - all files in all child directories should be read starting from url point directory
     )
     returns: promise of webdav request
    */
    function propfind(url, depth) {
        if (!XMLHttpRequest) {
            log.error('No XMLHttpRequest provider has been specified!');
        }

        return new Promise(function (resolve, reject) {
            var request = new XMLHttpRequest();

            request.open('PROPFIND', url, true);
            request.setRequestHeader('Depth', depth || 1);
            request.setRequestHeader('Content-Type', 'application/xml');
            request.send(ALLPROPS_XML_REQUEST);

            request.onreadystatechange = function () {
                if (request.readyState !== httpConstants.XML_REQUEST_READY_STATE.COMPLETE) return;

                if (request.status === httpConstants.HTTP_STATUS.OK || request.status === httpConstants.HTTP_STATUS.MULTI_STATUS) {
                    resolve(request.responseText);
                } else {
                    reject(request.responseText);
                }
            };
        });
    }
};
