const DOMParser = require('xmldom').DOMParser;

const Node = {
    ELEMENT_NODE: 1,
    ATTRIBUTE_NODE: 2,
    TEXT_NODE: 3,
    CDATA_SECTION_NODE: 4,
    ENTITY_REFERENCE_NODE: 5,
    ENTITY_NODE: 6,
    PROCESSING_INSTRUCTION_NODE: 7,
    COMMENT_NODE: 8,
    DOCUMENT_NODE: 9,
    DOCUMENT_TYPE_NODE: 10,
    DOCUMENT_FRAGMENT_NODE: 11,
    NOTATION_NODE: 12
};

var SKIP = {};

var domParser = new DOMParser();

function parseNode(node) {
    switch (node.nodeType) {
        case Node.DOCUMENT_NODE:
            return parseDocumentNode(node);
        case Node.ELEMENT_NODE:
            return parseElementNode(node);
        case Node.ATTRIBUTE_NODE:
            return parseAttributeNode(node);
        case Node.TEXT_NODE:
            return parseTextNode(node);
        case Node.COMMENT_NODE:
            return parseCommentNode(node);
        default:
            throw new Error("Unsupported node type! " + xml.nodeType);
    }
}

function parseCommentNode(node) {
    return ['skip', SKIP];
}

function parseAttributeNode(node) {
    return ['@' + node.nodeName, node.nodeValue || ''];
}

function parseTextNode(node) {
    return ['#text', node.nodeValue || ''];
}

function parseChildNodes(childNodes) {
    var map = {};
    for (var i = 0, length = childNodes.length; i < length; i++) {
        var childNode = parseNode(childNodes.item(i));
        if (childNode[1] !== SKIP) {
            map[childNode[0]] = map[childNode[0]] || [];
            map[childNode[0]].push(childNode[1]);
        }
    }

    var parsedNodes = [];
    for (var key in map) {
        var values = map[key];
        if (values.length === 1) {
            parsedNodes.push([key, values[0]]);
        } else /* is array */ {
            parsedNodes.push([key, values]);
        }
    }
    return parsedNodes;
}

function parseElementNode(node) {
    var content = null;

    var attributes = node.attributes;
    for (var i = 0, length = attributes.length; i < length; i++) {
        content = content || {};
        var attributeNode = parseNode(attributes.item(i));
        content[attributeNode[0]] = attributeNode[1];
    }

    var parsedNodes = parseChildNodes(node.childNodes);
    if (!content && parsedNodes.length === 1 && parsedNodes[0][0] === '#text') {
        content = parsedNodes[0][1];
    } else if (parsedNodes.length > 0) {
        content = content || {};
        for (var i = 0; i < parsedNodes.length; i++) {
            var currentNode = parsedNodes[i];
            content[currentNode[0]] = currentNode[1];
        }
    }

    return [node.nodeName, content];
}

function parseDocumentNode(node) {
    var parsedNode = parseNode(node.documentElement);
    var obj = {};
    obj[parsedNode[0]] = parsedNode[1];
    return obj;
}

function toJson(xml) {
    if (typeof xml === 'string') {
        var dom = domParser.parseFromString(xml, "text/xml");
        return parseNode(dom);
    } else {
        return parseNode(xml);
    }
}

module.exports = {
    toJson: toJson
};
