module.exports = {
    HEADER_MAGIC: [-3, 55, 122, 88, 90, 0],
    BACKWARD_SIZE_MAX: 17179869184,
    STREAM_HEADER_SIZE: 12,
    BLOCK_HEADER_SIZE_MAX: 1024,
    VLI_SIZE_MAX: 9,

    decodeStreamHeader: function (buf) {
        for (var i = 0; i < this.HEADER_MAGIC.length; ++i) {
            if (buf[i] != this.HEADER_MAGIC[i])
                throw 'XZFormatException';
        }

        // TODO check crc32

        try {
            return this.decodeStreamFlags(buf, this.HEADER_MAGIC.length);
        } catch (e) {
            throw 'Unsupported options in XZ Stream Header';
        }
    },

    decodeStreamFlags: function (buf, off) {
        if (buf[off] != 0x00 || (buf[off + 1] & 0xFF) >= 0x10)
            throw 'UnsupportedOptionsException';

        return {
            checkType: buf[off + 1],
            backwardSize: -1
        };
    },

    decodeVLI: function (input) {
        var b = input.read();
        if (b === -1)
            throw 'EOFException';

        var num = b & 0x7F;
        var i = 0;

        while ((b & 0x80) != 0x00) {
            if (++i >= this.VLI_SIZE_MAX)
                throw 'CorruptedInputException';

            b = input.read();
            if (b === -1)
                throw 'EOFException';

            if (b === 0x00)
                throw 'CorruptedInputException';

            num |= (b & 0x7F) << (i * 7);
        }

        return num;
    },

    getVLISize: function (num) {
        var size = 0;
        do {
            ++size;
            num >>= 7;
        } while (num != 0);

        return size;
    },

    arraycopy: function (src, srcPos, dest, destPos, length) {
        for (var i = srcPos; i < (srcPos + length); ++i) {
            if (i >= destPos)
                dest[i - srcPos + destPos] = src[i];
        }
    },

    create2DArray: function (row, col) {
        var arr = new Array(row);
        for (var i = 0; i < row; ++i)
            arr[i] = new Array(col);
        return arr;
    }
};
