const DecoderUtil = require('./decoderUtil');

module.exports = IndexBase;

function IndexBase() {
    this.blocksSum = 0;
    this.uncompressedSum = 0;
    this.indexListSize = 0;
    this.recordCount = 0;
}

IndexBase.prototype.getUnpaddedIndexSize = function () {
    return 1 + DecoderUtil.getVLISize(this.recordCount) + this.indexListSize + 4;
};

IndexBase.prototype.getIndexSize = function () {
    return (this.getUnpaddedIndexSize() + 3) & ~3;
};

IndexBase.prototype.getStreamSize = function () {
    return DecoderUtil.STREAM_HEADER_SIZE + this.blocksSum +
        this.getIndexSize() + DecoderUtil.STREAM_HEADER_SIZE;
};

IndexBase.prototype.getIndexPaddingSize = function () {
    return (4 - this.getUnpaddedIndexSize()) & 3;
};

IndexBase.prototype.add = function (unpaddedSize, uncompressedSize) {
    this.blocksSum += (unpaddedSize + 3) & ~3;
    this.uncompressedSum += uncompressedSize;
    this.indexListSize += DecoderUtil.getVLISize(unpaddedSize) + DecoderUtil.getVLISize(uncompressedSize);
    ++this.recordCount;

    if (this.blocksSum < 0 || this.uncompressedSum < 0
        || this.getIndexSize() > DecoderUtil.BACKWARD_SIZE_MAX || this.getStreamSize() < 0)
        throw 'invalidIndexException';
};
