const DecoderUtil = require('./decoderUtil');
const RangeCoder = require('./rangeCoder');
const State = require('./state');

module.exports = {
    LZMACoder: LZMACoder,
    LiteralCoder: LiteralCoder,
    LiteralSubcoder: LiteralSubcoder,
    LengthCoder: LengthCoder
};

function LZMACoder(pb) {
    this.POS_STATES_MAX = 1 << 4;

    this.MATCH_LEN_MIN = 2;

    this.DIST_STATES = 4;
    this.DIST_SLOTS = 1 << 6;
    this.DIST_MODEL_START = 4;
    this.DIST_MODEL_END = 14;
    this.FULL_DISTANCES = 1 << (this.DIST_MODEL_END / 2);

    this.ALIGN_BITS = 4;
    this.ALIGN_SIZE = 1 << this.ALIGN_BITS;
    this.ALIGN_MASK = this.ALIGN_SIZE - 1;

    this.REPS = 4;

    this.posMask = (1 << pb) - 1;

    this.reps = [];
    this.state = new State();

    this.isMatch = DecoderUtil.create2DArray(this.state.STATES, this.POS_STATES_MAX);
    this.isRep = new Array(this.state.STATES);
    this.isRep0 = new Array(this.state.STATES);
    this.isRep1 = new Array(this.state.STATES);
    this.isRep2 = new Array(this.state.STATES);
    this.isRep0Long = DecoderUtil.create2DArray(this.state.STATES, this.POS_STATES_MAX);
    this.distSlots = DecoderUtil.create2DArray(this.DIST_STATES, this.DIST_SLOTS);
    this.distSpecial = [
        new Array(2), new Array(2),
        new Array(4), new Array(4),
        new Array(8), new Array(8),
        new Array(16), new Array(16),
        new Array(32), new Array(32)
    ];
    this.distAlign = new Array(this.ALIGN_SIZE);

    this.rangeCoder = new RangeCoder();
}

LZMACoder.prototype.getDistState = function (len) {
    return len < this.DIST_STATES + this.MATCH_LEN_MIN
        ? len - this.MATCH_LEN_MIN
        : this.DIST_STATES - 1;
};

LZMACoder.prototype.reset = function () {
    this.reps[0] = 0;
    this.reps[1] = 0;
    this.reps[2] = 0;
    this.reps[3] = 0;
    this.state.reset();

    for (var i = 0; i < this.isMatch.length; ++i)
        this.rangeCoder.initProbs(this.isMatch[i]);

    this.rangeCoder.initProbs(this.isRep);
    this.rangeCoder.initProbs(this.isRep0);
    this.rangeCoder.initProbs(this.isRep1);
    this.rangeCoder.initProbs(this.isRep2);

    for (var i = 0; i < this.isRep0Long.length; ++i)
        this.rangeCoder.initProbs(this.isRep0Long[i]);

    for (var i = 0; i < this.distSlots.length; ++i)
        this.rangeCoder.initProbs(this.distSlots[i]);

    for (var i = 0; i < this.distSpecial.length; ++i)
        this.rangeCoder.initProbs(this.distSpecial[i]);

    this.rangeCoder.initProbs(this.distAlign);
};

function LiteralCoder(lc, lp) {
    this.lc = lc;
    this.literalPosMask = (1 << lp) - 1;
}

LiteralCoder.prototype.getSubcoderIndex = function (prevByte, pos) {
    var low = prevByte >> (8 - this.lc);
    var high = (pos & this.literalPosMask) << this.lc;
    return low + high;
};

function LiteralSubcoder() {
    this.probs = new Array(0x300);
}

LiteralSubcoder.prototype.reset = function () {
    new RangeCoder().initProbs(this.probs);
};

function LengthCoder() {
    // copied from LZMACoder
    this.POS_STATES_MAX = 1 << 4;

    this.LOW_SYMBOLS = 1 << 3;
    this.MID_SYMBOLS = 1 << 3;
    this.HIGH_SYMBOLS = 1 << 8;

    this.choice = new Array(2);
    this.low = DecoderUtil.create2DArray(this.POS_STATES_MAX, this.LOW_SYMBOLS);
    this.mid = DecoderUtil.create2DArray(this.POS_STATES_MAX, this.MID_SYMBOLS);
    this.high = new Array(this.HIGH_SYMBOLS);

    this.rangeCoder = new RangeCoder();
}

LengthCoder.prototype.reset = function () {
    this.rangeCoder.initProbs(this.choice);

    for (var i = 0; i < this.low.length; ++i)
        this.rangeCoder.initProbs(this.low[i]);

    for (var i = 0; i < this.low.length; ++i)
        this.rangeCoder.initProbs(this.mid[i]);

    this.rangeCoder.initProbs(this.high);
};
