const RangeCoder = require('./rangeCoder');

module.exports = RangeDecoder;

function RangeDecoder() {
    RangeCoder.call(this);
    this.range = 0;
    this.code = 0;
}

RangeDecoder.prototype = Object.create(RangeCoder.prototype);
RangeDecoder.prototype.constructor = RangeDecoder;

RangeDecoder.prototype.decodeBit = function (probs, index) {
    this.normalize();

    var prob = probs[index];
    var bound = (this.range >>> this.BIT_MODEL_TOTAL_BITS) * prob;
    var bit;

    if ((this.code ^ 0x80000000) < (bound ^ 0x80000000)) {
        this.range = bound;
        probs[index] = prob + ((this.BIT_MODEL_TOTAL - prob) >>> this.MOVE_BITS);
        bit = 0;
    } else {
        this.range -= bound;
        this.code -= bound;
        probs[index] = prob - (prob >>> this.MOVE_BITS);
        bit = 1;
    }

    // JavaScript uses 64-bit floats, so it is necessary to convert it
    // to 32-bit unsigned int to make the above algorithm work
    this.code >>>= 0;

    return bit;
};

RangeDecoder.prototype.decodeBitTree = function (probs) {
    var symbol = 1;

    do {
        symbol = (symbol << 1) | this.decodeBit(probs, symbol);
    } while (symbol < probs.length);

    return symbol - probs.length;
};

RangeDecoder.prototype.decodeReverseBitTree = function (probs) {
    var symbol = 1,
        i = 0,
        result = 0;

    do {
        var bit = this.decodeBit(probs, symbol);
        symbol = (symbol << 1) | bit;
        result |= bit << i++;
    } while (symbol < probs.length);

    return result;
};

RangeDecoder.prototype.decodeDirectBits = function (count) {
    var result = 0;
    do {
        this.normalize();

        this.range >>>= 1;
        var t = (this.code - this.range) >>> 31;
        this.code -= this.range & (t - 1);
        result = (result << 1) | (1 - t);
    } while (--count != 0);

    return result;
};
