const RangeDecoder = require('./rangeDecoder');

module.exports = RangeDecoderFromBuffer;

function RangeDecoderFromBuffer(inputSizeMax) {
    RangeDecoder.call(this);

    this.INIT_SIZE = 5;
    this.buf = new Array(inputSizeMax - this.INIT_SIZE);
    this.pos = 0;
    this.end = 0;
}

RangeDecoderFromBuffer.prototype = Object.create(RangeDecoder.prototype);
RangeDecoderFromBuffer.prototype.constructor = RangeDecoderFromBuffer;

RangeDecoderFromBuffer.prototype.prepareInputBuffer = function (dataInputStream, len) {
    if (len < this.INIT_SIZE)
        throw 'CorruptedInputException';

    if (dataInputStream.readUByte() != 0x00)
        throw 'CorruptedInputException';

    this.code = dataInputStream.readInt();
    this.range = 0xFFFFFFFF;

    this.pos = 0;
    this.end = len - this.INIT_SIZE;
    dataInputStream.readNTo(this.buf, 0, this.end);
};

RangeDecoderFromBuffer.prototype.isInBufferOK = function () {
    return this.pos <= this.end;
};

RangeDecoderFromBuffer.prototype.isFinished = function () {
    return this.pos === this.end && this.code === 0;
};

RangeDecoderFromBuffer.prototype.normalize = function () {
    if ((this.range & this.TOP_MASK) === 0) {
        try {
            this.code = (this.code << this.SHIFT_BITS) | (this.buf[this.pos++] & 0xFF);
            this.range <<= this.SHIFT_BITS;
        } catch (e) {
            throw 'CorruptedInputException';
        }
    }
};
