const IndexHash = require('./indexHash');
const DataInputStream = require('./dataInputStream');
const DecoderUtil = require('./decoderUtil');
const Check = require('./check');
const BlockInputStream = require('./blockInputStream');

module.exports = SingleXZInputStream;

function SingleXZInputStream(inputStream, streamHeader) {
    this.inputStream = inputStream;
    this.blockDecoder = null;
    this.endReached = false;
    this.indexHash = new IndexHash();

    if (!streamHeader) {
        streamHeader = new DataInputStream(inputStream).readN(DecoderUtil.STREAM_HEADER_SIZE);
    }
    this.streamHeaderFlags = DecoderUtil.decodeStreamHeader(streamHeader);
    this.check = Check.getInstance(this.streamHeaderFlags.checkType);
    this.exception = null;
}

SingleXZInputStream.prototype.readNTo = function (buf, off, len) {
    if (off < 0 || len < 0 || off + len < 0)
        throw 'IndexOutOfBoundsException';

    if (len === 0)
        return 0;

    if (this.exception != null)
        throw this.exception;

    if (this.endReached)
        return -1;

    var size = 0;

    try {
        while (len > 0) {
            if (this.blockDecoder === null) {
                try {
                    this.blockDecoder = new BlockInputStream(this.inputStream, this.check, null, -1, -1);
                } catch (ex) {
                    this.indexHash.validate(this.inputStream);
                    this.validateStreamFooter();
                    this.endReached = true;
                    return size > 0 ? size : -1;
                }
            }

            var ret = this.blockDecoder.readNTo(buf, off, len);
            if (ret > 0) {
                size += ret;
                off += ret;
                len -= ret;
            } else if (ret === -1) {
                this.indexHash.add(this.blockDecoder.getUnpaddedSize(),
                    this.blockDecoder.getUncompressedSize());
                this.blockDecoder = null;
            }
        }
    } catch (e) {
        this.exception = e;
        if (size === 0)
            throw e;
    }

    return size;
};

SingleXZInputStream.prototype.validateStreamFooter = function () {
    // TODO no validation, only skips bytes in the stream
    new DataInputStream(this.inputStream).readN(DecoderUtil.STREAM_HEADER_SIZE);
};
