const XZInputStream = require('./xzInputStream');

module.exports = {
    decode: decode
};

var shouldCancelReading = false;

function decode(arrayBuffer, progressListener) {
    shouldCancelReading = false;
    const PART_SIZE = 8192;
    var data = [];
    var tempBuf = [];

    var xzInputStream = new XZInputStream(arrayBuffer);
    var stream = xzInputStream.getInputStream();

    var part = 0;

    decodeNextPart();

    return function () {
        shouldCancelReading = true;
    };

    function decodeNextPart() {
        if (shouldCancelReading) return;

        var size = xzInputStream.readNTo(tempBuf, 0, PART_SIZE);

        if (size !== -1) {
            progressListener.onReadBytes(stream.getReadBytesCount(), stream.getTotalBytesCount());
            storeNextPart();
            callDecodingNextPart();
        } else {
            notifyWithArrayBuffer();
        }
    }

    function storeNextPart() {
        data.push(new Int8Array(tempBuf));
        tempBuf = [];
        ++part;
    }

    function notifyWithArrayBuffer() {
        var fileReader = new FileReader();
        fileReader.onload = function() {
            progressListener.onComplete(this.result);
        };
        fileReader.readAsArrayBuffer(new Blob(data));
    }

    function callDecodingNextPart() {
        // setTimeout is very slow, so don't do it every time,
        // just enough times to eliminate lags
        if (part % 100 === 0) {
            setTimeout(decodeNextPart);
        } else {
            decodeNextPart();
        }
    }
}
