const InputStream = require('./inputStream');
const SingleXZInputStream = require('./singleXZInputStream');
const DataInputStream = require('./dataInputStream');
const DecoderUtil = require('./decoderUtil');

module.exports = XZInputStream;

function XZInputStream(arrayBuffer) {
    this.inputStream = new InputStream(arrayBuffer);
    this.xzIn = new SingleXZInputStream(this.inputStream);
    this.exception = null;
}

XZInputStream.prototype.getInputStream = function () {
    return this.inputStream;
};

XZInputStream.prototype.readNTo = function (buf, off, len) {
    if (off < 0 || len < 0 || off + len < 0)
        throw 'IndexOutOfBoundsException';

    if (len == 0)
        return 0;

    if (this.exception != null)
        throw this.exception;

    if (this.endReached)
        return -1;

    var size = 0;

    try {
        while (len > 0) {
            if (this.xzIn === null) {
                this.prepareNextStream();
                if (this.endReached) {
                    return size === 0 ? -1 : size;
                }
            }

            var ret = this.xzIn.readNTo(buf, off, len);
            if (ret > 0) {
                size += ret;
                off += ret;
                len -= ret;
            } else if (ret === -1) {
                this.xzIn = null;
            }
        }
    } catch (e) {
        this.exception = e;
        if (size === 0)
            throw e;
    }

    return size;
};

XZInputStream.prototype.prepareNextStream = function () {
    var inData = new DataInputStream(this.inputStream);
    var buf = new Array(DecoderUtil.STREAM_HEADER_SIZE);

    do {
        var ret = inData.readNTo(buf, 0, 1);
        if (ret === -1) {
            this.endReached = true;
            return;
        }

        inData.readNTo(buf, 1, 3);

    } while (buf[0] === 0 && buf[1] === 0 && buf[2] === 0 && buf[3] === 0);

    inData.readNTo(buf, 4, DecoderUtil.STREAM_HEADER_SIZE - 4);

    try {
        xzIn = new SingleXZInputStream(this.inputStream, buf);
    } catch (e) {
        throw 'Garbage after a valid XZ Stream';
    }
};
