const validationLookup = require('../../src/adjacency/adjacencyValidationLookup.js');

describe('Factory: validationLookup', () => {

    it('should find lncel', () => {
        // given
        const lncel = {distName: '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-1'};
        const validationData = createFakeValidationData();
        // when
        const foundLncel = validationLookup.findLncel(lncel, validationData);
        // then
        assert.isDefined(foundLncel);
        assert.equal(lncel.distName, foundLncel.localCell.distName);
    });

    it('should not find lncel', () => {
        // given
        const lncel = {distName: '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-999'};
        const validationData = createFakeValidationData();
        // when
        const foundLncel = validationLookup.findLncel(lncel, validationData);
        // then
        assert.isNull(foundLncel);
    });

    it('should find lnadjl for given lnrel', () => {
        // given
        const lnrel = {distName: '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-1/LNREL-1'};
        const technologyData = createFakeTechnologyData();
        // when
        const foundLnadjl = validationLookup.findCorrectLnadj(lnrel, technologyData);
        // then
        assert.isDefined(foundLnadjl);
        assert.equal(foundLnadjl.distName, '/MRBTS-1/RAT-1/LNBTS-1/CPLANE-1/LNADJ-1/LNADJL-1');
    });

    it('should not find lnadjl for given lnrel', () => {
        // given
        const lnrel = {distName: '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-1/LNREL-2'};
        const technologyData = createFakeTechnologyData();
        // when
        const foundLnadjl = validationLookup.findCorrectLnadj(lnrel, technologyData);
        // then
        assert.isNull(foundLnadjl);
    });

    it('should find lnrel for given lnadjl', () => {
        // given
        const lnadjl = {distName: '/MRBTS-1/RAT-1/LNBTS-1/CPLANE-1/LNADJ-1/LNADJL-1'};
        const technologyData = createFakeTechnologyData();
        // when
        const foundLnrel = validationLookup.findCorrectLnrel(lnadjl, technologyData);
        // then
        assert.isDefined(foundLnrel);
        assert.equal(foundLnrel.distName, '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-1/LNREL-1');
    });

    it('should not find lnrel for given lnadjl', () => {
        // given
        const lnadjl = {distName: '/MRBTS-1/RAT-1/LNBTS-1/CPLANE-1/LNADJ-1/LNADJL-2'};
        const technologyData = createFakeTechnologyData();
        // when
        const foundLnrel = validationLookup.findCorrectLnrel(lnadjl, technologyData);
        // then
        assert.isNull(foundLnrel);
    });

    function createFakeTechnologyData() {
        return {
            relationStatuses: {
                corrects: [
                    {
                        distName: '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-1/LNREL-1',
                        matchingCell: '/MRBTS-1/RAT-1/LNBTS-1/CPLANE-1/LNADJ-1/LNADJL-1'
                    }
                ]
            },
            cellStatuses: {
                corrects: [
                    {
                        distName: '/MRBTS-1/RAT-1/LNBTS-1/CPLANE-1/LNADJ-1/LNADJL-1'
                    }
                ]
            }
        };
    }

    function createFakeValidationData() {
        return {
            localCellValidations: [
                {
                    localCell: {
                        distName: '/MRBTS-1/RAT-1/LNBTS-1/LNCEL-1'
                    }
                }
            ]
        };
    }
});
