const connectionInitializerFactory = require('../../src/adminApi/connectionInitializer');

describe('ADMIN API: connectionInitializer', function () {
    var connectionInitializer;

    beforeEach(function () {
        connectionInitializer = connectionInitializerFactory();
    });

    it('should invoke steps in order', function (done) {
        // given
        const step1 = sinon.spy(function () {
            return new Promise(function(resolve) {
                setTimeout(function () {
                    resolve();
                }, 100);
            });
        });
        const step2 = sinon.spy(function () {
            return new Promise(function(resolve) {
                resolve();
            });
        });
        connectionInitializer.addNewStep(step1, []);
        connectionInitializer.addNewStep(step2, ['arg1', 'arg2']);
        // when
        connectionInitializer.prepare().then(function () {
            // then
            assert(step1.calledOnce);
            assert(step2.withArgs('arg1', 'arg2').calledOnce);
            assert(step1.calledBefore(step2));
        }).then(done).catch(done);
    });

    it('should catch error and clean steps', function(done) {
        // given
        const step1 = sinon.spy(function () {
            return new Promise(function(resolve, reject) {
                setTimeout(function () {
                    reject();
                }, 100);
            });
        });
        const step2 = sinon.spy(function () {
            return new Promise(function(resolve) {
                resolve();
            });
        });
        connectionInitializer.addNewStep(step1, []);
        connectionInitializer.addNewStep(step2, ['arg1', 'arg2']);
        // when
        connectionInitializer.prepare().catch(function () {
            // then
            assert(step1.calledOnce);
            assert.equal(step2.callCount, 0);
            assert.lengthOf(connectionInitializer.getSteps(), 0);
        }).then(done).catch(done);
    });
});
