const connectionInitializerStepsFactory = require('../../src/adminApi/connectionInitializerSteps');

describe('ADMIN API: connectionInitializerSteps', function () {
    var connectionInitializerSteps, onlineImTreeMock, connectionServiceMock;
    beforeEach(function () {
        onlineImTreeMock = {
            getImTree: sinon.stub()
        };

        connectionServiceMock = {
            addListener: sinon.spy(),
            removeListener: sinon.spy(),
            isSignedIn: sinon.stub()
        };

        connectionInitializerSteps = connectionInitializerStepsFactory(onlineImTreeMock, connectionServiceMock);
    });

    it('should resolve instantly - already connected to bts', function (done) {
        // given
        connectionServiceMock.isSignedIn.returns(true);
        // when
        var promise = connectionInitializerSteps.connectToBts();
        // then
        promise.then(function () {
            assert.equal(connectionServiceMock.addListener.callCount, 0);
        }).then(done).catch(done);
    });

    it('should resolve after signing in - sign in succeeded', function (done) {
        // given
        connectionServiceMock.addListener = sinon.spy(function (listener) {
            listener.signIn();
        });
        connectionServiceMock.isSignedIn.returns(false);
        // when
        var promise = connectionInitializerSteps.connectToBts();
        // then
        promise.then(function () {
            assert(connectionServiceMock.addListener.calledOnce);
        }).then(done).catch(done);
    });

    it('should reject after signing in - sign in failed', function (done) {
        // given
        connectionServiceMock.addListener = sinon.spy(function (listener) {
            listener.error('some error');
        });
        connectionServiceMock.isSignedIn.returns(false);
        // when
        var promise = connectionInitializerSteps.connectToBts();
        // then
        promise.catch(function () {
            assert(connectionServiceMock.addListener.calledOnce);
            done();
        });
    });

    it('should resolve if object with given distName exists', function(done) {
        // given
        const distName = '/dist-1/name-1';
        const imTree = {
            findByDistName: sinon.stub().returns(true)
        };
        onlineImTreeMock.getImTree.returns(Promise.resolve(imTree));
        // when
        var promise = connectionInitializerSteps.checkIfDnExists(distName);
        // then
        promise.then(function() {
            assert(imTree.findByDistName.withArgs(distName).calledOnce);
            assert(onlineImTreeMock.getImTree.calledOnce);
        }).then(done).catch(done);
    });

    it('should reject if object with given distName does not exist', function(done) {
        // given
        const distName = '/dist-1/name-1';
        const imTree = {
            findByDistName: sinon.stub().returns(false)
        };
        onlineImTreeMock.getImTree.returns(Promise.resolve(imTree));
        // when
        var promise = connectionInitializerSteps.checkIfDnExists(distName);
        // then
        promise.catch(function(reason) {
            assert.equal(reason, 'No object with distname: ' + distName);
            done();
        });
    });
});
