const ADMIN_API_STATUSES = require('../../../../src/adminApi/adminApiStatuses');
const adjacenciesFactory = require('../../../../src/adminApi/infoModel/adjacencies/adjacencies');
const adjacencyValidation = require('../../../../src/adjacency/adjacencyValidation');
const ImTree = require('../../../../src/imtree');
const Promise = require('bluebird');

describe('getCellsAction', () => {
    let imTree;
    let adjacencies;
    let connectToBtsSpy;
    let connectionInitializerMock;

    function createConnectionInitializerMock() {
        connectToBtsSpy = sinon.spy(() => ({prepare: () => Promise.resolve()}));
        return {
            connectToBts: connectToBtsSpy
        };
    }

    beforeEach(() => {
        imTree = new ImTree();
        connectionInitializerMock = createConnectionInitializerMock();
        adjacencies = adjacenciesFactory({
            onlineImTree: {
                getImTree: () => Promise.resolve(imTree)
            },
            adjacencyValidation: adjacencyValidation,
            connectionInitializer: connectionInitializerMock
        });
        sinon.stub(adjacencyValidation, 'validateAdjacencies', () => Object.create({
            localCellValidations:
                [{
                    localCell: {
                        className: 'LNCEL',
                        id: '303',
                        getDecodedParameters: () => Object.create({phyCellId: '203'})
                    }
                },
                {
                    localCell: {
                        className: 'LNCEL',
                        id: '302',
                        getDecodedParameters: () => Object.create({phyCellId: '202'})
                    }
                }]
        }));
    });

    afterEach(() => {
        adjacencyValidation.validateAdjacencies.restore();
    });

    it('should return validated lnbts cells', done => {

        // given
        const msgData = createMsgData('getCells');
        // when
        adjacencies.execute(msgData, (requestStatus, payload) => {
            assert.equal(requestStatus, ADMIN_API_STATUSES.COMPLETED);
            assert.equal(JSON.stringify(payload), JSON.stringify([{'LNCEL-303': 'PCI-203'}, {'LNCEL-302': 'PCI-202'}]));
            done();
        });
    });


    function createMsgData(procedureName) {
        return {
            name: 'adjacencies',
            parameters: {
                procedureName: procedureName
            }
        };
    }
});
