const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');
const infoModelDispatcherFactory = require('../../../src/adminApi/infoModel/infoModelDispatcher');
const testUtils = require('../adminApi.test.util');

describe('ADMIN API: infoModelDispatcher', () => {

    it('should not forward unknown procedure', done => {
        // given
        const msgData = createMsgData('UnknownOperation');
        const infoModelDispatcher = infoModelDispatcherFactory({});
        // when
        infoModelDispatcher.dispatch(msgData, (status, payload) => {
            // then
            assert.deepEqual(status, ADMIN_API_STATUSES.FAILED);
            assert.deepEqual(payload, `No "UnknownOperation" infoModel getter in testing API`);
            done();
        });
    });

    it('should fail when msgData doesn\'t contain required name parameter', done => {
        // given
        const msgData = {};
        const infoModelDispatcher = infoModelDispatcherFactory({});
        // when
        infoModelDispatcher.dispatch(msgData, (status, payload) => {
            // then
            assert.deepEqual(status, ADMIN_API_STATUSES.FAILED);
            assert.deepEqual(payload, 'Missing required property: name');
            done();
        });
    });

    it('should fail when for invalid type of name', done => {
        // given
        const msgData = createMsgData(2);
        const infoModelDispatcher = infoModelDispatcherFactory({});
        // when
        infoModelDispatcher.dispatch(msgData, (status, payload) => {
            // then
            assert.deepEqual(status, ADMIN_API_STATUSES.FAILED);
            assert.deepEqual(payload, '/name, Invalid type: number (expected string)');
            done();
        });
    });

    it('should forward sfp', done => {
        //given
        const msgData = createMsgData('sfp');
        const infoModelDispatcher = infoModelDispatcherFactory({
            sfpMonitoring: {
                execute: sfpExecutor
            }
        });
        //when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function sfpExecutor() {
            // then
            done();
        }
    });

    it('should forward getParameters', (done) => {
        //given
        const msgData = createMsgData('getParameters');
        const infoModelDispatcher = infoModelDispatcherFactory({
            parametersGetter: {
                execute: executor
            }
        });
        //when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    it('should forward abnormalities', done => {
        //given
        const msgData = createMsgData('abnormalities');
        const infoModelDispatcher = infoModelDispatcherFactory({
            abnormalities: {
                execute: executor
            }
        });
        //when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    it('should forward lnbtsState', done => {
        //given
        const msgData = createMsgData('lnbtsState');
        const infoModelDispatcher = infoModelDispatcherFactory({
            lnbtsState: {
                execute: executor
            }
        });
        //when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    it('should forward adjacencies', done => {
        //given
        const msgData = createMsgData('adjacencies');
        const infoModelDispatcher = infoModelDispatcherFactory({
            adjacencies: {
                execute: executor
            }
        });
        //when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    it('should forward timings infoModel operation', done => {
        // given
        const msgData = createMsgData('timings');
        const infoModelDispatcher = infoModelDispatcherFactory({
            timings: {
                execute: executor
            }
        });
        // when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    it('should forward baseband infoModel operation', done => {
        // given
        const msgData = createMsgData('baseband');
        const infoModelDispatcher = infoModelDispatcherFactory({
            baseband: {
                execute: executor
            }
        });
        // when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    it('should forward stateInfo infoModel operation', done => {
        // given
        const msgData = createMsgData('stateInfo');
        const infoModelDispatcher = infoModelDispatcherFactory({
            stateInfo: {
                execute: executor
            }
        });
        // when
        infoModelDispatcher.dispatch(msgData, testUtils.noop);

        function executor() {
            // then
            done();
        }
    });

    function createMsgData(name) {
        return {
            name: name
        };
    }

});
