const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');
const ImTree = require('../../../src/imtree');
const lnbtsStateFactory = require('../../../src/adminApi/infoModel/lnbtsState');
const btsInfoServiceFactory = require('../../../src/utils/btsInfoService');
const Promise = require('bluebird');

describe('ADMIN API: lnbtsState', () => {

    let imTree,
        updateServiceStub,
        connectionInitializerMock,
        lnbtsState;

    beforeEach(() => {
        connectionInitializerMock = createConnectionInitializerMock();
        updateServiceStub = createUpdateServiceStub();
        imTree = new ImTree();
        lnbtsState = lnbtsStateFactory({
            connectionInitializer: connectionInitializerMock,
            btsInfoService: btsInfoServiceFactory.newInstance(updateServiceStub)
        });
    });

    it('should return lnbtsState value when proceduralState available in IM', done => {
        // given
        const msgData = createMsgData('lnbtsState');
        const completionListener = sinon.spy();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1', {stateInfo: {proceduralState: 'commissioned'}});
        updateServiceStub.pushData(imTree);

        // when
        lnbtsState.execute(msgData, completionListener)
            .then(() => {
                // then
                assert(connectionInitializerMock.connectToBts.calledOnce);
                assert(completionListener.withArgs(ADMIN_API_STATUSES.COMPLETED, 'commissioned').calledOnce);
            }).then(done).catch(done);
    });

    it('should fail if no proceduralState available in IM', done => {
        // given
        const completionListener = sinon.spy();
        updateServiceStub.pushData(imTree);
        const msgData = createMsgData('lnbtsState');

        // when
        lnbtsState.execute(msgData, completionListener)
            .then(() => {
                done(new Error('This promise should not be resolved'));
            })
            .catch(() => {
                // then
                assert(connectionInitializerMock.connectToBts.calledOnce);
                assert(completionListener.withArgs(ADMIN_API_STATUSES.FAILED, 'Unable to find lnbts in imTree').calledOnce);
                done();
            });
    });

    function createMsgData(procedureName) {
        return {
            name: procedureName
        };
    }

    function createUpdateServiceStub() {
        let imTreeListener = null;
        let data = null;

        return {
            addImTreeListener: listener => {
                imTreeListener = listener;
                if (data) imTreeListener.completeImTree(data);
            },
            removeImTreeListener: () => {
                imTreeListener = null;
            },
            pushData: tree => {
                data = tree;
                if (imTreeListener) imTreeListener.completeImTree(tree);
            }
        };
    }

    function createConnectionInitializerMock() {
        return {
            connectToBts: sinon.spy(() => {
                return {
                    prepare: () => Promise.resolve()
                };
            })
        };
    }

});
