const commissioningProcedureFactory = require('../../../src/adminApi/procedures/commissioningProcedure');
const testUtils = require('../adminApi.test.util');
const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');

describe('commissioningProcedure', () => {
    let suit;

    beforeEach(() => {
        suit = {};
        suit.commissioningExecutorMock = {
            executeDownloadPlan: sinon.spy(),
            executeActivatePlan: sinon.spy()
        };
        suit.completionListener = sinon.spy();
        suit.connectionService = {
            isSignedIn: () => true
        };
        suit.commissioningProcedure = commissioningProcedureFactory.newInstance({
            connectionService: suit.connectionService,
            commissioningExecutor: suit.commissioningExecutorMock,
            btsServiceProcedure: {registerListener: testUtils.noop}
        });
    });

    afterEach(() => (suit = null));
    describe('#executeDownloadPlan', () => {

        it('should execute DownloadPlan', (done) => {
            const CORRECT_MSG_DATA = {
                name: 'DownloadPlan',
                parameters: {
                    planId: '1',
                    btsId: '1',
                    scf: [1]
                }
            };
            suit.commissioningProcedure.executeDownloadPlan(CORRECT_MSG_DATA, testUtils.noop)
                .then(() => {
                    assert(suit.commissioningExecutorMock.executeDownloadPlan.calledOnce);
                }).then(done).catch(done);
        });

        it('should notify about missing planId during DownloadPlan', (done) => {
            const CORRECT_MSG_DATA = {
                name: 'DownloadPlan',
                parameters: {
                    btsId: '1',
                    scf: [1]
                }
            };
            suit.commissioningProcedure.executeDownloadPlan(CORRECT_MSG_DATA, suit.completionListener)
                .then(failAssert)
                .catch((reason) => {
                    assert.equal(reason, 'Invalid schema');
                    assert(suit.completionListener.withArgs(ADMIN_API_STATUSES.FAILED, '/parameters, Missing required property: planId').calledOnce);
                    done();
                });
        });
    });

    describe('#executeActivatePlan', () => {
        it('should execute ActivatePlan', (done) => {
            const msgData = {
                name: 'ActivatePlan',
                parameters: {
                    planId: '1'
                }
            };
            suit.commissioningProcedure.executeActivatePlan(msgData, testUtils.noop)
                .then(() => {
                    assert(suit.commissioningExecutorMock.executeActivatePlan.calledOnce);
                }).then(done).catch(done);
        });

        it('should notify about missing planId during ActivatePlan', (done) => {
            const CORRECT_MSG_DATA = {
                name: 'ActivatePlan',
                parameters: {}
            };
            suit.commissioningProcedure.executeActivatePlan(CORRECT_MSG_DATA, suit.completionListener)
                .then(failAssert)
                .catch((reason) => {
                    assert.equal(reason, 'Invalid schema');
                    assert(suit.completionListener.withArgs(ADMIN_API_STATUSES.FAILED, '/parameters, Missing required property: planId').calledOnce);
                    done();
                });
        });
    });

    function failAssert() {
        assert.isTrue(false);
    }
});
