const testUtils = require('../adminApi.test.util');
const procedureFactoryDefinition = require('../../../src/adminApi/procedures/defaultProcedureFactory');
const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');

describe('procedureFactory', () => {
    let suit;

    beforeEach(() => {
        suit = {};
        const connectionInitializer = testUtils.createConnectionInitializerMock();
        connectionInitializer.prepare.returns(Promise.resolve());
        suit.connectToBtsSpy = connectionInitializer.connectToBts;
        suit.procedureFactory = procedureFactoryDefinition.newInstance({
            connectionInitializer,
            btsServiceProcedure: {registerListener: testUtils.noop}
        });
        suit.SCHEMA = {
            type: 'object',
            id: 'procedure',
            properties: {
                name: {
                    type: 'string'
                }
            },
            required: ['name']
        };
        suit.VALID_MSG = {
            name: 'swRollback'
        };
        suit.INVALID_MSG = {};
        suit.procedureExecutor = sinon.spy(Promise.resolve);
        suit.completionListener = sinon.spy();

        suit.procedure = suit.procedureFactory.create(suit.SCHEMA, suit.procedureExecutor)
    });
    afterEach(() => (suit = null));

    it('should notify about incorrect schema', (done) => {
        suit.procedure(suit.INVALID_MSG, suit.completionListener)
            .then(failAssert)
            .catch((reason)=> {
                assert.equal(reason, 'Invalid schema');
                assert(suit.completionListener.withArgs(ADMIN_API_STATUSES.FAILED, 'Missing required property: name').calledOnce);
                done();
            });
    });

    it('should call executor', (done) => {
        suit.procedure(suit.VALID_MSG, suit.completionListener)
            .then(() => {
                assert(suit.connectToBtsSpy.calledOnce);
                assert(suit.procedureExecutor.calledOnce);
            })
            .then(done)
            .catch(done);
    });

    function failAssert() {
        assert.isTrue(false);
    }
});
