const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');
const imSnapshotProcedureFactory = require('../../../src/adminApi/procedures/imSnapshotProcedure');
const testUtils = require('../adminApi.test.util');
const Promise = require('bluebird');

describe('ADMIN API: imSnapshotProcedure', function () {

    let imSnapshotProcedure,
        connectionInitializerMock,
        ims2DownloaderMock;

    beforeEach(function () {
        ims2DownloaderMock = createIms2DownloaderMock();
        connectionInitializerMock = testUtils.createConnectionInitializerMock();
        imSnapshotProcedure = imSnapshotProcedureFactory({
            ims2Downloader: ims2DownloaderMock,
            connectionInitializer: connectionInitializerMock
        });
    });

    function createIms2DownloaderMock() {
        return {
            downloadDecompressed: sinon.spy(() =>
                Promise.resolve({fileData: new ArrayBuffer()}
            ))
        };
    }

    it('should execute imSnapshot', function (done) {
        // given
        const completionListener = sinon.spy();
        connectionInitializerMock.prepare.returns(Promise.resolve());
        const msgData = createMsgData('imSnapshot');
        // when
        imSnapshotProcedure.execute(msgData, completionListener)
            .then(() => {
                // then
                assert(connectionInitializerMock.connectToBts.calledOnce);
                assert(ims2DownloaderMock.downloadDecompressed.calledOnce);
                assert(completionListener.calledOnce);
            }).then(done).catch(done);
    });

    it('should notify when failed at existing procedure preparation', function (done) {
        const completionListener = sinon.spy();
        connectionInitializerMock.prepare.returns(Promise.reject('some error'));
        const msgData = createMsgData('imSnapshot');
        // when
        imSnapshotProcedure.execute(msgData, completionListener)
            .then(() => {
                done(new Error('This promise should not be resolved'));
            })
            .catch(() => {
                // then
                assert(completionListener.withArgs(ADMIN_API_STATUSES.FAILED, 'some error').calledOnce);
                done();
            });
    });

    function createMsgData(procedureName) {
        return {
            name: procedureName
        };
    }
})
;
