const otherProcedure = require('../../../src/adminApi/procedures/otherProcedure');
const testUtils = require('../adminApi.test.util');
const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');

describe('otherProcedure', () => {
    let suit;
    beforeEach(() => {
        suit = {};
        suit.completionListener = sinon.spy();
        suit.procedureExecutorMock = testUtils.createProcedureExecutorMock();
        suit.connectionServiceStub = {
            isSignedIn: () => true
        };
        suit.otherProcedure = otherProcedure.newInstance({
            procedureExecutor: suit.procedureExecutorMock,
            btsServiceProcedure: testUtils.createBtsServiceProcedureMock(),
            connectionService: suit.connectionServiceStub
        });
    });
    afterEach(() => (suit = null));

    describe('#executeSwRollback', () => {
        it('should not execute executeSwRollback when name is wrong', (done) => {
            // given
            const msgData = {
                name: 'swRollbackWrong'
            };
            // when
            suit.otherProcedure.executeSwRollback(msgData, suit.completionListener)
                .then(testUtils.failAssert)
                .catch((error) => {
                    // then
                    assert.equal(error, 'Invalid schema');
                    sinon.assert.calledOnce(suit.completionListener);
                    sinon.assert.calledWith(suit.completionListener, ADMIN_API_STATUSES.FAILED, '/name, String does not match pattern: ^swRollback$');
                    sinon.assert.notCalled(suit.procedureExecutorMock.executeSwRollback);
                    done();
                });
        });
        it('should execute executeSwRollback', (done) => {
            // given
            const msgData = {
                name: 'swRollback'
            };
            // when
            suit.otherProcedure.executeSwRollback(msgData, suit.completionListener)
                .then(() => {
                    // then
                    sinon.assert.calledOnce(suit.procedureExecutorMock.executeSwRollback);
                })
                .then(done).catch(done);
        });
    });

    describe('#executeChangeEthernetPortSecurity', () => {
        it('should not execute changeEthernetPortSecurity when name is wrong', (done) => {
            // given
            const msgData = {
                name: 'changeEthernetPortSecurityWrong',
                parameters: {
                    enableEthernetPortSec: true
                }
            };
            // when
            suit.otherProcedure.executeChangeEthernetPortSecurity(msgData, suit.completionListener)
                .then(testUtils.failAssert)
                .catch((error) => {
                    // then
                    assert.equal(error, 'Invalid schema');
                    sinon.assert.calledOnce(suit.completionListener);
                    sinon.assert.calledWith(suit.completionListener, ADMIN_API_STATUSES.FAILED, '/name, String does not match pattern: ^changeEthernetPortSecurity$');
                    sinon.assert.notCalled(suit.procedureExecutorMock.executeChangeEthernetPortSecurity);
                    done();
                });
        });

        it('should not execute changeEthernetPortSecurity when \'parameters\' are missing', (done) => {
            // given
            const msgData = {
                name: 'changeEthernetPortSecurity'
            };
            // when
            suit.otherProcedure.executeChangeEthernetPortSecurity(msgData, suit.completionListener)
                .then(testUtils.failAssert)
                .catch((error) => {
                    // then
                    assert.equal(error, 'Invalid schema');
                    sinon.assert.calledOnce(suit.completionListener);
                    sinon.assert.calledWith(suit.completionListener, ADMIN_API_STATUSES.FAILED, 'Missing required property: parameters');
                    sinon.assert.notCalled(suit.procedureExecutorMock.executeChangeEthernetPortSecurity);
                    done();
                });
        });

        it('should not execute changeEthernetPortSecurity when \'enableEthernetPortSec\' is missing', (done) => {
            // given
            const msgData = {
                name: 'changeEthernetPortSecurity',
                parameters: {}
            };
            // when
            suit.otherProcedure.executeChangeEthernetPortSecurity(msgData, suit.completionListener)
                .then(testUtils.failAssert)
                .catch((error) => {
                    // then
                    assert.equal(error, 'Invalid schema');
                    sinon.assert.calledOnce(suit.completionListener);
                    sinon.assert.calledWith(suit.completionListener, ADMIN_API_STATUSES.FAILED, '/parameters, Missing required property: enableEthernetPortSec');
                    sinon.assert.notCalled(suit.procedureExecutorMock.executeChangeEthernetPortSecurity);
                    done();
                });
        });

        it('should not execute changeEthernetPortSecurity when \'enableEthernetPortSec\' is other then boolean', (done) => {
            // given
            const msgData = {
                name: 'changeEthernetPortSecurity',
                parameters: {
                    enableEthernetPortSec: 'true'
                }
            };
            // when
            suit.otherProcedure.executeChangeEthernetPortSecurity(msgData, suit.completionListener)
                .then(testUtils.failAssert)
                .catch((error) => {
                    // then
                    assert.equal(error, 'Invalid schema');
                    sinon.assert.calledOnce(suit.completionListener);
                    sinon.assert.calledWith(suit.completionListener, ADMIN_API_STATUSES.FAILED, '/parameters/enableEthernetPortSec, Invalid type: string (expected boolean)');
                    sinon.assert.notCalled(suit.procedureExecutorMock.executeChangeEthernetPortSecurity);
                    done();
                });
        });

        it('should execute changeEthernetPortSecurity', (done) => {
            // given
            const msgData = {
                name: 'changeEthernetPortSecurity',
                parameters: {
                    enableEthernetPortSec: true
                }
            };
            // when
            suit.otherProcedure.executeChangeEthernetPortSecurity(msgData, suit.completionListener)
                .then(() => {
                    // then
                    sinon.assert.calledOnce(suit.procedureExecutorMock.executeChangeEthernetPortSecurity);
                })
                .then(done).catch(done);
        });
    });
});
