const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');
const remoteProcedureListenerFactory = require('../../../src/adminApi/procedures/remoteProcedureListener');
const testUtils = require('../adminApi.test.util');

describe('ADMIN API: remoteProcedureListener', function () {
    let remoteProcedureListener,
        btsServiceProcedureMock;

    beforeEach(function () {
        btsServiceProcedureMock = testUtils.createBtsServiceProcedureMock();
        remoteProcedureListener = remoteProcedureListenerFactory(btsServiceProcedureMock);
    });

    it('should notify about completed status', function () {
        // given
        const procedureId = 666;
        const event = createUpdateEvent(procedureId, 'Completed');
        const completionListener = function (requestStatus, payload) {
            // then
            assert.equal(requestStatus, ADMIN_API_STATUSES.COMPLETED);
            assert.equal(payload, event.objects[0].parameters.status);
        };
        // when
        const listener = remoteProcedureListener.create(procedureId, completionListener);
        listener.update(event);
        // then
        assert(btsServiceProcedureMock.removeListener.calledOnce);
    });

    it('should notify about failure status', function () {
        // given
        const procedureId = 666;
        const event = createUpdateEvent(procedureId, 'Failed');
        const completionListener = function (requestStatus, payload) {
            // then
            assert.equal(requestStatus, ADMIN_API_STATUSES.FAILED);
            assert.equal(payload, event.objects[0].parameters.status);
        };
        // when
        const listener = remoteProcedureListener.create(procedureId, completionListener);
        listener.update(event);
        // then
        assert(btsServiceProcedureMock.removeListener.calledOnce);
    });

    it('should notify about ongoing status', function () {
        // given
        const procedureId = 666;
        const event = createUpdateEvent(procedureId, 'Accepted');
        const completionListener = function (requestStatus, payload) {
            // then
            assert.equal(requestStatus, ADMIN_API_STATUSES.ONGOING);
            assert.equal(payload, event.objects[0].parameters.status);
        };
        // when
        const listener = remoteProcedureListener.create(procedureId, completionListener);
        listener.update(event);
        // then
        assert.equal(btsServiceProcedureMock.removeListener.callCount, 0);
    });

    it('should ignore notification about procedure with different id', function () {
        // given
        const procedureId = 666;
        const receivedProcedureId = 999;
        const event = createUpdateEvent(receivedProcedureId, 'whatever');
        const completionListener = sinon.spy();
        // when
        const listener = remoteProcedureListener.create(procedureId, completionListener);
        listener.update(event);
        // then
        assert.equal(completionListener.callCount, 0);
        assert.equal(btsServiceProcedureMock.removeListener.callCount, 0);
    });

    function createUpdateEvent(procedureId, status) {
        return {
            objects: [
                {
                    parameters: {
                        clientId: procedureId,
                        status: status
                    }
                }
            ]
        };
    }
});
