const snapshotFileSaverFactory = require('../../../src/adminApi/procedures/snapshotFileSaver');
const ADMIN_API_STATUSES = require('../../../src/adminApi/adminApiStatuses');

describe('ADMIN API: snapshotFileSaver', function () {
    'use strict';

    var testData = new Uint8Array([1, 2, 3]);
    var resultData = 'AQID';
    var errorMessage = 'error message';
    var completionListenerMock;
    var snapshotFileSaver;

    beforeEach(function () {
        completionListenerMock = sinon.spy();

        snapshotFileSaver = snapshotFileSaverFactory();
        snapshotFileSaver.setListener(completionListenerMock);
    });

    describe('saveAs method', function () {
        it('should invoke listener with base64 encoded binary data', function () {
            snapshotFileSaver.saveAs(testData);

            assert(completionListenerMock.calledWith(ADMIN_API_STATUSES.COMPLETED, resultData));
        });
    });

    describe('failure method', function () {
        it('should invoke listener with message', function () {
            snapshotFileSaver.failure(errorMessage);

            assert(completionListenerMock.calledWith(ADMIN_API_STATUSES.FAILED, errorMessage));
        });
    });

    describe('setListener method', function () {
        it('should reset aborted lock', function () {
            snapshotFileSaver.setAborted();
            snapshotFileSaver.setListener(completionListenerMock);

            snapshotFileSaver.saveAs(testData);

            assert(completionListenerMock.calledOnce);
        });
    });

    describe('setAborted method', function () {
        it('should stop saveAs method from calling completion listener', function () {
            snapshotFileSaver.setListener(completionListenerMock);
            snapshotFileSaver.setAborted();

            snapshotFileSaver.saveAs(testData);

            assert(completionListenerMock.notCalled);
        });
    });
});
