const arrays = require('../src/arrays');

describe('arrays', function () {

    describe('process', function () {

        it('should process array', function (done) {
            var items = ['1', '2', '3', '4'];

            var processedItems = [];

            function processItem(item) {
                processedItems.push(item);
            }

            function complete() {
                assert.equal(processedItems.length, items.length);
                done();
            }

            arrays.process(items, 1, processItem, complete);
        });

        it('should process array in intervals', function (done) {
            function timeout(fun) {
                timeout.invoked = timeout.invoked || 0;
                timeout.invoked++;
                fun();
            }

            var items = ['1', '2', '3', '4'];

            var processedItems = [];

            function processItem(item) {
                processedItems.push(item);
            }

            function complete() {
                assert.equal(processedItems.length, items.length);
                done();
            }

            arrays.process(items, -1, processItem, complete, timeout);
            assert.equal(timeout.invoked, 4);
        });

    });

    describe('process until success', function () {
        var items = ['1', '2', '3', '4'],
            timeoutFun,
            processingFun,
            onDoneFun,
            processedItems;

        beforeEach(function (done) {
            processedItems = [];

            timeoutFun = function (fun) {
                timeoutFun.invoked = timeoutFun.invoked || 0;
                timeoutFun.invoked++;
                fun();
            };

            processingFun = function (item) {
                processedItems.push(item);
                return item === '2';
            };

            onDoneFun = function () {
                onDoneFun.invoked = onDoneFun.invoked || 0;
                onDoneFun.invoked++;
                done();
            };

            arrays.processUntilSuccess(items, -1, processingFun, onDoneFun, timeoutFun);
        });

        it('should process array until success', function () {
            assert.lengthOf(processedItems, 2);
            assert.deepEqual(processedItems[0], '1');
            assert.deepEqual(processedItems[1], '2');
        });

        it('should process array in intervals', function () {
            assert.deepEqual(timeoutFun.invoked, 1);
        });

        it('should run onDone function only once', function () {
            assert.deepEqual(onDoneFun.invoked, 1);
        });

    });

});
