describe('bytesToStringConverter', function () {
    var converter;

    beforeEach(function () {
        converter = infoModel.bytesToStringConverter;
    });

    function createDataView() {
        var buffer = new ArrayBuffer(9);
        var dataView = new DataView(buffer);
        return dataView;
    }

    describe('convertBytesToString', function () {

        var inputBytesTable = [115, 111, 109, 101, 32, 116, 101, 115, 116];

        function fillDataView(start, end, dataView, inputBytesTable) {
            for (var i = start; i < end; i++) {
                dataView.setUint8(i, inputBytesTable[i]);
            }
        }

        it('should create correct string from bytes range: [0-9)', function () {
            var dataView = createDataView();
            var expectedString = 'some test';
            var start = 0;
            var end = 9;

            fillDataView(start, end, dataView, inputBytesTable);
            assert.deepEqual(expectedString, converter.convertBytesToString(dataView, start, end));
        });

        it('should create correct string from bytes range: [0-5)', function () {
            var dataView = createDataView();
            var expectedString = 'some ';
            var start = 0;
            var end = 5;
            fillDataView(start, end, dataView, inputBytesTable);
            assert.deepEqual(expectedString, converter.convertBytesToString(dataView, start, end));
        });

        it('should create correct string bytes range: [4-9)', function () {
            var dataView = createDataView();
            var expectedString = ' test';
            var start = 4;
            var end = 9;
            fillDataView(start, end, dataView, inputBytesTable);
            assert.deepEqual(expectedString, converter.convertBytesToString(dataView, start, end));
        });

        it('should create correct string bytes range: [3-6)', function () {
            var dataView = createDataView();
            var expectedString = 'e t';
            var start = 3;
            var end = 6;
            fillDataView(start, end, dataView, inputBytesTable);
            assert.deepEqual(expectedString, converter.convertBytesToString(dataView, start, end));
        });
    });

    describe('convertStringToBytes', function () {

        function createResultTable(dataView) {
            var resultBytesTable = [];
            for (var i = 0; i < 9; i++) {
                resultBytesTable[i] = dataView.getUint8(i);
            }
            return resultBytesTable;
        }

        it('should correctly fill DataView from characters range: [0-7)', function () {
            var inputString = 'some test';
            var start = 0;
            var end = 7;
            var dataView = createDataView();
            var expectedBytesTable = [115, 111, 109, 101, 32, 116, 101, 0, 0];

            converter.convertStringToBytes(dataView, start, end, inputString);
            var resultBytesTable = createResultTable(dataView);
            assert.deepEqual(expectedBytesTable, resultBytesTable);
        });

        it('should correctly fill DataView on positions: [4-9)', function () {
            var inputString = 'some test';
            var start = 4;
            var end = 9;
            var dataView = createDataView();
            var expectedBytesTable = [0, 0, 0, 0, 115, 111, 109, 101, 32];

            converter.convertStringToBytes(dataView, start, end, inputString);
            var resultBytesTable = createResultTable(dataView);
            assert.deepEqual(expectedBytesTable, resultBytesTable);
        });

        it('should correctly fill DataView on positions: [0-9)', function () {
            var inputString = 'some test';
            var start = 0;
            var end = 9;
            var dataView = createDataView();
            var expectedBytesTable = [115, 111, 109, 101, 32, 116, 101, 115, 116];

            converter.convertStringToBytes(dataView, start, end, inputString);
            var resultBytesTable = createResultTable(dataView);
            assert.deepEqual(expectedBytesTable, resultBytesTable);
        });

        it('should correctly fill DataView on positions: [3-7)', function () {
            var inputString = 'some test';
            var start = 3;
            var end = 7;
            var dataView = createDataView();
            var expectedBytesTable = [0, 0, 0, 115, 111, 109, 101, 0, 0];

            converter.convertStringToBytes(dataView, start, end, inputString);
            var resultBytesTable = createResultTable(dataView);
            assert.deepEqual(expectedBytesTable, resultBytesTable);
        });
    });
});
