const cacheFactory = require('../src/cacheFactory');

describe('cacheFactory', function () {

    var cache;
    beforeEach(function () {
        cache = cacheFactory.newInstance(2);
    });

    it('should store value', function () {
        var value = {};
        cache.set(1, value);

        assert.equal(cache.get(1), value);
    });

    it('should store up to capacity', function () {
        cache.set(1, {});
        cache.set(2, {});

        assert.deepEqual(cache.get(1), {});
        assert.deepEqual(cache.get(2), {});

        cache.set(3, {});
        assert.deepEqual(cache.get(3), {});
        assert.deepEqual(cache.get(2), {});
        assert.notOk(cache.get(1));
    });

    it('should clear', function () {
        cache.set(1, {});
        cache.set(2, {});
        cache.set(3, {});

        cache.clear();

        assert.notOk(cache.get(1));
    });

    it('should work correctly after clearing', function () {
        cache.set(1, {});
        cache.set(2, {});
        cache.set(3, {});
        cache.clear();

        cache.set(4, {});
        cache.set(5, {});

        assert.ok(cache.get(4));
        assert.ok(cache.get(5));
    });

    it('should update stored value', function () {
        cache.set(1, '1');
        cache.set(2, '2');

        cache.set(1, '11');
        cache.set(1, '11');
        cache.set(1, '11');
        cache.set(1, '11');

        assert.equal(cache.get(1), '11');
        assert.equal(cache.get(2), '2');
    });

    it('should create missing value if producer given', function () {
        const value = cache.get(1, function (key) {
            return key * 2;
        });

        assert.equal(value, 2);
        assert.equal(cache.get(1), 2);
    });

    describe('removing least recently used values', function () {

        it('reading value should mark it used', function () {
            cache.set(1, '10');
            cache.set(2, '20');

            cache.get(1);
            cache.set(3, '30');

            assert.notOk(cache.get(2));
            assert.equal(cache.get(1), '10');
            assert.equal(cache.get(3), '30');
        });

        it('writing value should mark it used', function () {
            cache.set(1, '10');
            cache.set(2, '20');

            cache.set(1, '11');
            cache.set(3, '30');

            assert.notOk(cache.get(2));
            assert.equal(cache.get(1), '11');
            assert.equal(cache.get(3), '30');
        });

        it('should handle additions after removing oldest entry', function () {
            cache.set(1, '10');
            cache.set(2, '20');
            cache.set(3, '30');

            cache.set(4, '30');
            cache.set(5, '50');
            cache.set(6, '60');

            assert.notOk(cache.get(1));
            assert.notOk(cache.get(2));
            assert.notOk(cache.get(3));

            assert.equal(cache.get(5), '50');
            assert.equal(cache.get(6), '60');
        });
    });
});
