describe('btsClient: heartBeatHandler', function () {
    var heartBeatHandler;
    var btsClient;

    beforeEach(function () {
        btsClient = createBtsClientMock();
        heartBeatHandler = infoModel.btsClient.heartBeatHandler(btsClient);
    });

    describe('startHeartBeat', function () {

        it('should disconnect btsClient after not receiving heart beat on time', function (done) {
            heartBeatHandler.startHeartBeat(1);

            heartBeatHandler.onStop(function () {
                assert.deepEqual(btsClient.states, [
                        'connected',
                        'sendHeartBeatRequest',
                        'disconnected'
                    ]
                );
                assert.isFalse(heartBeatHandler.isStarted());
                done();
            });
        });

        it('should disconnect btsClient after not receiving heart beat on time - multiple request handled', function (done) {
            heartBeatHandler.startHeartBeat(1);
            var numOfSendHeartBeatReq = 0;
            var maxHeartBeats = 3;

            btsClient.sendHeartBeatRequest = function () {
                btsClient.states.push('sendHeartBeatRequest');
                if (numOfSendHeartBeatReq < maxHeartBeats) {
                    assert.isTrue(heartBeatHandler.isStarted());

                    btsClient.pushHeartBeatResponse();
                    numOfSendHeartBeatReq++;
                }
            };

            heartBeatHandler.onStop(function () {
                assert.deepEqual(btsClient.states, [
                        'connected',
                        'sendHeartBeatRequest', 'receivedHeartBeatResponse',
                        'sendHeartBeatRequest', 'receivedHeartBeatResponse',
                        'sendHeartBeatRequest', 'receivedHeartBeatResponse',
                        'sendHeartBeatRequest',
                        'disconnected'
                    ]
                );
                assert.isFalse(heartBeatHandler.isStarted());
                done();
            });
        });

        it('should stop sending heart beats after disconnect', function (done) {
            heartBeatHandler.startHeartBeat(1);

            btsClient.sendHeartBeatRequest = function () {
                btsClient.states.push('sendHeartBeatRequest');
                btsClient.pushHeartBeatResponse();
                btsClient.disconnect();
            };

            heartBeatHandler.onStop(function () {
                assert.deepEqual(btsClient.states, [
                        'connected',
                        'sendHeartBeatRequest', 'receivedHeartBeatResponse',
                        'disconnected']
                );
                assert.isFalse(heartBeatHandler.isStarted());
                done();
            });
        });

        it('should do nothing when stopping not started handler', function () {
            heartBeatHandler.stopHeartBeat();

            assert.isFalse(heartBeatHandler.isStarted());
        });

        it('should stop started handler', function () {
            heartBeatHandler.startHeartBeat(1);
            assert.isTrue(heartBeatHandler.isStarted());

            heartBeatHandler.stopHeartBeat();
            assert.isFalse(heartBeatHandler.isStarted());
        });

    });

    function createBtsClientMock() {
        var connected = true;
        var heartBeatResponseHandler = null;
        var states = ['connected'];
        return {
            isConnected: function () {
                return connected;
            },
            connect: function () {
                connected = true;
                states.push('connected');
            },
            disconnect: function () {
                connected = false;
                states.push('disconnected');
            },
            sendHeartBeatRequest: function () {
                states.push('sendHeartBeatRequest');
            },
            addHeartBeatResponseListener: function (listener) {
                heartBeatResponseHandler = events.newEvent();
                return heartBeatResponseHandler.watch(listener);
            },
            //for testing
            pushHeartBeatResponse: function () {
                if (heartBeatResponseHandler) {
                    states.push('receivedHeartBeatResponse');
                    heartBeatResponseHandler.notify();
                }
            },
            states: states
        };
    }

});
