const objectInfoNormalizer = require('../../src/communication/objectInfoNormalizer');
const ProtoByteBuffer = require('protobufjs').ByteBuffer;

describe('objectInfoNormalizer', () => {
    var normalizer = objectInfoNormalizer.newInstance();
    const OBJECT_ID = '0a';
    const PARENT_ID = '0d';


    describe('#buildObjectInfo', () => {
        it('should create objectInfo for root object', () => {
            var rootImiObjectInfo = {
                object_create_info: {
                    instance_id: 0
                },
                object_id: ProtoByteBuffer.fromHex(OBJECT_ID),
                object_type: 0
            };
            var objectInfo = normalizer.buildObjectInfo(rootImiObjectInfo);

            assert.strictEqual(objectInfo.uuid, OBJECT_ID);
            assert.strictEqual(objectInfo.objectType, 0);
            assert.strictEqual(objectInfo.instanceId, 0);
            assert.isUndefined(objectInfo.parentUuid);
        });

        it('should create objectInfo for leaf object', () => {
            var leafImiObjectInfo = {
                object_create_info: {
                    instance_id: 0,
                    parent_id: ProtoByteBuffer.fromHex(PARENT_ID)
                },
                object_id: ProtoByteBuffer.fromHex(OBJECT_ID),
                object_type: 0
            };
            var objectInfo = normalizer.buildObjectInfo(leafImiObjectInfo);

            assert.strictEqual(objectInfo.uuid, OBJECT_ID);
            assert.strictEqual(objectInfo.objectType, 0);
            assert.strictEqual(objectInfo.instanceId, 0);
            assert.strictEqual(objectInfo.parentUuid, PARENT_ID);
        });

        it('should create objectInfo if objectCreateInfo is not present', () => {
            var imiObjectInfo = {
                object_id: ProtoByteBuffer.fromHex(OBJECT_ID),
                object_type: 0
            };
            var objectInfo = normalizer.buildObjectInfo(imiObjectInfo);

            assert.strictEqual(objectInfo.uuid, OBJECT_ID);
            assert.strictEqual(objectInfo.objectType, 0);
            assert.isUndefined(objectInfo.instanceId);
            assert.isUndefined(objectInfo.parentUuid);
        });
    });

    describe('#buildImiObjectInfo', () => {
        it('should create imiObjectInfo for root object', () => {
            var rootObjectInfo = {
                uuid: OBJECT_ID,
                objectType: 0,
                instanceId: 0
            };

            var imiObjectInfo = normalizer.buildImiObjectInfo(rootObjectInfo);

            assert.deepEqual(imiObjectInfo.object_id, ProtoByteBuffer.fromHex(OBJECT_ID));
            assert.strictEqual(imiObjectInfo.object_type, 0);
            assert.strictEqual(imiObjectInfo.object_create_info.instance_id, 0);
            assert.isUndefined(imiObjectInfo.object_create_info.parent_id);
        });

        it('should create imiObjectInfo for leaf object', () => {
            var leafObjectInfo = {
                uuid: OBJECT_ID,
                objectType: 0,
                instanceId: 0,
                parentUuid: PARENT_ID
            };
            var imiObjectInfo = normalizer.buildImiObjectInfo(leafObjectInfo);

            assert.deepEqual(imiObjectInfo.object_id, ProtoByteBuffer.fromHex(OBJECT_ID));
            assert.strictEqual(imiObjectInfo.object_type, 0);
            assert.strictEqual(imiObjectInfo.object_create_info.instance_id, 0);
            assert.deepEqual(imiObjectInfo.object_create_info.parent_id, ProtoByteBuffer.fromHex(PARENT_ID));
        });
    });
});
