const TranslatingBtsClient = require('../../src/communication/translatingBtsClient');
const BtsClient = require('../../src/communication/btsClient');
const ProtoByteBuffer = require('protobufjs').ByteBuffer;

describe('translatingBtsClient', () => {

    var btsClient;
    var translatingBtsClient;
    var sicInstance;
    var socketListener;
    var testPromise;

    beforeEach(() => {
        btsClient = BtsClient();
        translatingBtsClient = TranslatingBtsClient.newInstance({btsClient});
        sicInstance = btsClient.getSicInstance();
        socketListener = btsClient.getSocketListener();
        testPromise = createTestPromise();
    });

    describe('#addListener', () => {
        it('should notify listener that was added to translatingBtsClient on socket message', (done) => {
            translatingBtsClient.addListener({
                loggedIn: testPromise.resolve
            });

            const msg = sicInstance.imgwLoginSuccessResp();
            msg.setProfile('testProfile');
            msg.setUserName('testUser');
            msg.setFailedLoginNo(0);
            msg.setLastSuccess('yesterday');

            socketListener.completedMessage(msg);

            testPromise.promise
                .then(function(result){
                    const expectedResult = {
                        profile: 'testProfile',
                        userName: 'testUser',
                        failedLoginNo: 0,
                        lastSuccess: 'yesterday'
                    };
                    assert.deepEqual(result, expectedResult);
                })
                .then(done)
                .catch(done);
        });
    });

    describe('#resolveManagedObjects', () => {
        it('should convert updated object_info to distNames', (done) => {
            const OBJECT_ID = '0a';

            translatingBtsClient.getMapper().addClassName(0, 'CLASS_A');

            translatingBtsClient.addListener({
                data: testPromise.resolve
            });

            var updateNotification = sicInstance.updateNotification();
            updateNotification.setManagedObjects([{
                object_info: {
                    object_create_info: {
                        instance_id: 0
                    },
                    object_id: ProtoByteBuffer.fromHex(OBJECT_ID),
                    object_type: 0
                }
            }]);
            updateNotification.setRemovedObjectIds([ProtoByteBuffer.fromHex(OBJECT_ID)]);

            socketListener.completedMessage(updateNotification);

            testPromise.promise
                .then((updateData) => {
                    assert.strictEqual(updateData.managedObjects[0].distName, '/CLASS_A-0');
                    assert.strictEqual(updateData.removedDistNames[0], '/CLASS_A-0');
                })
                .then(done)
                .catch(done);
        });
    });

    function createTestPromise() {
        const result = {};
        result.promise = new Promise((resolve, reject)=> {
            result.resolve = resolve;
            result.reject = reject;
        });
        return result;
    }
});
