const FileTransferConnection = require('../../src/fileTransfer/fileTransferConnection');
const fileTransferTestUtil = require('./fileTransferTestUtil');
const SESSION_TYPE_PLAIN = 1;

describe('fileTransferConnection', function () {

    var socket, sic, fileTransferClient, fileTransferConnection, testUtil;

    beforeEach(function () {
        testUtil = fileTransferTestUtil();
        sic = testUtil.sic;
        socket = testUtil.socket;
        const authService = testUtil.authService;
        const applicationInfo = testUtil.applicationInfo;
        fileTransferClient = testUtil.fileTransferClient;
        fileTransferConnection = new FileTransferConnection(authService, applicationInfo);
        fileTransferConnection.setWebSocketClient(fileTransferClient);
    });

    it('should ask about gateway protocol version during start', function (done) {
        const ignoredMessages = 0;
        fileTransferConnection
            .connect()
            .then(assertSentGetGatewayProtocolVersionRequest)
            .then(done).catch(done);

        testUtil.performConnect();

        function assertSentGetGatewayProtocolVersionRequest() {
            const getProtocolVersionId = sic.imgwGetProtocolVersionReq().getId();
            const msg = socket.getMessage(ignoredMessages);
            assert.strictEqual(msg.getId(), getProtocolVersionId);
        }
    });

    it('should send login request with plain session type during start', function (done) {
        const ignoredMessages = 1;
        fileTransferConnection
            .connect()
            .then(assertSentLoginRequest)
            .then(done).catch(done);

        testUtil.performConnect();

        function assertSentLoginRequest() {
            const loginRequestId = sic.imgwLoginReq().getId();
            const msg = socket.getMessage(ignoredMessages);
            assert.strictEqual(msg.getId(), loginRequestId);
            assert.strictEqual(msg.getSessionType(), SESSION_TYPE_PLAIN);
        }
    });
});