"use strict";

const sic = require('../../src/sic');
const btsClient = require('../../src/communication/btsClient');

module.exports = function fileTransferTestUtil() {
    const authenticationData = {
        host: '127.0.0.1',
        port: 123,
        isSecure: false,
        userName: 'developer',
        token: 'ABC',
        authMethod: {id: 1}
    };
    const sentMessages = [];
    const sicInstance = sic(7);
    const sicBuffer = sicInstance.createSicMessageBuffer({
        completedMessage: function (message) {
            sentMessages.push(message);
        }
    });
    const socket = {
        isConnected: isConnected,
        send: send,
        close: close,
        //methods for testing
        isOpen: true,
        getMessage: getMessage,
        receive: receive
    };
    const socketFactory = {
        create: function () {
            return socket;
        }
    };
    const fileTransferClient = btsClient(socketFactory);
    const authService = {
        getHost: getHost,
        getPort: getPort,
        isSecure: isSecure,
        getUserName: getUserName,
        getToken: getToken,
        getAuthMethod: getAuthMethod
    };

    const applicationInfo = {
        getReleaseInfo: function () {
            return {
                branch: 'BRANCH',
                hash: '1234556'
            }
        }
    };

    return {
        fileTransferClient: fileTransferClient,
        socket: socket,
        authService: authService,
        applicationInfo: applicationInfo,
        sic: sicInstance,
        performConnect: receiveLoginSuccessResp
    };

    function toSicMessageBytes(msg) {
        const payloadBuffer = msg.isSicPlus ? msg.getPayload() : msg.getBuffer();
        const payloadLength = payloadBuffer.byteLength;
        const header = sicInstance.createHeader();
        header.setId(msg.getId());
        header.setPayloadLength(payloadLength);
        const sicMsg = sicInstance.createMessage(header);
        sicMsg.setPayload(payloadBuffer);
        return sicMsg.getBuffer();
    }

    function receiveLoginSuccessResp() {
        receiveProtocolVersionResp();
        socket.receive(sicInstance.imgwLoginSuccessResp());
    }

    function receiveProtocolVersionResp() {
        webSocketConnect();
        socket.receive(sicInstance.imgwGetProtocolVersionResp());
    }

    function webSocketConnect() {
        socket.onopen();
    }

    function isConnected() {
        return socket.isOpen;
    }

    function send(arrayBuffer) {
        sicBuffer.push(arrayBuffer);
    }

    function close(code, reason) {
        socket.isOpen = false;
        if (socket.onclose) socket.onclose({code: code, reason: reason});
    }

    function getMessage(position) {
        return sentMessages[position];
    }

    function receive(message) {
        const event = {
            data: toSicMessageBytes(message)
        };
        socket.onmessage(event);
    }

    function getHost() {
        return authenticationData.host;
    }

    function getPort() {
        return authenticationData.port;
    }

    function isSecure() {
        return authenticationData.isSecure;
    }

    function getUserName() {
        return authenticationData.userName;
    }

    function getToken() {
        return authenticationData.token;
    }

    function getAuthMethod() {
        return authenticationData.authMethod;
    }
};
