describe('historyFactory - iterator', function () {

    var history,
        historyFactory = infoModel.historyFactory,
        assertions = infoModel.historyEntryAssertions;

    describe('entries iterator', function () {
        beforeEach(function () {
            history = historyFactory.newInstance(1000);
            history.addEntry(createChangeEntry(0, '/A-1'));
            history.addEntry(createChangeEntry(1, '/B-1'));
            history.addEntry(createChangeEntry(2, '/C-1'));
            history.addEntry(createChangeEntry(3, '/D-1'));
        });

        it('should get size', function () {
            const iterator = history.getEntriesIterator();

            assert.equal(iterator.size(), 4);
        });

        it('should get next', function () {
            const iterator = history.getEntriesIterator();

            assert.deepEqual(iterator.next(), {value: history.getEntry(0), done: false});
            assert.deepEqual(iterator.next(), {value: history.getEntry(1), done: false});
            assert.deepEqual(iterator.next(), {value: history.getEntry(2), done: false});
            assert.deepEqual(iterator.next(), {value: history.getEntry(3), done: false});
            assert.deepEqual(iterator.next(), {done: true});
            assert.deepEqual(iterator.next(), {done: true});
        });

    });

    describe('entries iterator - no entries', function () {
        beforeEach(function () {
            history = historyFactory.newInstance(1000);
        });

        it('should get size', function () {
            const iterator = history.getEntriesIterator();

            assert.equal(iterator.size(), 0);
        });

        it('should get next', function () {
            const iterator = history.getEntriesIterator();

            assert.deepEqual(iterator.next(), {done: true});
            assert.deepEqual(iterator.next(), {done: true});
        });

    });

    function createChangeEntry(id, distNames) {
        return assertions.createChangeEntry(id, distNames);
    }
});
