describe('ObjectListenersContainer', function () {

    var objectListenersContainer,
        emptyCallbacksMock,
        callbacksWithParametersTrackingMock,
        callbacksWithChildrenTrackingMock;

    beforeEach(function () {
        objectListenersContainer = new adminCore.imTracker.ObjectListenersContainer();
    });


    describe('testing objectListeners container', function () {
        var DISTNAME = "/MRBTS-1/BTS-1",
            PARAMETERS = {'param-1': ""};

        var objectListeners;

        beforeEach(function () {
            objectListeners = objectListenersContainer;

            emptyCallbacksMock = {};
            callbacksWithParametersTrackingMock = {
                onParameterChanged: _.noop,
                onParametersChanged: _.noop
            };
            callbacksWithChildrenTrackingMock = {
                onChildAdded: _.noop
            }
        });

        it('should create new instance of DistNameListeners', function () {
            assert.isNotNull(objectListeners);
            assert.notEqual(objectListeners, new adminCore.imTracker.ObjectListenersContainer())
        });

        it('should add listeners', function () {
            objectListeners.addListener(createObjectListener(emptyCallbacksMock));
            objectListeners.addListener(createObjectListener(callbacksWithParametersTrackingMock));
            assert.equal(objectListeners.getListeners().length, 2)
        });

        it('should add listeners and remove one', function () {
            var listener = createObjectListener(callbacksWithParametersTrackingMock);
            objectListeners.addListener(createObjectListener(emptyCallbacksMock));
            objectListeners.addListener(listener);
            objectListeners.removeListener(listener);
            assert.equal(objectListeners.getListeners().length, 1)
        });

        it("should not have parameters' listeners", function () {
            objectListeners.addListener(createObjectListener(emptyCallbacksMock));
            assert.isFalse(objectListeners.hasParametersListeners())
        });

        it("should have parameters' listeners", function () {
            objectListeners.addListener(createObjectListener(callbacksWithParametersTrackingMock));
            assert.isTrue(objectListeners.hasParametersListeners())
        });

        it("should not have parameters' listeners after removal", function () {
            var listener = createObjectListener(callbacksWithParametersTrackingMock);
            objectListeners.addListener(listener);
            objectListeners.removeListener(listener);
            assert.isFalse(objectListeners.hasParametersListeners())
        });

        it("should have parameters' listeners after removal", function () {
            var firstListener = createObjectListener(callbacksWithParametersTrackingMock);
            var secondListener = createObjectListener(callbacksWithParametersTrackingMock);
            objectListeners.addListener(firstListener);
            objectListeners.addListener(secondListener);
            objectListeners.removeListener(firstListener);
            assert.isTrue(objectListeners.hasParametersListeners())
        });

        it("should not have listeners on child added", function () {
            objectListeners.addListener(createObjectListener(emptyCallbacksMock));
            assert.isFalse(objectListeners.hasChildListeners())
        });

        it("should have listeners on child added", function () {
            objectListeners.addListener(createObjectListener(callbacksWithChildrenTrackingMock));
            assert.isTrue(objectListeners.hasChildListeners())
        });

        it("should not have listeners on child added after removal", function () {
            var listener = createObjectListener(callbacksWithChildrenTrackingMock);
            objectListeners.addListener(listener);
            objectListeners.removeListener(listener);
            assert.isFalse(objectListeners.hasChildListeners())
        });

        it("should have listeners on child added after removal", function () {
            var firstListener = createObjectListener(callbacksWithChildrenTrackingMock);
            var secondListener = createObjectListener(callbacksWithChildrenTrackingMock);
            objectListeners.addListener(firstListener);
            objectListeners.addListener(secondListener);
            objectListeners.removeListener(firstListener);
            assert.isTrue(objectListeners.hasChildListeners())
        });

        function createObjectListener(callbacks) {
            return {
                distName: DISTNAME,
                callbacks: callbacks,
                parameters: PARAMETERS
            };
        }
    });
});
