const testUtil = require('./test.util');
const ims2FromZip = require('../src/ims2FromZip');

describe('ims2FromZip', function () {
    'use strict';

    var progressListener;
    beforeEach(function () {
        progressListener = {
            onComplete: function (ims2) {
            },
            onReadFrame: function (frame) {
            },
            onReset: function (reset) {
            },
            onUpdate: function (update) {
            },
            onMetaBuffer: function (metaBuffer) {
            },
            onReadBytes: function (title, loaded, total) {
            },
            onError: function (msg) {
            }
        }
    });

    it('should decompress zip, xz and load compressed ims2', function (done) {
        testUtil.loadBuffer('base/resources/xz/BTS6249_1011_part_8.zip')
            .then(function (arrayBuffer) {
                progressListener.onReadBytes = sinon.spy();
                progressListener.onComplete = function (ims2) {
                    assert.equal(SparkMD5.ArrayBuffer.hash(ims2), 'd41d8cd98f00b204e9800998ecf8427e');
                    assert(progressListener.onReadBytes.called);
                    done();
                };
                ims2FromZip.readAsync(arrayBuffer, progressListener);
            });
    });

    it('should decompress zip and not find xz', function (done) {
        testUtil.loadBuffer('base/resources/xz/noXzFile.zip')
            .then(function (arrayBuffer) {
                progressListener.onError = function (msg) {
                    assert.equal(msg, 'NO_IMS_XZ_FILE');
                    done();
                };
                ims2FromZip.readAsync(arrayBuffer, progressListener);
            });
    });

    it('should decompress zip and cancel decoding xz', function (done) {
        testUtil.loadBuffer('base/resources/xz/30mb.zip')
            .then(function (arrayBuffer) {
                var cancelFn;
                var bytesRead = 0;
                progressListener.onReadBytes = function (title, read, total) {
                    if (title === 'XZ_FILE_DECODING') {
                        bytesRead = read;
                        cancelFn();
                        assert(bytesRead < arrayBuffer.byteLength);
                        done();
                    }
                };
                cancelFn = ims2FromZip.readAsync(arrayBuffer, progressListener);
            });
    });
});
