const testUtil = require('../../test.util');

describe('pmSnapshot', () => {
    let pmLocations = null;
    const fileTransferStub = {
        connect: function () {
            return Promise.resolve();
        },
        disconnect: function () {},
        getFile: function (myFile) {
            return testUtil.loadBuffer(myFile)
                .then((arrayBuffer) => {
                    const modifArray = {
                        buffer: arrayBuffer
                    };
                    return modifArray;
                });
        }
    };
    const pmCounters = adminCore.measurements.pmSnapshot.pmSnapshot(fileTransferStub);

    it('should decompress xz and convert file to JSON', (done) => {
        pmLocations = ['base/resources/xz/PM-bts.xml.xz'];

        pmCounters.downloadCompressed(pmLocations)
            .then((pmCountersJson) => {
                assert.equal(pmCountersJson.length, 44, 'Size of pmCOuntersJson.values is not 44');
                const jsonHash = SparkMD5.hash(JSON.stringify(pmCountersJson));
                assert.equal(jsonHash, '850868653bed3dc6b0ef933afc92503c');
            })
            .then(done)
            .catch(done);
    });

    it('should decompress two xz files and convert all values to JSON with no duplicates', (done) => {
        pmLocations = ['base/resources/xz/PM-bts.xml.xz', 'base/resources/xz/PM-bts2.xml.xz'];
        const startPmCounters = {
            values: {},
            startTimes: []
        };
        pmCounters.downloadCompressed(pmLocations)
            .then((pmCountersJson) => {
                const pmCountersFormated = pmCounters.formatPmCounters(pmCountersJson, startPmCounters);
                const jsonHash = SparkMD5.hash(JSON.stringify(pmCountersFormated));
                assert.equal(jsonHash, '951939e70cae8715a8cf151bef76175e');
            })
            .then(done)
            .catch(done);
    });

    it('should return formatted JSON for PM counters in the correct format from harcoded value', () => {
        const pmJSON = [
            {
                '@startTime': '2016-07-22T13:00:00.000+00:00:00',
                'PMMOResult': [{
                    'NE-WBTS_1.0': {
                        '@measurementType': 'LTE_S1AP',
                        'M8000C6': '0',
                        'M8000C7': '0'
                    }
                }]
            },
            {
                '@startTime': '2016-07-22T13:15:00.000+00:00:00',
                'PMMOResult': [{
                    'NE-WBTS_1.0': {
                        '@measurementType': 'LTE_S1AP',
                        'M8000C6': '10',
                        'M8000C7': '50'
                    }
                }]
            }
        ];
        const finalJSON = {
            'values': {
                'LTE_S1AP': {
                    'M8000C6': {
                        '2016-07-22T13:00:00.000+00:00:00': '0',
                        '2016-07-22T13:15:00.000+00:00:00': '10'
                    },
                    'M8000C7': {
                        '2016-07-22T13:00:00.000+00:00:00': '0',
                        '2016-07-22T13:15:00.000+00:00:00': '50'
                    }
                }
            },
            'startTimes': ['2016-07-22T13:00:00.000+00:00:00', '2016-07-22T13:15:00.000+00:00:00']
        };
        const startPmCounters = {
            values: {},
            startTimes: []
        };

        const pmCountersFormated = pmCounters.formatPmCounters(pmJSON, startPmCounters);
        const jsonHash = SparkMD5.hash(JSON.stringify(pmCountersFormated));
        const finalHash = SparkMD5.hash(JSON.stringify(finalJSON));
        assert.equal(jsonHash, finalHash);
    });
});
