const metaReader = require('../src/metaReader');

describe('MetaReader', function () {

    it('should load legacy meta', function (done) {
        loadBlob('base/resources/meta/meta_v17.zip', function (buffer) {
            metaReader.read(buffer, function (entries) {
                assert.lengthOf(entries, 4);
                assert.equal(countObjects(entries), 398);
                done();
            });
        })
    });

    it('should load meta', function (done) {
        loadBlob('base/resources/meta/meta.zip', function (buffer) {
            metaReader.read(buffer, function (entries) {
                assert.lengthOf(entries, 2);
                assert.equal(countObjects(entries), 692);
                done();
            });
        })
    });

    it('should load meta v20', function (done) {
        loadBlob('base/resources/meta/meta_v20.zip', function (buffer) {
            metaReader.read(buffer, function (entries) {
                assert.lengthOf(entries, 5);
                assert.equal(countObjects(entries), 549);
                done();
            });
        })
    });

    function countObjects(entries){
        return entries.map(function (metaEntry) {
            return metaEntry.objects.length;
        }).reduce(function(previous, current){
            return previous + current;
        }, 0);
    }

    function loadBlob(filePath, doneCallback) {
        var request = new XMLHttpRequest();
        request.open('GET', filePath, true);
        request.responseType = 'arraybuffer';

        request.onload = function () {
            if (this.status == 200) {
                var buffer = request.response;
                doneCallback(buffer);
            }
        };
        request.send();
    }
});













