const btsServiceDefaultListenerFactory = require('../../src/procedures/btsServiceDefaultListener');
const BTS_SERVICE_PROCEDURE_STATUS = require('../../src/procedures/btsServiceProcedureStatus');

describe('btsServiceDefaultListener', () => {
    let suit;
    beforeEach(() => {
        suit = {};
        suit.onSuccess = sinon.spy();
        suit.onFail = sinon.spy();
        suit.clientId = 12345;
        suit.listener = btsServiceDefaultListenerFactory.newInstance(suit.clientId, suit.onSuccess, suit.onFail);
    });

    describe('#update', () => {

        it('should do nothing when clientId is different', () => {
            const procedureUpdate = createProcedureUpdate(9000);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.deepEqual(suit.onFail.callCount, 0);
        });

        it('should call success action when status is COMPLETED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.COMPLETED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.isTrue(suit.onSuccess.calledOnce);
            assert.deepEqual(suit.onFail.callCount, 0);
        });

        it('should do nothing when status is ACCEPTED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.ACCEPTED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.deepEqual(suit.onFail.callCount, 0);
        });

        it('should do nothing when status is ENQUEUED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.ENQUEUED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.deepEqual(suit.onFail.callCount, 0);
        });

        it('should do nothing when status is REQUESTED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.REQUESTED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.deepEqual(suit.onFail.callCount, 0);
        });

        it('should call fail action when status is REJECTED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.REJECTED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.isTrue(suit.onFail.calledOnce);
        });

        it('should call fail action when status is FAILED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.FAILED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.isTrue(suit.onFail.calledOnce);
        });

        it('should call fail action when status is ABORTED', () => {
            const procedureUpdate = createProcedureUpdate(BTS_SERVICE_PROCEDURE_STATUS.ABORTED);
            const event = {
                objects: [procedureUpdate]
            };

            suit.listener.update(event);

            assert.deepEqual(suit.onSuccess.callCount, 0);
            assert.isTrue(suit.onFail.calledOnce);
        });

    });

    function createProcedureUpdate(status) {
        return {
            parameters: {
                procedureType: '',
                clientId: suit.clientId,
                status
            }
        };
    }
});
