const BtsServiceProcedure = require('../../src/procedures/btsServiceProcedure');
const ProcedureIdFactory = require('../../src/procedures/procedureIdFactory');
const ImTracker = require('../../src/imTracker/imTracker');
const OnlineImTree = require('../../src/onlineImTree');

describe('Service: btsServiceProcedure', function () {
    var btsServiceProcedure, onlineImTree, procedureIdFactory, decoderServiceStub;
    const metaServiceStub = {
        getLastMetaUid: function () {
            return 0;
        }
    }, BTS_SERVICE_PROCEDURE_DISTNAME = '/MRBTS-1/RAT-1/BTS_L-1/FUNC_L-1/BTS_SERVICE-1/BTS_SERVICE_PROCEDURE-1';

    beforeEach(function () {
        decoderServiceStub = (function () {
            var clientId, procedureType, status, targetDN;

            return {
                setClientId: function (value) {
                    clientId = value;
                },
                setProcedureType: function (value) {
                    procedureType = value;
                },
                setStatus: function (value) {
                    status = value;
                },
                setTargetDN: function (value) {
                    targetDN = value;
                },
                decodeByClassName: function () {
                    return {
                        clientId: clientId,
                        status: status,
                        procedureType: procedureType,
                        targetDN: targetDN
                    };
                }
            };
        })();

        onlineImTree = OnlineImTree({
            metaService: metaServiceStub,
            decoderService: decoderServiceStub
        });

        procedureIdFactory = ProcedureIdFactory({});
        btsServiceProcedure = BtsServiceProcedure(ImTracker({
            onlineImTree: onlineImTree
        }), procedureIdFactory);
    });

    it('should notify when matching object appears', function (done) {
        var objects;

        btsServiceProcedure.registerListener({
            update: function (event) {
                objects = event.objects;
            }
        }).then(function () {
            pushUpdate(procedureIdFactory.getId().id);
            assert.isDefined(objects);
            assert.lengthOf(objects, 1);
        }).then(done).catch(done);
    });

    it('should notify with objects that exists before', function (done) {
        var objects;

        startBtsProcedure()
            .then(function () {
                pushUpdate(procedureIdFactory.getId().id);

                return btsServiceProcedure.registerListener({
                    update: function (event) {
                        objects = event.objects;
                    }
                });
            })
            .then(function () {
                assert.isDefined(objects);
                assert.lengthOf(objects, 1);
            }).then(done).catch(done);
    });

    it('should not notify if objects where deleted', function (done) {
        var objects;

        startBtsProcedure()
            .then(function () {
                pushUpdate(procedureIdFactory.getId().id);
                pushRemove();
                return btsServiceProcedure.registerListener({
                    update: function (event) {
                        objects = event.objects;
                    }
                });
            })
            .then(function () {
                assert.isUndefined(objects);
            }).then(done).catch(done);
    });

    it('should not notify if clientId does not match', function (done) {
        var objects;

        startBtsProcedure()
            .then(function () {
                pushUpdate(procedureIdFactory.encodeId(procedureIdFactory.getClientId + 1, 1));
                return btsServiceProcedure.registerListener({
                    update: function (event) {
                        objects = event.objects;
                    }
                });
            })
            .then(function () {
                assert.isUndefined(objects);
            }).then(done).catch(done);
    });

    function startBtsProcedure() {
        return btsServiceProcedure.registerListener({update: noop});

        function noop() {
        }
    }

    function pushUpdate(clientId) {
        decoderServiceStub.setClientId(clientId);
        decoderServiceStub.setStatus('Completed');
        decoderServiceStub.setProcedureType('Block');
        decoderServiceStub.setTargetDN('/DISTNAME-1');
        const update = {distName: BTS_SERVICE_PROCEDURE_DISTNAME, payload: 1};
        onlineImTree.pushData({removedDistNames: [], managedObjects: [update]});
    }

    function pushRemove() {
        onlineImTree.pushData({removedDistNames: [BTS_SERVICE_PROCEDURE_DISTNAME], managedObjects: []});
    }
});
