"use strict";
describe('procedureIdFactory', function () {
    var procedureIdFactory;
    const MIN_CLIENT_ID = Math.pow(2, 18);
    const MEDIUM_CLIENT_ID = 1.5 * MIN_CLIENT_ID;
    const MAX_CLIENT_ID = (2 * MIN_CLIENT_ID) - 1;
    const MIN_PROCEDURE_ID = 0;
    const MAX_PROCEDURE_ID = Math.pow(2, 6) - 1;
    const MEDIUM_PROCEDURE_ID = Math.floor(MAX_PROCEDURE_ID / 2);
    const $localStorage = {clientId: MEDIUM_CLIENT_ID};

    beforeEach(function () {
        procedureIdFactory = adminCore.procedures.procedureIdFactory($localStorage);
        delete $localStorage.procedureId;
    });

    describe('decodeClientId method', function () {
        it('should return correct clientId for MAX_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MAX_CLIENT_ID, MIN_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MAX_CLIENT_ID);
        });

        it('should return correct clientId for MIN_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MIN_CLIENT_ID, MIN_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MIN_CLIENT_ID);
        });

        it('should return correct clientId for MEDIUM_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MIN_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MEDIUM_CLIENT_ID);
        });

        it('should return correct clientId for MAX_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MAX_CLIENT_ID, MAX_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MAX_CLIENT_ID);
        });

        it('should return correct clientId for MIN_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MIN_CLIENT_ID, MAX_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MIN_CLIENT_ID);
        });

        it('should return correct clientId for MEDIUM_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MAX_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MEDIUM_CLIENT_ID);
        });

        it('should return correct clientId for MAX_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MAX_CLIENT_ID, MEDIUM_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MAX_CLIENT_ID);
        });

        it('should return correct clientId for MIN_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MIN_CLIENT_ID, MEDIUM_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MIN_CLIENT_ID);
        });

        it('should return correct clientId for MEDIUM_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MEDIUM_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeClientId(completeId), MEDIUM_CLIENT_ID);
        });
    });

    describe('decodeProcedureId method', function () {
        it('should return correct procedureId for MAX_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MAX_CLIENT_ID, MIN_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MIN_PROCEDURE_ID);
        });

        it('should return correct procedureId for MIN_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MIN_CLIENT_ID, MIN_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MIN_PROCEDURE_ID);
        });

        it('should return correct procedureId for MEDIUM_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MIN_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MIN_PROCEDURE_ID);
        });

        it('should return correct procedureId for MAX_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MAX_CLIENT_ID, MEDIUM_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MEDIUM_PROCEDURE_ID);
        });

        it('should return correct procedureId for MIN_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MIN_CLIENT_ID, MEDIUM_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MEDIUM_PROCEDURE_ID);
        });

        it('should return correct procedureId for MEDIUM_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MEDIUM_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MEDIUM_PROCEDURE_ID);
        });

        it('should return correct procedureId for MAX_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MAX_CLIENT_ID, MAX_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MAX_PROCEDURE_ID);
        });

        it('should return correct procedureId for MIN_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MIN_CLIENT_ID, MAX_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MAX_PROCEDURE_ID);
        });

        it('should return correct procedureId for MEDIUM_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const completeId = procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MAX_PROCEDURE_ID);
            assert.strictEqual(procedureIdFactory.decodeProcedureId(completeId), MAX_PROCEDURE_ID);
        });
    });

    describe('encodeId method', function () {

        it('should return correct Id for MIN_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const EXPECTED_ID = 16777216;
            assert.strictEqual(procedureIdFactory.encodeId(MIN_CLIENT_ID, MIN_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MIN_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const EXPECTED_ID = 16777247;
            assert.strictEqual(procedureIdFactory.encodeId(MIN_CLIENT_ID, MEDIUM_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MIN_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const EXPECTED_ID = 16777279;
            assert.strictEqual(procedureIdFactory.encodeId(MIN_CLIENT_ID, MAX_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MEDIUM_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const EXPECTED_ID = 25165824;
            assert.strictEqual(procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MIN_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MEDIUM_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const EXPECTED_ID = 25165855;
            assert.strictEqual(procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MEDIUM_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MEDIUM_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const EXPECTED_ID = 25165887;
            assert.strictEqual(procedureIdFactory.encodeId(MEDIUM_CLIENT_ID, MAX_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MAX_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const EXPECTED_ID = 33554368;
            assert.strictEqual(procedureIdFactory.encodeId(MAX_CLIENT_ID, MIN_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MAX_CLIENT_ID and MEDIUM_PROCEDURE_ID', function () {
            const EXPECTED_ID = 33554399;
            assert.strictEqual(procedureIdFactory.encodeId(MAX_CLIENT_ID, MEDIUM_PROCEDURE_ID), EXPECTED_ID);
        });

        it('should return correct Id for MAX_CLIENT_ID and MAX_PROCEDURE_ID', function () {
            const EXPECTED_ID = 33554431;
            assert.strictEqual(procedureIdFactory.encodeId(MAX_CLIENT_ID, MAX_PROCEDURE_ID), EXPECTED_ID);
        });
    });

    describe('getId method', function () {

        it('should return clientId present in local storage', function () {
            assert.strictEqual(procedureIdFactory.getId().clientId, MEDIUM_CLIENT_ID);
        });

        it('should return MIN_PROCEDURE_ID if not present in local storage', function () {
            assert.strictEqual(procedureIdFactory.getId().procedureId, MIN_PROCEDURE_ID);
        });

        it('should return MIN_PROCEDURE_ID if value stored in local storage is equal MAX_PROCEDURE_ID', function () {
            $localStorage.procedureId = MAX_PROCEDURE_ID;
            assert.strictEqual(procedureIdFactory.getId().procedureId, MIN_PROCEDURE_ID);
        });

        it('should return 1 if value stored in local storage is equal MIN_PROCEDURE_ID', function () {
            $localStorage.procedureId = MIN_PROCEDURE_ID;
            assert.strictEqual(procedureIdFactory.getId().procedureId, 1);
        });

        it('should return incremented value of procedureId stored in local storage if lower then MAX_PROCEDURE_ID', function () {
            $localStorage.procedureId = MEDIUM_PROCEDURE_ID;
            assert.strictEqual(procedureIdFactory.getId().procedureId, MEDIUM_PROCEDURE_ID + 1);
        });

        it('should return correctly encoded id for MEDIUM_CLIENT_ID and MIN_PROCEDURE_ID', function () {
            const EXPECTED_ID = 25165824;
            const idObject = procedureIdFactory.getId();
            assert.strictEqual(idObject.clientId, MEDIUM_CLIENT_ID);
            assert.strictEqual(idObject.procedureId, MIN_PROCEDURE_ID);
            assert.strictEqual(idObject.id, EXPECTED_ID);
        });
    });

    describe('getClientId method', function () {
        it('should return incremented value of procedureId stored in local storage if lower then MAX_PROCEDURE_ID', function () {
            const clientId = procedureIdFactory.getId().clientId;
            assert.strictEqual(procedureIdFactory.getClientId(), clientId);
        });
    });
});
