(function () {
    'use strict';

    const ResetRequest = require('../../src/procedures/resetRequest');

    describe('ResetRequest', function () {

        const DISTNAME_1 = '/MRBTS-1/RAT-1/BTS_L-1/FUNC_L-1/RESET_REQ-1';
        const DISTNAME_2 = '/MRBTS-1/RAT-1/BTS_L-1/FUNC_L-1/RESET_REQ-2';
        const OBJECT_DN = '/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1';
        const OBJECT_DN_2 = '/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/LNCEL-21713';

        const parameters = {
            objectDN: OBJECT_DN
        };

        describe('Method: hasDistName', function () {
            it('should return true if request object has given distName', function () {
                const requestObject = ResetRequest.newInstance(DISTNAME_1, parameters);
                assert.ok(requestObject.hasDistName(DISTNAME_1));
            });

            it('should return false if request object hasn\'t given distName', function () {
                const requestObject = ResetRequest.newInstance(DISTNAME_1, parameters);
                assert.notOk(requestObject.hasDistName(DISTNAME_2));
            });

        });

        describe('Method: isEqual', function () {

            it('should return true if request objects have the same objectDN and distName', function () {
                const requestObject1 = ResetRequest.newInstance(DISTNAME_1, parameters);
                const requestObject2 = ResetRequest.newInstance(DISTNAME_1, parameters);
                assert.ok(requestObject1.isEqual(requestObject2));
            });

            it('should return false if request objects have different objectDN', function () {
                const parameters2 = {
                    objectDN: OBJECT_DN_2
                };
                const requestObject1 = ResetRequest.newInstance(DISTNAME_1, parameters);
                const requestObject2 = ResetRequest.newInstance(DISTNAME_1, parameters2);
                assert.notOk(requestObject1.isEqual(requestObject2));
            });

            it('should return false if request objects have different distName', function () {
                const requestObject1 = ResetRequest.newInstance(DISTNAME_1, parameters);
                const requestObject2 = ResetRequest.newInstance(DISTNAME_2, parameters);
                assert.notOk(requestObject1.isEqual(requestObject2));
            });
        });
    });
})();

