const assert = require('chai').assert;

const filesInfoFactory = require('../../../src/procedures/snapshot/filesInfoFactory.js');

describe('Service: filesInfo', function () {
    'use strict';

    var filesInfo;

    const EMPTY_FILE_INFO = {};
    const SUCCESFULL_FILE_INFO = {status: 'status-downloaded'};

    beforeEach(function () {
        filesInfo = filesInfoFactory().createFileInfo();
    });

    describe('createFileInfo', function () {
        it('should create files info', function () {
            assert.ok(filesInfo);
        });
    });

    describe('addFileInfoListener', function () {
        it('should return unwatcher', function () {
            const listener = function () {
            };
            const unwatcher = filesInfo.addFileInfoListener(listener);
            assert.ok(unwatcher);
        });

        it('should notify newly added listener', function () {
            const listener = function (filesInfoTable) {
                assert.strictEqual(filesInfoTable.length, 1);
                assert.deepEqual(filesInfoTable[0], EMPTY_FILE_INFO);
            };
            filesInfo.push(EMPTY_FILE_INFO);

            const unwatcher = filesInfo.addFileInfoListener(listener);
            assert.ok(unwatcher);
        });

        describe('push', function () {
            it('should add fileInfo for filesInfo', function () {
                const listener = function (filesInfoTable) {
                    assert.strictEqual(filesInfoTable.length, 1);
                    assert.deepEqual(filesInfoTable[0], EMPTY_FILE_INFO);
                };
                filesInfo.push(EMPTY_FILE_INFO);
                filesInfo.addFileInfoListener(listener);
            });
        });

        describe('isSuccessful', function () {
            it('should return true if all file infos has downloaded status', function () {
                filesInfo.push(SUCCESFULL_FILE_INFO);
                filesInfo.push(SUCCESFULL_FILE_INFO);
                assert.isTrue(filesInfo.isSuccessful());
            });

            it('should return false if at least one file info has error status', function () {
                const errorFileInfo = {status: 'status-error'};
                filesInfo.push(SUCCESFULL_FILE_INFO);
                filesInfo.push(errorFileInfo);
                assert.isFalse(filesInfo.isSuccessful());
            });
        });
    });

});
