const assert = require('chai').assert;

const snapshotTRAgent = require('../../../src/procedures/snapshot/snapshotTRAgent.js');

describe('Service: snapshotTRAgent', () => {

    const imTrackerStub = {
        trackByClassName: sinon.spy()
    };
    let snapshotTRAgentInstance = null;

    beforeEach(() => {
        snapshotTRAgentInstance = snapshotTRAgent(imTrackerStub);
    });
    afterEach(() => {
        snapshotTRAgentInstance = null;
    });

    describe('getActiveTRAgentsParents method', () => {
        it('should return an array with the class names of the active trAgents', () => {
            //given
            const dummyEvent = {
                objects: [{
                    distName: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/TRAGENT_L-1',
                    parameters: {
                        usageState: 'Active'
                    }
                }]
            };
            //when
            snapshotTRAgentInstance.parameterChanged(dummyEvent);
            //then
            assert.equal(snapshotTRAgentInstance.getActiveTRAgentsParents()[0], 'CCU_L', 'Class CCU_L not found as an tragent active parent!');
        });
    });

    describe('isSnapshotTRAgentActive method', () => {
        it('should return true if a TR Agent is present and has the active usage state', () => {
            //given
            const dummyEvent = {
                objects: [{
                    distName: 'elem1',
                    parameters: {
                        usageState: 'Active'
                    }
                }]
            };
            //when
            snapshotTRAgentInstance.parameterChanged(dummyEvent);
            //then
            assert.isTrue(snapshotTRAgentInstance.isSnapshotTRAgentActive());
        });

        it('should return false if a TR Agent is present and has the usage state not active', () => {
            //given
            const dummyEvent = {
                objects: [{
                    distName: 'elem1',
                    parameters: {
                        usageState: 'Inactive'
                    }
                }]
            };
            //when
            snapshotTRAgentInstance.parameterChanged(dummyEvent);
            //then
            assert.isFalse(snapshotTRAgentInstance.isSnapshotTRAgentActive());
        });

        it('should return false if a TR Agent usage state turns from active to not active', () => {
            //given
            const dummyEvent1 = {
                objects: [{
                    distName: 'elem1',
                    parameters: {
                        usageState: 'Active'
                    }
                }]
            };
            const dummyEvent2 = {
                objects: [{
                    distName: 'elem1',
                    parameters: {
                        usageState: 'Inactive'
                    }
                }]
            };
            //when
            snapshotTRAgentInstance.parameterChanged(dummyEvent1);
            snapshotTRAgentInstance.parameterChanged(dummyEvent2);
            //then
            assert.isFalse(snapshotTRAgentInstance.isSnapshotTRAgentActive());
        });

        it('should return false if an active TR Agent is removed from Info Model', () => {
            //given
            const dummyEvent = {
                objects: [{
                    distName: 'elem2',
                    parameters: {
                        usageState: 'Active'
                    }
                }]
            };
            const dummyObject = {
                distNames: ['elem2']
            };
            //when
            snapshotTRAgentInstance.parameterChanged(dummyEvent);
            snapshotTRAgentInstance.objectRemoved(dummyObject);
            //then
            assert.isFalse(snapshotTRAgentInstance.isSnapshotTRAgentActive());
        });
    });
});
