const zipFactory = require('../../../src/procedures/snapshot/zipFactory');
const assert = require('assert');
const Zlib = require('zlibjs/bin/unzip.min').Zlib;

describe('Factory: zipFactory', function () {
    'use strict';

    describe('newZip', function () {
        it('should return new zipping object', function () {
            const zip1 = zipFactory().newZip('zip1');
            const zip2 = zipFactory().newZip('zip2');
            assert.ok(zip1);
            assert.ok(zip2);
            assert.notEqual(zip1, zip2);
        });
    });

    describe('zip', function () {
        const data = new Uint8Array([ 116, 101, 115, 116 ]);
        const path = 'test/file';
        const file = 'test.zip';

        it('should create zip with data in appropriate named file', function (done) {
            var fileSaver = {
                saveAs: function (content, zipName) {
                    assert.equal(zipName, file);

                    var unzip = new Zlib.Unzip(content);
                    var unzipFile = unzip.getFilenames().pop();
                    assert.equal(path, unzipFile);

                    var unzipData = unzip.decompress(unzipFile);
                    assert.deepEqual(unzipData, data);

                    done();
                }
            };
            const zip = zipFactory(fileSaver).newZip(file);
            zip.addFile(data, path);
            zip.saveZip();
        });
    });
});
