const assert = require('assert');

const util = require('../../../test.util');
const targetbdParser = require('../../../../src/procedures/swUpdate/openSoftwareZip/targetbdParser');
const softwareZipDispatcherFactory = require('../../../../src/procedures/swUpdate/openSoftwareZip/softwareZipDispatcherFactory');

const ZIP_FILE = 'base/resources/zip/swUpdate.zip';

const newInstanceCopy = targetbdParser.newInstance;
const targetBdMock = {};
const targetbdParserInstanceMock = {
    getTargetBd: sinon.stub().returns(targetBdMock),
    isTargetBd: sinon.stub().returns(true)
};

targetbdParser.newInstance = sinon.stub().returns(targetbdParserInstanceMock);

describe('Factory: softwareZipDispatcherFactory', function () {
    const fileName = '/file';
    const fileContent = new Uint8Array([ 99, 111, 110, 116, 101, 110, 116, 10 ]);

    var softwareZipDispatcherInstance;
    var zip;
    before(function (done) {
        util.loadBuffer(ZIP_FILE).then((buffer) => {
            zip = buffer;
            done();
        });
    });

    beforeEach(function () {
        softwareZipDispatcherInstance = softwareZipDispatcherFactory();
    });

    after(function () {
        targetbdParser.newInstance = newInstanceCopy;
    });

    describe('isDispatcherCreated', function () {
        it('should return true if dispatcher whose created', function () {
            assert.equal(softwareZipDispatcherInstance.isDispatcherCreated(), false);
            softwareZipDispatcherInstance.fromZipFile(zip);
            assert(targetbdParser.newInstance.calledOnce);
            assert.equal(softwareZipDispatcherInstance.isDispatcherCreated(), true);
        });
    });

    describe('getDispatcher', function () {
        it('should return dispatcher object', function () {
            softwareZipDispatcherInstance.fromZipFile(zip);
            const dispatcher = softwareZipDispatcherInstance.getDispatcher();

            assert(dispatcher.isZipLoaded);
            assert(dispatcher.getTargetBd);
        });
    });

    describe('dispatcher', function () {
        describe('isZipLoaded', function () {
            it('should return true if zip whose loaded', function () {
                softwareZipDispatcherInstance.fromZipFile(zip);

                const dispatcher = softwareZipDispatcherInstance.getDispatcher();
                assert(dispatcher.isZipLoaded());
                assert(targetbdParserInstanceMock.isTargetBd.calledOnce);
            });
        });

        describe('getTargetBd', function () {
            it('should return dispatcher', function () {
                softwareZipDispatcherInstance.fromZipFile(zip);

                const dispatcher = softwareZipDispatcherInstance.getDispatcher();
                assert.equal(dispatcher.getTargetBd(), targetBdMock);
            });
        });

        describe('getFile', function () {
            it('should return file as ArrayBuffer', function () {
                softwareZipDispatcherInstance.fromZipFile(zip);
                const dispatcher = softwareZipDispatcherInstance.getDispatcher();

                var file = dispatcher.getFile(fileName);
                assert(file instanceof ArrayBuffer);
                var uint8Array = new Uint8Array(file);
                assert.deepEqual(uint8Array, fileContent);
            });
        });
    });
});
