const assert = require('assert');
const swListDownloader = require('../../../../src/procedures/swUpdate/selectSoftwareServer/swListDownloader');
const Promise = require('bluebird');

describe('Factory: swListDownloader', function () {
    const data = {};
    const address = '1.1.1.1';
    const softwareServer = {
        hostname: 'hostname',
        port: 777,
        protocol: 'protocol'
    };

    var webdavStub;
    var swListDownloaderInstance;

    beforeEach(function () {
        webdavStub = {
            propfind: sinon.stub().returns(Promise.resolve(data))
        };

        swListDownloaderInstance = swListDownloader(webdavStub);
    });

    describe('getSoftwareList', function () {
        it('should return resolved promise and set connectionAvailability to true', function (done) {
            swListDownloaderInstance.getSoftwareList(address)
                .then(function (received) {
                    assert(webdavStub.propfind.calledWith(address));
                    assert.equal(received, data);

                    done();
                });
        });

        it('should return rejected promise and set connectionAvailability to false', function (done) {
            const error = new Error();
            webdavStub.propfind = sinon.stub().returns(Promise.reject(error));

            const swListDownloaderReject = swListDownloader(webdavStub);
            swListDownloaderReject.getSoftwareList(address)
                .catch(function (reason) {
                    assert.equal(reason, error);
                    assert.equal(swListDownloaderReject.isConnectionAvailable(), false);

                    done();
                });
        });
    });

    describe('getHostname', function () {
        it('should return host name', function () {
            swListDownloaderInstance.setSoftwareServer(softwareServer);

            const hostname = swListDownloaderInstance.getHostname();
            assert.equal(softwareServer.hostname, hostname);
        });
    });

    describe('setSoftwareServer', function () {
        it('should set default address', function (done) {
            const defaultAddress = softwareServer.protocol + '//' + softwareServer.hostname + ':' + softwareServer.port;

            swListDownloaderInstance.setSoftwareServer(softwareServer);
            swListDownloaderInstance.getSoftwareList()
                .then(function () {
                    assert(webdavStub.propfind.calledWith(defaultAddress));

                    done();
                });
        });
    });

    describe('isConnectionAvailable', function () {
        it('should return a value based on state of the connection', function (done) {
            assert.equal(swListDownloaderInstance.isConnectionAvailable(), false);

            swListDownloaderInstance.getSoftwareList(address)
                .then(function () {
                    assert.equal(swListDownloaderInstance.isConnectionAvailable(), true);

                    webdavStub.propfind = sinon.stub().returns(Promise.reject());
                    swListDownloaderInstance.getSoftwareList(address)
                        .catch(function () {
                            assert.equal(swListDownloaderInstance.isConnectionAvailable(), false);

                            done();
                        });
                });
        });
    });
});
