const sequentialExecutorFactory = require('../src/sequentialExecutorFactory');

describe('sequentialExecutorFactory', function () {

    var sequentialExecutor;
    var obj;

    beforeEach(function () {
        sequentialExecutor = sequentialExecutorFactory.newInstance();

        obj = {};
        obj.syncFun = function () {
            return "syncFun";
        };
        obj.asyncFun = function () {
            return new Promise(function (resolve) {
                setTimeout(function () {
                    resolve("asyncFun");
                }, 200);
            });
        };
    });

    it('should wrap sync function', function (done) {
        var syncFun = sequentialExecutor.promisifyFunction(obj, obj.syncFun);

        syncFun().then(function (result) {
            assert.equal(result, "syncFun");
            done();
        });

    });

    it('should wrap async function', function (done) {
        var asyncFun = sequentialExecutor.promisifyFunction(obj, obj.asyncFun);

        asyncFun().then(function (result) {
            assert.equal(result, "asyncFun");
            done();
        });

    });

    it('should execute function is sequential order', function (done) {
        var order = [];
        obj.syncFun = function () {
            order.push("syncFun");
        };
        obj.asyncFun = function () {
            return new Promise(function (resolve) {
                setTimeout(function () {
                    order.push("asyncFun");
                    resolve();
                }, 200);
            });
        };

        var asyncFun = sequentialExecutor.promisifyFunction(obj, obj.asyncFun);
        var syncFun = sequentialExecutor.promisifyFunction(obj, obj.syncFun);

        Promise.join(asyncFun(), syncFun(), asyncFun(), syncFun()).then(function () {
            assert.deepEqual(order, ["asyncFun", "syncFun", "asyncFun", "syncFun"]);
            done();
        });
    });

    it('should bind "this" variable to host object', function (done) {
        obj.x = null;
        obj.setX = function (value) {
            this.x = value;
        };

        var setX = sequentialExecutor.promisifyFunction(obj, obj.setX);

        setX("Hello").then(function () {
            assert.equal(obj.x, "Hello");
            done();
        });
    });

    it('should rethrow ex', function (done) {
        obj.error = function () {
            throw Error("Bum!");
        };

        var error = sequentialExecutor.promisifyFunction(obj, obj.error);

        error().catch(function (e) {
            assert.ok(e);
            done();
        });
    });

});
