const currentSettingsService = require('../../src/settings/currentSettingsService');

describe('Service: currentSettings', () => {

    let currentSettings;

    beforeEach(() => {
        const localStorage = {};
        currentSettings = currentSettingsService({localStorage: localStorage});
    });

    describe('Method: hideNotSupportedBrowserWarning', () => {
        it('should return default false value if settings are not stored in $localStorage yet', () => {
            assert.notOk(currentSettings.hideNotSupportedBrowserWarning());
        });

        it('should set hideNotSupportedBrowserWarning flag to true', () => {
            currentSettings.hideNotSupportedBrowserWarning(true);
            assert.ok(currentSettings.get().hideNotSupportedBrowserWarning);
        });

        it('should set hideNotSupportedBrowserWarning flag to false', () => {
            currentSettings.hideNotSupportedBrowserWarning(false);
            assert.notOk(currentSettings.get().hideNotSupportedBrowserWarning);
        });
    });

    describe('Method: updateInterval', () => {
        it('should return default value 5000 if settings are not stored in $localStorage yet', () => {
            assert.equal(currentSettings.updateInterval(), 5000);
        });

        it('should set update interval to 1000', () => {
            currentSettings.updateInterval(1000);
            assert.equal(currentSettings.get().updateInterval, 1000);
        });
    });

    describe('Method: equals', () => {
        it('should return true for identical settings', () => {
            const expectedSettings = {
                hideNotSupportedBrowserWarning: true,
                updateInterval: 3000,
                screenLockTimeout: 3000
            };
            currentSettings.hideNotSupportedBrowserWarning(true);
            currentSettings.updateInterval(3000);
            currentSettings.updateScreenLockTimeout(3000);
            assert.ok(currentSettings.equals(expectedSettings));
        });

        it('should return false for different settings', () => {
            const settings = {
                hideNotSupportedBrowserWarning: true,
                updateInterval: 1000
            };
            currentSettings.hideNotSupportedBrowserWarning(true);
            currentSettings.updateInterval(3000);
            assert.notOk(currentSettings.equals(settings));
        });
    });
});
