const commissioningStatusResolverFactory = require('../../../src/site/commissioningStatus/commissioningStatusResolverFactory'),
    ImTree = require('../../../src/imtree');

describe('commissioning status', function () {
    'use strict';


    const STATUSES = {
        COMMISSIONED: 'COMMISSIONED',
        NOT_INSTALLED: 'NOT_INSTALLED',
        UNCOMMISSIONED: 'UNCOMMISSIONED'
    };

    var statusResolver, imTree;
    beforeEach(function () {
        statusResolver = commissioningStatusResolverFactory;
        imTree = new ImTree();
        fillImTreeWithDefaultModules();
    });

    function fillImTreeWithDefaultModules() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1', {
            currentNp: '/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1'
        });
    }

    function fillImTreeWithCommissionedModules() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1/RU_L-1', {
            stateInfo: {configurationStatus: 'Configured'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/FTM-1', {
            stateInfo: {proceduralStatus: 'Configured'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/TRS_DATA-1', {
            stateInfo: {proceduralState: 'activated'}
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1', {
            stateInfo: {operationalState: 'Enabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/BBMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1', {
            stateInfo: {operationalState: 'Enabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/MHA_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1', {
            stateInfo: {operationalState: 'Enabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RET_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1'
        });
    }

    function fillImTreeWithUncommissionedModules() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1/RU_L-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/FTM-1', {
            stateInfo: {proceduralStatus: 'NotConfigured'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/TRS_DATA-1', {
            stateInfo: {proceduralState: 'activated'}
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/BBMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-2'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/MHA_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-2'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RET_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-2'
        });
    }

    function fillImTreeWithNotInstalledModules() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1/RU_L-1', {
            stateInfo: {configurationStatus: 'Unconfigured'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1', {
            stateInfo: {operationalState: 'Disabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/BBMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1', {
            stateInfo: {operationalState: 'Disabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/MHA_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1'
        });

        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1', {
            stateInfo: {operationalState: 'Disabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RET_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1'
        });
    }

    it('should get commissioned status for rmod', function () {
        fillImTreeWithCommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1';
        assert.equal(statusResolver.getCommissioningData(distName, imTree).status, STATUSES.COMMISSIONED);
    });

    it('should get uncommissioned status for rmod', function () {
        // given
        fillImTreeWithUncommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCOMMISSIONED');
    });

    it('should get "not installed" status for rmod - no RU_L object', function () {
        // given
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1'
        });
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.NOT_INSTALLED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.NO_RUL_OBJECT');
    });

    it('should get "not installed" status for rmod', function () {
        // given
        fillImTreeWithNotInstalledModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.NOT_INSTALLED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCONFIGURED');
    });

    it('should get commissioned status for transport', function () {
        fillImTreeWithCommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/TRU_L-1';
        assert.equal(statusResolver.getCommissioningData(distName, imTree).status, STATUSES.COMMISSIONED);
    });

    it('should get uncommissioned status for transport', function () {
        // given
        fillImTreeWithUncommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/TRU_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'FTM.proceduralStatus: NotConfigured');
    });

    it('should get uncommissioned status for transport - no FTM object', function () {
        // given
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/TRU_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.NO_FTM_OBJECT');
    });

    it('should get uncommissioned status for transport - no TRS_DATA object', function () {
        // given
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/FTM-1', {
            stateInfo: {proceduralStatus: 'Configured'}
        });
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/TRU_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.NO_TRSDATA_OBJECT');
    });

    it('should get commissioned status for bbmod', function () {
        fillImTreeWithCommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1';
        assert.equal(statusResolver.getCommissioningData(distName, imTree).status, STATUSES.COMMISSIONED);
    });

    it('should get commissioned status for bbmod when bbmodA points to different bbmodL', function () {
        addBbmodAPoitingToDifferentBbmodL();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-2';
        assert.equal(statusResolver.getCommissioningData(distName, imTree).status, STATUSES.COMMISSIONED);
    });

    function addBbmodAPoitingToDifferentBbmodL() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-2', {
            stateInfo: {operationalState: 'Enabled'}
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/BBMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-2'
        });
    }

    it('should get uncommissioned status for bbmod', function () {
        // given
        fillImTreeWithUncommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCOMMISSIONED');
    });

    it('should get uncommissioned status for bbmod - no EQM_A object', function () {
        // given
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1', {});
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCOMMISSIONED');
    });

    it('should get uncommissioned status for bbmod when bbmodA points to non-existing bbmodL', function () {
        // given
        mergeBbmodAPoitingToNonexistingBbmodL();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCOMMISSIONED');
    });

    function mergeBbmodAPoitingToNonexistingBbmodL() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1', {
            operationalState: 'Enabled'
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/BBMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-2'
        });
    }

    it('should get "not installed" status for bbmod', function () {
        // given
        fillImTreeWithNotInstalledModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1/BBMOD_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.NOT_INSTALLED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCONFIGURED');
    });

    it('should get commissioned status for mha', function () {
        fillImTreeWithCommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1';
        assert.equal(statusResolver.getCommissioningData(distName, imTree).status, STATUSES.COMMISSIONED);
    });

    it('should get uncommissioned status for mha', function () {
        // given
        fillImTreeWithUncommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCOMMISSIONED');
    });

    it('should get "not installed" status for mha', function () {
        // given
        fillImTreeWithNotInstalledModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/MHA_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.NOT_INSTALLED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCONFIGURED');
    });

    it('should get commissioned status for ret', function () {
        fillImTreeWithCommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1';
        assert.equal(statusResolver.getCommissioningData(distName, imTree).status, STATUSES.COMMISSIONED);
    });

    it('should get uncommissioned status for ret', function () {
        // given
        fillImTreeWithUncommissionedModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.UNCOMMISSIONED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCOMMISSIONED');
    });

    it('should get "not installed" status for ret', function () {
        // given
        fillImTreeWithNotInstalledModules();
        var distName = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RET_L-1';
        // when
        var commissioningData = statusResolver.getCommissioningData(distName, imTree);
        // then
        assert.equal(commissioningData.status, STATUSES.NOT_INSTALLED);
        assert.equal(commissioningData.errorMsg, 'imc.views.site.commissioningStatus.UNCONFIGURED');
    });
});
