const linksFactory = require('../../../src/site/factories/linksFactory'),
    ImTree = require('../../../src/imtree'),
    imTreeNodeFactory = require('../../../src/site/imTreeNodeFactory');


describe('links factory', () => {
    const HWTOP_L_DISTNAME = '/MRBTS-1/RAT-1/HWTOP_L-1';
    const CABINET_DISTNAME = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-1';

    let imTree,
        eqmL,
        hwtopL;

    beforeEach(() => {
        imTree = new ImTree();
        hwtopL = imTree.mergeNode(HWTOP_L_DISTNAME, {});
        eqmL = imTree.mergeNode('/MRBTS-1/RAT-1/EQM_L-1', {});
    });

    describe('fake links', () => {

        it('Should return empty array if tree is empty', () => {
            const linkModel = linksFactory.create(imTree);

            assert.lengthOf(linkModel.links, 0);
        });

        it('Should create link between smod and bbmod if they are connected via cabinet', () => {
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-1',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-2',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);

            const linkModel = linksFactory.create(imTree);

            assertLink(linkModel.links[0],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1`,
                {configuration: false, data: false, dc: false, synchronization: false}
            );
            assert.lengthOf(linkModel.links, 1);
        });

        it('Should create only one link between smod and bbmod despite number of connections between them', () => {
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addLink(`${CABINET_DISTNAME}/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-2',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-3',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-4',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-2`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-2`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-2/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);

            const linkModel = linksFactory.create(imTree);

            assert.lengthOf(linkModel.links, 1);
        });

        it('Should create link between smods and all its bbmods', () => {
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-3/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-2/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-1',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-2',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);

            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-3',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-3/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-2/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-2`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-3/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);

            const linkModel = linksFactory.create(imTree);

            assert.lengthOf(linkModel.links, 2);
            assertLink(linkModel.links[0],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1`,
                {configuration: false, data: false, dc: false, synchronization: false});
            assertLink(linkModel.links[1],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-2/CONCENTRATOR_L-1`,
                {configuration: false, data: false, dc: false, synchronization: false});
        });

        it('should merge linkMode between cablnks', () => {
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-1',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                {configuration: true});
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-2',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2`,
                {data: true});
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-3',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-2`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);

            const linkModel = linksFactory.create(imTree);

            assert.lengthOf(linkModel.links, 1);
            assertLink(linkModel.links[0],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1`,
                {configuration: true, data: true, dc: false, synchronization: false});
        });

        it('should merge linkMode between hwlinks', () => {
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-1',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-2',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-3',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`,
                {dc: true});
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-2`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`,
                {synchronization: true});

            const linkModel = linksFactory.create(imTree);

            assert.lengthOf(linkModel.links, 1);
            assertLink(linkModel.links[0],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1`,
                {configuration: false, data: false, dc: true, synchronization: true});
        });

        it('should not merge linkMode if connections are not between the same modules', () => {
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/CONCENTRATOR_L-3/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);
            addNodeWithParents(`${CABINET_DISTNAME}/BBMOD_L-2/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`);

            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-1',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-2',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2`,
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-2`);

            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-3',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1/CABLNK_L-4',
                `${CABINET_DISTNAME}/CONCENTRATOR_L-3/CONNECTOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-2/CONCENTRATOR_L-1/CONNECTOR_L-1`);
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-2/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-1/HWPORT_L-1`,
                {dc: true});
            addLink(`${CABINET_DISTNAME}/HWTOP_L-1/HWLINK_L-2`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-3/CONNECTOR_L-1/HWPORT_L-1`,
                `${CABINET_DISTNAME}/CONCENTRATOR_L-1/CONNECTOR_L-2/HWPORT_L-1`,
                {synchronization: true});

            const linkModel = linksFactory.create(imTree);

            assert.lengthOf(linkModel.links, 2);
            assertLink(linkModel.links[0],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-1/CONCENTRATOR_L-1`,
                {configuration: false, data: false, dc: true, synchronization: false});
            assertLink(linkModel.links[1],
                `${CABINET_DISTNAME}/SMOD_L-1/CONCENTRATOR_L-1`,
                `${CABINET_DISTNAME}/BBMOD_L-2/CONCENTRATOR_L-1`,
                {configuration: false, data: false, dc: false, synchronization: true});
        });

        function addLink(linkDistName, destination, source, linkMode) {
            linkMode = linkMode || {};
            imTree.mergeNode(linkDistName, {
                destinationDistName: destination,
                sourceDistName: source,
                linkMode
            });
        }

        function addNodeWithParents(distName) {
            let parent = '';
            const distNamePath = distName.split('/');

            distNamePath.forEach((distNamePart) => {
                parent += distNamePart;
                imTree.mergeNode(parent, {});
                parent += '/';
            });
        }

        function assertLink(link, source, target, linkMode) {
            assert.equal(link.source.distName, source);
            assert.equal(link.target.distName, target);
            assert.deepEqual(link.linkMode, linkMode);
        }
    });

    it('should create links', () => {
        const module = createModule('SMOD_L', 1);
        const source = createBackEndConnector(module, 1);
        const target = createFrontEndConnector(module, 2);

        createLink(1, source, target);

        const link = linksFactory.create(imTree).links[0];

        assert.deepEqual(link.source.distName, source.distName);
        assert.deepEqual(link.source.direction, 'Right');
        assert.deepEqual(link.target.distName, target.distName);
        assert.deepEqual(link.target.direction, 'Left');
    });

    it('should create link with backend connector direction Right', () => {
        const module = createModule('SMOD_L', 1);
        const source = createBackEndConnector(module, 1);
        const target = createBackEndConnector(module, 2);

        createLink(1, source, target);

        const link = linksFactory.create(imTree).links[0];

        assert.deepEqual(link.source.direction, 'Right');
        assert.deepEqual(link.target.direction, 'Right');
    });

    it('should create link with frontend connector direction Left', () => {
        const module = createModule('SMOD_L', 1);
        const source = createFrontEndConnector(module, 1);
        const target = createFrontEndConnector(module, 2);

        createLink(1, source, target);

        const link = linksFactory.create(imTree).links[0];

        assert.deepEqual(link.source.direction, 'Left');
        assert.deepEqual(link.target.direction, 'Left');
    });

    function createFrontEndConnector(parentNode, connectorNumber) {
        return imTreeNodeFactory.createFrontEndConnector(imTree, parentNode, connectorNumber);
    }

    function createBackEndConnector(parentNode, connectorNumber) {
        return imTreeNodeFactory.createBackEndConnector(imTree, parentNode, connectorNumber);
    }

    function createModule(className, id, parameters, parentNode) {
        return imTreeNodeFactory.createModule(imTree, eqmL, className, id, parameters, parentNode);
    }

    function createLink(linkNumber, source, target) {
        return imTreeNodeFactory.createLink(imTree, hwtopL, linkNumber, source, target);
    }
});
