const siteConcentratorsFactory = require('../../../src/site/factories/siteConcentratorsFactory'),
    ImTree = require('../../../src/imtree');


describe('siteConcentratorsFactory', () => {

    let imTree;
    beforeEach(() => {
        imTree = new ImTree();
    });

    describe('#create', () => {
        it('should return empty array if there is no concentrators', () => {
            // given
            const containerNode = imTree.mergeNode('/CABINET_L-1/BBMOD_L-1', {});

            // when
            const concentrators = siteConcentratorsFactory.create(containerNode);

            // then
            assert.lengthOf(concentrators, 0);
        });

        it('should create concentrators for direct descendants', () => {
            // given
            const containerNode = imTree.mergeNode('/CABINET_L-1/BBMOD_L-1', {});
            imTree.mergeNode('/CABINET_L-1/BBMOD_L-1/CONCENTRATOR_L-1', {});
            imTree.mergeNode('/CABINET_L-1/BBMOD_L-1/CONCENTRATOR_L-2', {});
            imTree.mergeNode('/CABINET_L-1/BBMOD_L-1/BBU_L-1/CONCENTRATOR_L-1', {});

            // when
            const concentrators = siteConcentratorsFactory.create(containerNode);

            // then
            assert.lengthOf(concentrators, 2);
        });

        it('should create concentrators with proper structure', () => {
            // given
            const containerNode = imTree.mergeNode('/CABINET_L-1/BBMOD_L-1', {});
            const concentratorType = 'RP3',
                positionInRack = 'RightSide',
                slotNr = 1,
                stateInfo = {administrativeState: 'Enabled'},
                alarms = [];

            const concentratorDistName = '/CABINET_L-1/BBMOD_L-1/CONCENTRATOR_L-1';
            imTree.mergeNode(concentratorDistName, {
                concentratorType: concentratorType,
                positionInRack: positionInRack,
                slotNr: slotNr,
                stateInfo: stateInfo,
                alarms: alarms
            });

            // when
            const concentrators = siteConcentratorsFactory.create(containerNode);

            // then
            assert.lengthOf(concentrators, 1);
            const concentrator = concentrators[0];
            assert.equal(concentrator.distName, concentratorDistName);
            assert.equal(concentrator.type, concentratorType);
            assert.equal(concentrator.side, positionInRack);
            assert.equal(concentrator.slot, slotNr);
            assert.deepEqual(concentrator.stateInfo, stateInfo);
            assert.deepEqual(concentrator.alarms, alarms);
        });
    });
});
