const siteModuleFactory = require('../../../src/site/factories/siteModuleFactory'),
    ImTree = require('../../../src/imtree'),
    imTreeNodeFactoryService = require('../../../src/site/imTreeNodeFactory');

(function () {
    'use strict';
    describe('siteModuleFactory', function () {

        var factory,
            imTree,
            imTreeNodeFactory;

        var eqmL;

        beforeEach(function () {
            factory = siteModuleFactory;
            imTreeNodeFactory = imTreeNodeFactoryService;

            imTree = new ImTree();
            eqmL = imTree.mergeNode('/MRBTS-1/RAT-1/EQM_L-1', {});
        });

        describe('RAU', function () {
            const RAE_DISTNAME = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RAE_L-1';
            const RAE_CONNECTOR_DISTNAME = RAE_DISTNAME + '/CONNECTOR_L-1';
            var raeNode;

            beforeEach(function () {
                raeNode = imTree.mergeNode(RAE_DISTNAME, {
                    productCode: 'pc1',
                    serialNumber: 'sn1',
                    stateInfo: {availabilityStatus: 'Online'}
                });
                imTree.mergeNode(RAE_CONNECTOR_DISTNAME, {});
            });

            it('should create REA module', function () {
                const raeModule = factory.createRaeModule(raeNode);

                assert.equal(raeModule.name, 'RAE_L-1');
                assert.equal(raeModule.distName, RAE_DISTNAME);
                assert.equal(raeModule.className, 'RAE_L');
                assert.equal(raeModule.number, 1);
                assert.equal(raeModule.pc, 'pc1');
                assert.equal(raeModule.sn, 'sn1');
                assert.deepEqual(raeModule.stateInfo, {availabilityStatus: 'Online'});
                assert.lengthOf(raeModule.leftConnectors, 0);
                assert.lengthOf(raeModule.rightConnectors, 1);
                assert.lengthOf(raeModule.children, 0);
                assert.lengthOf(raeModule.alarms, 0);
            });

            it('should create REA module wirh RAEU', function () {
                const RAEU_DISTNAME = RAE_DISTNAME + '/RAEU_L-1';
                imTree.mergeNode(RAEU_DISTNAME, {stateInfo: {availabilityStatus: 'Online'}});

                const raeModule = factory.createRaeModule(raeNode);
                const raeuModule = raeModule.children[0];

                assert.equal(raeuModule.name, 'RAEU_L-1');
                assert.equal(raeuModule.distName, RAEU_DISTNAME);
                assert.equal(raeuModule.className, 'RAEU_L');
                assert.equal(raeuModule.number, 1);
                assert.deepEqual(raeuModule.stateInfo, {availabilityStatus: 'Online'});
                assert.lengthOf(raeuModule.leftConnectors, 0);
                assert.lengthOf(raeuModule.rightConnectors, 0);
                assert.lengthOf(raeuModule.children, 0);
                assert.lengthOf(raeuModule.alarms, 0);
            });

        });

        describe('MHA', function () {

            ['HWLINK_L', 'HWLINK_NG_L'].forEach(function (hwLinkName) {
                it('should create one MHA', function () {
                    const mhaNode = createModule('MHA_L', 1);
                    const hwtop = createModule('HWTOP_L', 1, {}, mhaNode);
                    const lna1 = createModule('LNA_L', 1, {}, mhaNode);
                    const lna2 = createModule('LNA_L', 2, {}, mhaNode);
                    const mhaConnector1 = createFrontEndConnector(mhaNode, 1);
                    const mhaConnector2 = createFrontEndConnector(mhaNode, 2);
                    const mhaConnector3 = createBackEndConnector(mhaNode, 3);
                    const mhaConnector4 = createBackEndConnector(mhaNode, 4);

                    const lna1Hwport1 = createHwport(lna1, 1);
                    const lna1Hwport2 = createHwport(lna1, 2);
                    const lna2Hwport1 = createHwport(lna2, 1);
                    const lna2Hwport2 = createHwport(lna2, 2);
                    const mhaConnector1Hwport1 = createHwport(mhaConnector1, 1);
                    const mhaConnector2Hwport1 = createHwport(mhaConnector2, 1);
                    const mhaConnector3Hwport1 = createHwport(mhaConnector3, 1);
                    const mhaConnector4Hwport1 = createHwport(mhaConnector4, 1);

                    createHwlink(1, hwtop, mhaConnector1Hwport1, lna1Hwport1, hwLinkName);
                    createHwlink(2, hwtop, mhaConnector3Hwport1, lna1Hwport2, hwLinkName);
                    createHwlink(3, hwtop, mhaConnector2Hwport1, lna2Hwport1, hwLinkName);
                    createHwlink(4, hwtop, mhaConnector4Hwport1, lna2Hwport2, hwLinkName);

                    const mhaModule = factory.createMhaModule(mhaNode, imTree);

                    assert.deepEqual(mhaModule.distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1');
                    assert.deepEqual(mhaModule.children.length, 2);
                    const lna1Module = mhaModule.children[0];
                    const lna2Module = mhaModule.children[1];
                    assert.deepEqual(lna1Module.distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1/LNA_L-1');
                    assert.deepEqual(lna2Module.distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1/LNA_L-2');

                    assert.deepEqual(lna1Module.leftConnectors.length, 1);
                    assert.deepEqual(lna1Module.leftConnectors[0].distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1/CONNECTOR_L-1');
                    assert.deepEqual(lna1Module.rightConnectors.length, 1);
                    assert.deepEqual(lna1Module.rightConnectors[0].distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1/CONNECTOR_L-3');
                    assert.deepEqual(lna2Module.leftConnectors.length, 1);
                    assert.deepEqual(lna2Module.leftConnectors[0].distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1/CONNECTOR_L-2');
                    assert.deepEqual(lna2Module.rightConnectors.length, 1);
                    assert.deepEqual(lna2Module.rightConnectors[0].distName, '/MRBTS-1/RAT-1/EQM_L-1/MHA_L-1/CONNECTOR_L-4');
                });

            });

            function createHwlink(linkId, hwtop, source, target, hwLinkName) {
                const node = imTree.addNode(hwtop, hwLinkName, linkId);
                node.parameters = {
                    sourceDistName: source.distName,
                    destinationDistName: target.distName
                };
                return node;
            }

            function createHwport(parentNode, hwportNumber) {
                const node = imTree.addNode(parentNode, 'HWPORT_L', hwportNumber);
                node.parameters = {};
                return node;
            }

            function createFrontEndConnector(parentNode, connectorNumber) {
                return imTreeNodeFactory.createFrontEndConnector(imTree, parentNode, connectorNumber);
            }

            function createBackEndConnector(parentNode, connectorNumber) {
                return imTreeNodeFactory.createBackEndConnector(imTree, parentNode, connectorNumber);
            }

        });

        describe('generic module', function () {

            it('should get visible product code when available', function () {
                const smodNode = createModule('SMOD_L', 1, {
                    serialNumber: '111',
                    visibleProductCode: '222',
                    productCode: '333',
                    productName: 'pn'
                });

                const smod = factory.createGenericModule(smodNode);

                assert.deepEqual(smod.name, 'SMOD_L-1');
                assert.deepEqual(smod.sn, '111');
                assert.deepEqual(smod.pc, '222');
                assert.deepEqual(smod.pn, 'pn');
            });

            it('should create module with undefined values when required properties are missing', function () {
                const smodNode = createModule('SMOD_L', 1);

                const smod = factory.createGenericModule(smodNode);

                assert(!smod.sn);
                assert(!smod.pc);
            });
        });

        function createModule(className, id, parameters, parentNode) {
            return imTreeNodeFactory.createModule(imTree, eqmL, className, id, parameters, parentNode);
        }
    });
})();
