const siteModuleFactoryProvider = require('../../../src/site/factories/siteModuleFactory'),
    factoryProvider = require('../../../src/site/factories/siteModulesFactory'),
    ImTree = require('../../../src/imtree');

describe('site-modules-factory', function () {
    var factory, siteModuleFactory, moduleFactories, imTree;

    beforeEach(function () {
        factory = factoryProvider;
        siteModuleFactory = siteModuleFactoryProvider;

    });

    beforeEach(function () {
        imTree = new ImTree();
        moduleFactories = {
            generic: siteModuleFactory.createGenericModule
        };
    });

    describe('#createModules', function () {
        it('should create pwrmod-module', function () {
            imTree.mergeNode('/MRBTS-1/RAT-1/EQM_L-1/PWRMOD_L-1', {});
            const modules = factory.createModules(imTree, moduleFactories);

            assert.lengthOf(modules.pwrmods, 1);
        });

        it('should create pdu-module', function () {
            imTree.mergeNode('/MRBTS-1/RAT-1/EQM_L-1/PWRMOD_L-1', {});
            imTree.mergeNode('/MRBTS-1/RAT-1/EQM_L-1/PWRMOD_L-1/PDU_L-1', {
                productCode: '472084A.203',
                productName: 'FPFD'
            });

            const modules = factory.createModules(imTree, moduleFactories);

            assert.lengthOf(modules.pwrmods, 1);
            assert.lengthOf(modules.pwrmods[0].children, 1);
            assert.equal(modules.pwrmods[0].children[0].pc, '472084A.203');
            assert.equal(modules.pwrmods[0].children[0].pn, 'FPFD');
        });
    });
});
