const hardwareSorterProvider = require('../../../src/site/siteHardware/siteHardwareSorterService');

describe('Service: siteHardwareSorterService', function () {
    'use strict';

    var hardwareSorter;
    beforeEach(function () {
        hardwareSorter = hardwareSorterProvider;
    });

    var LAST = 1000000000;
    describe('compute position', function () {
        var module1;
        var module2;

        beforeEach(function () {
            module1 = {distName: '/SMOD_L-1', number: 1};
            module2 = {distName: '/RMOD_L-1', number: 1};
        });

        it('should compute position of module based on graph connections', function () {
            var graph = {
                adj: function () {
                    return [{distName: module1.distName, module: module1}];
                }
            };
            module1.position = 1000;
            hardwareSorter.computePosition([module2], graph);

            assert.deepEqual(module2.position, 1 + module1.position);
        });

        it('should have last position when modules are not connected', function () {
            var graph = {
                adj: function () {
                    return [];
                }
            };

            hardwareSorter.computePosition([module1], graph);

            assert.deepEqual(module1.position, 1 + LAST);
        });

        it('should compute position of depended modules', function () {
            var graph = {
                adj: function (distName) {
                    if (distName === module1.distName) return [{distName: module2.distName, module: module2}];
                    if (distName === module2.distName) return [{distName: module1.distName, module: module1}];
                    return [];
                }
            };
            module1.position = 1;
            module2.position = 2;
            hardwareSorter.computePosition([module1, module2], graph);

            assert.deepEqual(module1.position, 1 + 2);
            assert.deepEqual(module2.position, 2 + 3);
        });

    });


});
