'use strict';

const TypeInfoV1 = require('../src/typeInfo');
const TypeInfoV2 = require('../src/typeInfoV2');

function loadJsonFile(path) {
    return new Promise(function (resolve) {
        var request = new XMLHttpRequest();
        request.open('GET', path, true);
        request.responseType = 'json';
        request.onload = function () {
            if (this.status == 200) resolve(request.response);
        };
        request.send();
    });
}

function createLoadTypeInfo(TypeInfoConstructor){
    return function(file){
        return new Promise(function (resolve, reject) {
            loadJsonFile(file)
                .then(function (json) {
                    resolve(TypeInfoConstructor(json));
                }).catch(function (e) {
                    reject(e);
                })
        });
    }
}
function loadBuffer(filePath) {
    return new Promise(function (resolve) {
        loadBlob(filePath, function (blob) {
            blobToArrayBuffer(blob, function (arrayBuffer) {
                resolve(arrayBuffer);
            });
        });
    });
}

function blobToArrayBuffer(blob, doneCallback) {
    var fileReader = new FileReader();
    fileReader.onload = function () {
        var arrayBuffer = this.result;
        doneCallback(arrayBuffer);
    };
    fileReader.readAsArrayBuffer(blob);
}

function loadBlob(filePath, doneCallback) {
    var request = new XMLHttpRequest();
    request.open('GET', filePath, true);
    request.responseType = 'blob';

    request.onload = function (event) {
        if (this.status == 200) {
            var blob = this.response;
            doneCallback(blob);
        }
    };
    request.send();
}

module.exports = {
    loadJsonFile: loadJsonFile,
    loadTypeInfo: createLoadTypeInfo(TypeInfoV1),
    loadTypeInfoV2: createLoadTypeInfo(TypeInfoV2),
    loadBuffer: loadBuffer
};
