const timingService = require('../../src/timings/timingService');
const ImTree = require('../../src/imtree');

describe('timingService', () => {

    let updateService;
    let timingServiceInstance;
    let imTree;

    function createUpdateService() {
        let listener;
        return {
            addImTreeListener,
            pushTree,
            removeImTreeListener
        };
        function addImTreeListener(theListener) {
            listener = theListener;
        }
        function pushTree(im) {
            if (listener)
                listener.completeImTree(im);
        }
        function removeImTreeListener() {
            listener = null;
        }
    }

    beforeEach(() => {
        updateService = createUpdateService();
        timingServiceInstance = timingService.newInstance(updateService);
        imTree = new ImTree();
    });

    it('should remove listener', () => {
        let counter = 0;
        function imTreeListener() {
            counter++;
            assert.equal(counter, 1);
        }

        timingServiceInstance.addListener(imTreeListener);
        updateService.pushTree(imTree);
        timingServiceInstance.removeListener(imTreeListener);

        imTree.mergeNode('/MRBTS-1/XYZ_L-1', {});
        updateService.pushTree(imTree);
        imTree.mergeNode('/MRBTS-1/XYZw_L-1', {});
        updateService.pushTree(imTree);
        imTree.mergeNode('/MRBTS-1/XYZw_L-1', {});
        updateService.pushTree(imTree);
        imTree.mergeNode('/MRBTS-1/XYZw_L-1', {});
        updateService.pushTree(imTree);

    });

    it('should return empty list for empty imTree', (done) => {

        timingServiceInstance.addListener(updateTimings);
        updateService.pushTree(imTree);

        function updateTimings(im) {
            const timings = timingServiceInstance.findTimingsForPattern('', im);
            assert.equal(timings.length, 0);
            done();
        }
    });

    it('should return empty list if there is no TIMINGS_L-1 object in imTree', (done) => {

        timingServiceInstance.addListener(updateTimings);

        imTree.mergeNode('/MRBTS-1/XYZ_L-1', {});
        imTree.mergeNode('/MRBTS-1/XYZw_L-1', {});
        updateService.pushTree(imTree);

        function updateTimings(im) {
            const timings = timingServiceInstance.findTimingsForPattern('', im);
            assert.equal(timings.length, 0);
            done();
        }
    });

    it('should return all timings objects in imTree in respect to given pattern', (done) => {

        timingServiceInstance.addListener(updateTimings);
        imTree.mergeNode('/MRBTS-1/AAAA_L-1', {
            procedureName: 'procName'
        });
        imTree.mergeNode('/MRBTS-1/TIMINGS_L-1', {
            procedureName: 'procName'
        });
        imTree.mergeNode('/MRBTS-1/XYZ_A-2/TIMINGS_L-2', {
            procedureName: 'procName'
        });
        imTree.mergeNode('/MRBTS-1/ABC_R-666/TIMINGS_L-2', {
            procedureName: 'procName'
        });
        updateService.pushTree(imTree);

        function updateTimings(im) {
            const timings1 = timingServiceInstance.findTimingsForPattern('', im);
            assert.equal(timings1.length, 3);
            const timings2 = timingServiceInstance.findTimingsForPattern('XYZ_A', im);
            assert.equal(timings2.length, 1);
            const timings3 = timingServiceInstance.findTimingsForPattern('procName', im);
            assert.equal(timings3.length, 3);
            done();
        }
    });
});
