describe('complex parameters v1', function () {
    var meta;

    var assert = chai.assert;

    beforeEach(function () {
        meta = {p: []};
    });

    it('should get undefined when inner parameter not exist', function () {
        meta.p.push({
            "@name": "empty",
            "complexType": {}
        });

        meta.p.push({
            "@name": "null",
            "complexType": null
        });

        meta.p.push({
            "@name": "pnull",
            "complexType": {
                p: null
            }
        });

        var emptyParameter = parameterByName("empty");
        assert.isTrue(emptyParameter.isComplex());
        assert.notOk(emptyParameter.getParameterByName("any name"));

        var nullParameter = parameterByName("null");
        assert.isTrue(nullParameter.isComplex());
        assert.notOk(nullParameter.getParameterByName("any name"));

        var pNullParameter = parameterByName("pnull");
        assert.isTrue(pNullParameter.isComplex());
        assert.notOk(pNullParameter.getParameterByName("any name"));
    });

    it('should get inner parameter', function () {
        meta.p.push({
            "@name": "complex",
            "complexType": {
                "p": {
                    "@name": "simple",
                    "simpleType": {"@base": "integer"}
                }
            }
        });

        var complex = parameterByName("complex");
        assert.isTrue(complex.isComplex());

        var simple = complex.getParameterByName("simple");
        assert.isTrue(simple.isSimple());
    });

    it('should get all inner parameters', function () {
        meta.p.push({
            "@name": "complex",
            "complexType": {
                "p": [
                    {
                        "@name": "simple1",
                        "simpleType": {"@base": "integer"}
                    },
                    {
                        "@name": "simple2",
                        "simpleType": {"@base": "integer"}
                    }
                ]
            }
        });

        var complex = parameterByName("complex");
        assert.isTrue(complex.isComplex());

        var simple1 = complex.getParameterByName("simple1");
        assert.isTrue(simple1.isSimple());

        var simple2 = complex.getParameterByName("simple2");
        assert.isTrue(simple2.isSimple());
    });

    it('should get all parameters', function () {
        meta.p.push({
            "@name": "complex",
            "complexType": {
                "p": [
                    {
                        "@name": "simple1",
                        "simpleType": {"@base": "integer"}
                    },
                    {
                        "@name": "simple2",
                        "simpleType": {"@base": "integer"}
                    }
                ]
            }
        });

        var complex = parameterByName("complex");
        assert.lengthOf(complex.getParameters(), 2);
    });

    it('should throw when getType is called', function () {
        meta.p.push({
            "@name": "complex",
            "complexType": {
                "p": [
                    {
                        "@name": "simple",
                        "simpleType": {"@base": "integer"}
                    }
                ]
            }
        });

        var complex = parameterByName('complex');
        assert.throws(function () {
            complex.getBaseType();
        });
    });

    function create(metaDefinition) {
        return infoModel.typeInfo(metaDefinition);
    }

    function parameterByName(name) {
        return create(meta).getParameterByName(name);
    }

});
