(function (global) {
    'use strict';
    describe('complex parameters v2', function () {
        var meta;

        var assert = chai.assert;

        beforeEach(function () {
            meta = {p: [], struct: [], enumeration: []};
        });

        it('should get undefined when inner parameter not exist', function () {
            meta.struct.push({
                '@name': 'Empty',
                p: []
            });

            meta.p.push({
                '@id': 1,
                '@name': 'empty',
                '@type': 'Empty'
            });

            meta.p.push({
                '@id': 2,
                '@name': 'null',
                '@type': null
            });

            meta.struct.push({
                '@name': 'PNull',
                p: null
            });

            meta.p.push({
                '@id': 3,
                '@name': 'pnull',
                '@type': 'PNull'
            });

            var emptyParameter = parameterByName('empty');
            assert.isTrue(emptyParameter.isComplex());
            assert.notOk(emptyParameter.getParameterByName('any name'));

            var nullParameter = parameterByName('null');
            assert.isFalse(nullParameter.isComplex());
            assert.notOk(nullParameter.getParameterByName('any name'));

            var pNullParameter = parameterByName('pnull');
            assert.isTrue(pNullParameter.isComplex());
            assert.notOk(pNullParameter.getParameterByName('any name'));
        });

        it('should get inner parameter', function () {
            meta.struct.push({
                '@name': 'MyComplexType',
                p: [{
                    '@id': 111,
                    '@name': 'inner',
                    '@type': 'integer'
                }]
            });

            meta.p.push({
                '@id': 1,
                '@name': 'complex',
                '@type': 'myComplexType'
            });

            var complex = parameterByName('complex');
            assert.isTrue(complex.isComplex());

            var simple = complex.getParameterByName('inner');
            assert.isTrue(simple.isSimple());
        });

        it('should handle definitions at top level', function () {
            meta.enumeration.push({
                '@name': 'MyInnerEnum',
                'enum': [
                    {'@value': '-1', '@name': 'X'},
                    {'@value': '0', '@name': 'Y'},
                    {'@value': '1', '@name': 'Z'}
                ]
            });

            meta.struct.push({
                '@name': 'MyInnerType1',
                p: [{
                    '@id': 11,
                    '@name': 'inner1',
                    '@type': 'MyInnerType2'
                }]
            }, {
                '@name': 'MyInnerType2',
                p: [{
                    '@id': 111,
                    '@name': 'inner2',
                    '@type': 'integer'
                }, {
                    '@id': 112,
                    '@name': 'innerE',
                    '@type': 'MyInnerEnum'
                }]
            });

            meta.p.push({
                '@id': 1,
                '@name': 'complex',
                '@type': 'MyInnerType1'
            });

            var complex = parameterByName('complex');
            assert.isTrue(complex.isComplex());

            var inner1 = complex.getParameterByName('inner1');
            assert.isTrue(inner1.isComplex());

            var inner2 = inner1.getParameterByName('inner2');
            assert.isTrue(inner2.isSimple());

            var innerE = inner1.getParameterByName('innerE');
            assert.isTrue(innerE.isEnum());
        });

        it('should get all inner parameters', function () {
            meta.struct.push({
                '@name': 'MyComplexType',
                p: [{
                    '@id': 111,
                    '@name': 'inner1',
                    '@type': 'integer'
                }, {
                    '@id': 112,
                    '@name': 'inner2',
                    '@type': 'integer'
                }]
            });

            meta.p.push({
                '@id': 1,
                '@name': 'complex',
                '@type': 'MyComplexType'
            });

            var complex = parameterByName('complex');
            assert.isTrue(complex.isComplex());

            var simple1 = complex.getParameterByName('inner1');
            assert.equal(simple1.getId(), 111);

            var simple2 = complex.getParameterByName('inner2');
            assert.equal(simple2.getId(), 112);

            assert.lengthOf(complex.getParameters(), 2);
        });

        it('should throw when getType is called', function () {
            meta.struct.push({
                '@name': 'MyComplexType',
                p: [{
                    '@id': 111,
                    '@name': 'inner',
                    '@type': 'integer'
                }]
            });

            meta.p.push({
                '@id': 1,
                '@name': 'complex',
                '@type': 'MyComplexType'
            });

            var complex = parameterByName('complex');
            assert.throws(function () {
                complex.getBaseType();
            });
        });

        function create(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition);
        }

        function parameterByName(name) {
            return create(meta).getParameterByName(name);
        }
    });
})(this);
