(function (global) {
    'use strict';

    describe('create empty parameters v2', function () {
        var meta;

        beforeEach(function () {
            meta = {p: [], struct: []};
        });

        it('should create empty simple', function () {
            meta.p.push({
                '@id': 1,
                '@name': 'any name',
                '@type': 'integer'
            });
            assert.deepEqual(createEmptyParameters(meta), {
                'any name': null
            });
        });

        it('should create empty simple array', function () {
            meta.p.push({
                '@id': 1,
                '@name': 'any name',
                '@type': 'integer',
                '@recurrence': 'repeated'
            });
            assert.deepEqual(createEmptyParameters(meta), {
                'any name': []
            });
        });

        it('should create empty complex', function () {
            meta.struct.push({
                '@name': 'MyType',
                p: [{
                    '@id': 111,
                    '@name': 'simple1',
                    '@type': 'integer'
                }, {
                    '@id': 112,
                    '@name': 'simple2',
                    '@type': 'integer'
                }]
            });

            meta.p.push({
                '@id': 1,
                '@name': 'complex',
                '@type': 'MyType'
            });
            assert.deepEqual(createEmptyParameters(meta), {
                'complex': {
                    'simple1': null,
                    'simple2': null
                }
            });
        });

        it('should create empty complex array', function () {
            meta.struct.push({
                '@name': 'MyType',
                p: [{
                    '@id': 111,
                    '@name': 'integer1',
                    '@type': 'integer'
                }, {
                    '@id': 112,
                    '@name': 'integer2',
                    '@type': 'integer'
                }]
            });

            meta.p.push({
                '@id': 1,
                '@name': 'complexArrayInnerTwo',
                '@recurrence': 'repeated',
                '@type': 'MyType'
            });
            assert.deepEqual(createEmptyParameters(meta), {'complexArrayInnerTwo': []});
        });

        function createEmptyParameters(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition).createEmptyParameters();
        }

    });
})(this);
