(function (global) {
    'use strict';
    describe('get default value v2', function () {

        var meta;

        beforeEach(function () {
            meta = {p: [], enumeration: []};
        });

        it('should get string value', function () {
            meta.p.push({
                '@id': 1,
                '@type': 'string',
                '@default': 'UNDEFINED'
            });

            assert.strictEqual(firstParameter().getDefaultValue(), 'UNDEFINED');
        });

        it('should get integer value', function () {
            meta.p.push({
                '@id': 1,
                '@type': 'integer',
                '@default': '100'
            });

            assert.strictEqual(firstParameter().getDefaultValue(), 100);
        });

        it('should get double value', function () {
            meta.p.push({
                '@id': 1,
                '@type': 'double',
                '@default': '-14.5'
            });

            assert.strictEqual(firstParameter().getDefaultValue(), -14.5);
        });

        it('should get true boolean value', function () {
            meta.p.push({
                '@id': 1,
                '@type': 'boolean',
                '@default': 'true'
            });

            assert.isTrue(firstParameter().getDefaultValue());
        });

        it('should get false boolean value', function () {
            meta.p.push({
                '@id': 1,
                '@type': 'boolean',
                '@default': 'false'
            });

            assert.isFalse(firstParameter().getDefaultValue());
        });

        it('should get enum value', function () {
            meta.enumeration.push({
                '@name': 'EmyEnum',
                'enum': [
                    {'@value': '0', '@name': 'SnapshotFailed'}
                ]
            });
            meta.p.push({
                '@id': 1,
                '@type': 'EmyEnum',
                '@default': '0'
            });

            assert.strictEqual(firstParameter().getDefaultValue(), 0);
        });

        function create(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition);
        }

        function firstParameter() {
            return create(meta).getParameters()[0];
        }

    });
})(this);
