describe('typeInfo v1', function () {
    var meta;

    beforeEach(function () {
        meta = {p: []};
    });

    describe('legacy enum', function () {
        var simpleType;

        beforeEach(function () {
            simpleType = {
                "@base": "integer",
                "enumeration": [
                    {"@value": "-1", "@text": "X"},
                    {"@value": "0", "@text": "Y"},
                    {"@value": "1", "@text": "Z"}
                ]
            };
            meta.p.push({"simpleType": simpleType});
        });

        it('should get enum text based on enum integer value', function () {
            var typeInfo = firstParameter();
            assert.strictEqual(typeInfo.getEnumText(-1), "X");
            assert.strictEqual(typeInfo.getEnumText(0), "Y");
            assert.strictEqual(typeInfo.getEnumText(1), "Z");
        });

        it('should get enum value based on enum text', function () {
            var typeInfo = firstParameter();
            assert.strictEqual(typeInfo.getEnumValue("X"), -1);
            assert.strictEqual(typeInfo.getEnumValue("Y"), 0);
            assert.strictEqual(typeInfo.getEnumValue("Z"), 1);
        });

        it('should get enum possible text', function () {
            var typeInfo = firstParameter();
            assert.deepEqual(typeInfo.getEnumPossibleText(), ["X", "Y", "Z"]);
        });

        it('should get enum text based on enum integer value - special chars', function () {
            simpleType = {
                "@base": "integer",
                "enumeration": [
                    {
                        "@value": "0",
                        "@text": "__Space__A__Asterisk__B__Dot__C__Minus____Plus__"
                    }
                ]
            };
            meta.p = [{"simpleType": simpleType}];

            assert.strictEqual(firstParameter().getEnumText(0), " A*B.C-+");
        });

        it('should get enum text based on enum integer value - multiple the same special chars', function () {
            simpleType = {
                "@base": "integer",
                "enumeration": [
                    {
                        "@value": "0",
                        "@text": "__Space__A__Space__B__Space__"
                    }
                ]
            };
            meta.p = [{"simpleType": simpleType}];

            assert.strictEqual(firstParameter().getEnumText(0), " A B ");
        });


        it('should get enum value based on enum text - special chars', function () {
            simpleType = {
                "@base": "integer",
                "enumeration": [
                    {
                        "@value": "0",
                        "@text": "__Space__A__Asterisk__B__Dot__C__Minus____Plus__"
                    }
                ]
            };
            meta.p = [{"simpleType": simpleType}];

            assert.strictEqual(firstParameter().getEnumValue(" A*B.C-+"), 0);
        });

    });

    function create(metaDefinition) {
        return infoModel.typeInfo(metaDefinition);
    }

    function firstParameter() {
        return create(meta).getParameters()[0];
    }

});
