(function (global) {
    'use strict';

    describe('typeInfo v2', function () {
        var meta;

        beforeEach(function () {
            meta = {p: [], enumeration: []};
        });

        describe('enum', function () {
            beforeEach(function () {
                const enumeration = {
                    '@name': 'EmyEnum',
                    'enum': [
                        {'@value': '-1', '@name': 'X'},
                        {'@value': '0', '@name': 'Y'},
                        {'@value': '1', '@name': 'Z'}
                    ]
                };

                const parameter = {
                    '@id': 1,
                    '@type': 'EMyEnum'
                };
                meta.enumeration.push(enumeration);
                meta.p.push(parameter);
            });

            it('should get enum text based on enum integer value', function () {
                var typeInfo = firstParameter();
                assert.strictEqual(typeInfo.getEnumText(-1), "X");
                assert.strictEqual(typeInfo.getEnumText(0), "Y");
                assert.strictEqual(typeInfo.getEnumText(1), "Z");
            });

            it('should get enum value based on enum text', function () {
                var typeInfo = firstParameter();
                assert.strictEqual(typeInfo.getEnumValue("X"), -1);
                assert.strictEqual(typeInfo.getEnumValue("Y"), 0);
                assert.strictEqual(typeInfo.getEnumValue("Z"), 1);
            });

            it('should get enum possible text', function () {
                var typeInfo = firstParameter();
                assert.deepEqual(typeInfo.getEnumPossibleText(), ["X", "Y", "Z"]);
            });

            it('should get enum text based on enum integer value - special chars', function () {
                const enumeration = {
                    '@name': 'EmySpecialEnum',
                    'enum': [
                        {'@value': '0', '@name': '__Space__A__Asterisk__B__Dot__C__Minus____Plus__'}
                    ]
                };

                const parameter = {
                    '@id': 1,
                    '@type': 'EmySpecialEnum'
                };

                meta.p = [parameter];
                meta.enumeration = [enumeration];

                assert.strictEqual(firstParameter().getEnumText(0), " A*B.C-+");
            });

            it('should get enum text based on enum integer value - multiple the same special chars', function () {
                const enumeration = {
                    '@name': 'EmySpecialEnum',
                    'enum': [
                        {'@value': '0', '@name': '__Space__A__Space__B__Space__'}
                    ]
                };

                const parameter = {
                    '@id': 1,
                    '@type': 'EmySpecialEnum'
                };

                meta.p = [parameter];
                meta.enumeration = [enumeration];

                assert.strictEqual(firstParameter().getEnumText(0), " A B ");
            });


            it('should get enum value based on enum text - special chars', function () {
                const enumeration = {
                    '@name': 'EmySpecialEnum',
                    'enum': [
                        {'@value': '0', '@name': '__Space__A__Asterisk__B__Dot__C__Minus____Plus__'}
                    ]
                };

                const parameter = {
                    '@id': 1,
                    '@type': 'EmySpecialEnum'
                };

                meta.p = [parameter];
                meta.enumeration = [enumeration];
                assert.strictEqual(firstParameter().getEnumValue(" A*B.C-+"), 0);
            });

        });

        function create(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition);
        }

        function firstParameter() {
            return create(meta).getParameters()[0];
        }
    });
})(this);
