describe('exclude meta voodoo stuff v1', function () {
    var meta;

    beforeEach(function () {
        meta = {p: []};
    });

    describe('case of "MO MinOccurs"', function () {

        it('should exclude when productData contains "MO MinOccurs"', function () {
            meta.p = [
                {
                    "@name": "SHOULD_BE_EXCLUDED",
                    "creation": {"@priority": "mandatory"},
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "MO MinOccurs",
                            "@value": "0",
                            "@dataType": "Number"
                        }
                    },
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.deepEqual(parameters(), []);
        });

        it('should exclude only when productData contains "MO MinOccurs"', function () {
            meta.p = [
                {
                    "@name": "SHOULD_BE_EXCLUDED",
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "MO MinOccurs",
                            "@value": "0",
                            "@dataType": "Number"
                        }
                    },
                    "simpleType": {
                        "@base": "integer"
                    }
                },
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "simpleType": {
                        "@base": "integer"
                    }
                }

            ];
            assert.equal(firstParameter().getId(), 1);
        });

        it('should not exclude when productData exist', function () {
            meta.p = [
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "productData": null,
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.lengthOf(parameters(), 1);
        });

        it('should not exclude when productData not contain "MO MinOccurs"', function () {
            meta.p = [
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "XXX"
                        }
                    },
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.lengthOf(parameters(), 1);
        });

    });

    describe('case of hidden "isVendorSpecific"', function () {

        it('should exclude when parameter is hidden and  has simpleType and isVendorSpecific is set to true', function () {
            meta.p = [
                {
                    "@name": "SHOULD_BE_EXCLUDED",
                    "@hidden": "true",
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "isVendorSpecific",
                            "@value": "false"
                        }
                    },
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.deepEqual(parameters(), []);
        });

        it('should not exclude when isVendorSpecific is set to false', function () {
            meta.p = [
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "@hidden": "true",
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "isVendorSpecific",
                            "@value": "true"
                        }
                    },
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.lengthOf(parameters(), 1);
        });

        it('should not exclude when parameter is not hidden', function () {
            meta.p = [
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "@hidden": "false",
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "isVendorSpecific",
                            "@value": "true"
                        }
                    },
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.lengthOf(parameters(), 1);
        });

        it('should not exclude when parameter is complex', function () {
            meta.p = [
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "@hidden": "true",
                    "productData": {
                        "@name": "LTE BTS",
                        "property": {
                            "@name": "isVendorSpecific",
                            "@value": "false"
                        }
                    },
                    "complexType": null
                }
            ];
            assert.lengthOf(parameters(), 1);
        });

    });

    describe('case of hidden', function () {

        it('should exclude when parameter is hidden and simpleType', function () {
            meta.p = [
                {
                    "@name": "SHOULD_BE_EXCLUDED",
                    "@hidden": "true",
                    "simpleType": {
                        "@base": "integer"
                    }
                }
            ];
            assert.lengthOf(parameters(), 0);
        });

        it('should not exclude when parameter is hidden and complexType', function () {
            meta.p = [
                {
                    "@name": "SHOULD_NOT_BE_EXCLUDED",
                    "@hidden": "true",
                    "complexType": null
                }
            ];
            assert.lengthOf(parameters(), 1);
        });

    });

    function create(metaDefinition) {
        return infoModel.typeInfo(metaDefinition);
    }

    function firstParameter() {
        return create(meta).getParameters()[0];
    }

    function parameters() {
        return create(meta).getParameters();
    }

});
