(function (global) {
    'use strict';
    describe('managed object information v2', function () {

        var typeInfo;
        var meta;

        beforeEach(function () {
            meta = {p: []};
        });

        it('should get class name', function () {
            typeInfo = create({'@class': 'SMOD'});
            assert.strictEqual(typeInfo.getClassName(), 'SMOD');
        });

        it('should get id', function () {
            typeInfo = create({'@id': 5});
            assert.strictEqual(typeInfo.getId(), 5);
        });

        it('should return false when is not hidden', function () {
            typeInfo = create({'@id': 0});
            assert.isFalse(typeInfo.isHidden());
        });

        it('should return true when is hidden', function () {
            typeInfo = create({});
            assert.isTrue(typeInfo.isHidden());
        });

        it('should return empty array of child managed objects', function () {
            typeInfo = create({'childManagedObject': null});
            assert.deepEqual(typeInfo.getChildManagedObjects(), []);

            typeInfo = create({});
            assert.deepEqual(typeInfo.getChildManagedObjects(), []);
        });

        it('should return array of child managed object', function () {
            typeInfo = create({'childManagedObject': {'@class': 'A'}});
            assert.deepEqual(typeInfo.getChildManagedObjects(), ['A']);
        });

        it('should return array of child managed objects', function () {
            typeInfo = create({
                'childManagedObject': [
                    {'@class': 'A'}, {'@class': 'B'}, {'@class': 'C'}
                ]
            });
            assert.deepEqual(typeInfo.getChildManagedObjects(), ['A', 'B', 'C']);
        });

        function create(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition);
        }
    });
})(this);
