(function (global) {
    'use strict';

    describe('get operations type info v2', function () {

        var typeInfo;
        var meta;

        beforeEach(function () {
            meta = {p: [], action: []};
        });

        it('should return [] when operations not exist', function () {
            typeInfo = create({'p': null});
            assert.deepEqual(typeInfo.getOperations(), []);
        });

        it('should return name', function () {
            meta.action.push({
                '@name': 'takeOwnership',
                p: []
            });

            assert.strictEqual(firstOperation().getName(), 'takeOwnership');
        });

        it('should return true if is operation', function () {
            meta.action.push({
                '@name': 'takeOwnership',
                p: []
            });

            assert.isTrue(firstOperation().isOperation());
        });

        it('should return parameters', function () {
            meta.action.push({
                '@name': 'takeOwnership',
                struct: [{
                    '@name': 'MyParam'
                }],
                p: [{
                    '@id': 111,
                    '@name': 'inner1',
                    '@type': 'integer'
                }, {
                    '@id': 112,
                    '@name': 'inner2',
                    '@type': 'MyParam'
                }]
            });

            var operation = firstOperation();
            assert.strictEqual(operation.getName(), 'takeOwnership');
            assert.isTrue(operation.getParameterByName('inner1').isSimple());
            assert.isTrue(operation.getParameterByName('inner2').isComplex());
        });

        function create(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition);
        }

        function firstOperation() {
            return create(meta).getOperations()[0];
        }
    });
})(this);

