describe('typeInfo v1', function () {
    var meta;

    beforeEach(function () {
        meta = {p: []};
    });

    describe('legacy range of simple type', function () {
        var simpleType;

        describe('integer', function () {

            beforeEach(function () {
                simpleType = {
                    "@base": "integer",
                    "editing": {
                        "range": {
                            "@minIncl": "0",
                            "@maxIncl": "65535",
                            "@step": 5
                        }
                    }
                };
                meta.p.push({"simpleType": simpleType});
            });


            it('should get range inside editing', function () {
                var typeInfo = firstParameter();
                assert.isTrue(typeInfo.isInteger());
                assert.deepEqual(typeInfo.getRange(), {min: 0, max: 65535, step: 5});
            });

            it('should get range outside of editing', function () {
                var range = simpleType.editing.range;
                simpleType.editing = undefined;
                simpleType.range = range;

                assert.deepEqual(firstParameter().getRange(), {min: 0, max: 65535, step: 5});
            });

            it('should get null min range when min is undefined', function () {
                simpleType.editing.range["@minIncl"] = undefined;

                assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535, step: 5});
            });

            it('should get null max range when min is undefined', function () {
                simpleType.editing.range["@maxIncl"] = undefined;

                assert.deepEqual(firstParameter().getRange(), {min: 0, max: null, step: 5});
            });

            it('should get null step when step is undefined', function () {
                simpleType.editing.range["@step"] = undefined;

                assert.deepEqual(firstParameter().getRange(), {min: 0, max: 65535, step: null});
            });
        });

        describe('string with editing tag', function () {

            beforeEach(function () {
                simpleType = {
                    "@base": "string",
                    "editing": {
                        "minLength": {"@value": "0"},
                        "maxLength": {"@value": "65535"}
                    }
                };
                meta.p.push({"simpleType": simpleType});
            });


            it('should get range inside editing', function () {
                var typeInfo = firstParameter();
                assert.isTrue(typeInfo.isString());
                assert.deepEqual(typeInfo.getRange(), {min: 0, max: 65535, step: null});
            });

            it('should get null when min is undefined', function () {
                simpleType.editing.minLength = undefined;
                assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535, step: null});
            });

            it('should get null when max is undefined', function () {
                simpleType.editing.maxLength = undefined;
                assert.deepEqual(firstParameter().getRange(), {min: 0, max: null, step: null});
            });

            it('should get min and max based on length value', function () {
                simpleType = {
                    "@base": "string",
                    "length": {"@value": "15"}
                };
                meta.p = [{"simpleType": simpleType}];

                assert.deepEqual(firstParameter().getRange(), {min: 15, max: 15, step: null});
            });

        });

        describe('string without editing tag', function () {
            beforeEach(function () {
                simpleType = {
                    "@base": "string",
                    "minLength": {"@value": "0"},
                    "maxLength": {"@value": "65535"}

                };
                meta.p.push({"simpleType": simpleType});
            });


            it('should get range', function () {
                var typeInfo = firstParameter();
                assert.isTrue(typeInfo.isString());
                assert.deepEqual(typeInfo.getRange(), {min: 0, max: 65535, step: null});
            });

            it('should get null when min is undefined', function () {
                simpleType.minLength = undefined;
                assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535, step: null});
            });

            it('should get null when max is undefined', function () {
                simpleType.maxLength = undefined;
                assert.deepEqual(firstParameter().getRange(), {min: 0, max: null, step: null});
            });

            it('should get min and max based on length value', function () {
                simpleType = {
                    "@base": "string",
                    "length": {"@value": "15"}
                };
                meta.p = [{"simpleType": simpleType}];

                assert.deepEqual(firstParameter().getRange(), {min: 15, max: 15, step: null});
            });
        });

        describe('decimal', function () {

            beforeEach(function () {
                simpleType = {
                    "@base": "decimal",
                    "editing": {
                        "range": {
                            "@minIncl": "0",
                            "@maxIncl": "65535",
                            "@step": 0.5
                        }
                    }
                };
                meta.p.push({"simpleType": simpleType});
            });

            it('should get range inside editing', function () {
                var typeInfo = firstParameter();
                assert.isTrue(typeInfo.isInteger());
                assert.deepEqual(typeInfo.getRange(), {min: 0, max: 65535, step: 0.5});
            });

            it('should get null when min is undefined', function () {
                simpleType.editing.range["@minIncl"] = undefined;
                assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535, step: 0.5});
            });

            it('should get null when max is undefined', function () {
                simpleType.editing.range["@maxIncl"] = undefined;
                assert.deepEqual(firstParameter().getRange(), {min: 0, max: null, step: 0.5});
            });
        });

    });

    function create(metaDefinition) {
        return infoModel.typeInfo(metaDefinition);
    }

    function firstParameter() {
        return create(meta).getParameters()[0];
    }

});
