(function (global) {
    'use strict';

    describe('typeInfo v2', function () {
        var meta;

        beforeEach(function () {
            meta = {p: []};
        });

        describe('get range of simple type', function () {
            var parameter;

            describe('integer', function () {

                beforeEach(function () {
                    parameter = {
                        '@id': 1,
                        '@type': 'integer',
                        'range': {
                            '@min': '0',
                            '@max': '65535'
                        }
                    };
                    meta.p.push(parameter);
                });


                it('should get range inside editing', function () {
                    var typeInfo = firstParameter();
                    assert.isTrue(typeInfo.isInteger());
                    assert.deepEqual(typeInfo.getRange(), {min: 0, max: 65535, step: null});
                });

                it('should get null min range when min is undefined', function () {
                    parameter.range['@min'] = undefined;

                    assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535, step: null});
                });

                it('should get null max range when min is undefined', function () {
                    parameter.range['@max'] = undefined;

                    assert.deepEqual(firstParameter().getRange(), {min: 0, max: null, step: null});
                });
            });

            describe('string with length tag', function () {

                beforeEach(function () {
                    parameter = {
                        '@id': 1,
                        '@type': 'string',
                        'length': {
                            '@min': '0',
                            '@max': '65535'
                        }
                    };
                    meta.p.push(parameter);
                });


                it('should get range inside length', function () {
                    var typeInfo = firstParameter();
                    assert.isTrue(typeInfo.isString());
                    assert.deepEqual(typeInfo.getRange(), {min: 0, max: 65535, step: null});
                });

                it('should get null when min is undefined', function () {
                    parameter.length['@min'] = undefined;
                    assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535, step: null});
                });

                it('should get null when max is undefined', function () {
                    parameter.length['@max'] = undefined;
                    assert.deepEqual(firstParameter().getRange(), {min: 0, max: null, step: null});
                });
            });

            describe('double', function () {

                beforeEach(function () {
                    parameter = {
                        '@id': 1,
                        '@type': 'double',
                        'range': {
                            '@min': '0.5',
                            '@max': '65535.0'
                        }
                    };
                    meta.p.push(parameter);
                });

                it('should get range', function () {
                    var typeInfo = firstParameter();
                    assert.isTrue(typeInfo.isFloat());
                    assert.deepEqual(typeInfo.getRange(), {min: 0.5, max: 65535.0, step: null});
                });

                it('should get null when min is undefined', function () {
                    parameter.range['@min'] = undefined;
                    assert.deepEqual(firstParameter().getRange(), {min: null, max: 65535.0, step: null});
                });

                it('should get null when max is undefined', function () {
                    parameter.range['@max'] = undefined;
                    assert.deepEqual(firstParameter().getRange(), {min: 0.5, max: null, step: null});
                });
            });

        });

        function create(metaDefinition) {
            return infoModel.typeInfoV2(metaDefinition);
        }

        function firstParameter() {
            return create(meta).getParameters()[0];
        }
    });
})(this);
