const unzipper = require('../src/unzipper');
const testUtil = require('./test.util');

describe('unzipper', ()=> {

    it('should get file names', (done)=> {
        loadZipBuffer().then((zipBuffer)=> {
                const instance = unzipper.newInstance(zipBuffer);
                const fileNames = instance.getFileNames();

                assert.deepEqual(fileNames, [
                    'root_folder/',
                    'root_folder/folder/',
                    'root_folder/folder/file2.txt',
                    'root_folder/folder/file1.txt',
                    'root_folder/empty_folder/'
                ])
            })
            .then(done).catch(done);
    });

    it('should unzip by filename', (done)=> {
        loadZipBuffer().then((zipBuffer)=> {
                const instance = unzipper.newInstance(zipBuffer);
                const fileContent = instance.unzipByFileName('root_folder/folder/file1.txt');
                assert.deepEqual(fileContent, 'file1.txt\n');
            })
            .then(done).catch(done);
    });

    it('should unzip file content to Uint8Array', (done)=> {
        loadZipBuffer().then((zipBuffer)=> {
                const instance = unzipper.newInstance(zipBuffer);
                const fileContent = instance.unzipByFileName('root_folder/folder/file1.txt', {fileContentAsUint8: true});

                assert.deepEqual(fileContent.constructor, Uint8Array);
            })
            .then(done).catch(done);
    });

    it('should unzip all with fileIndex and total', (done)=> {
        const expected = [];
        const onEachFile = (filename, result, fileIndex, total)=> expected.push([fileIndex, total]);

        const onDone = ()=> {
            assert.deepEqual(expected, [
                [1, 5], [2, 5], [3, 5], [4, 5], [5, 5]
            ]);
            done();
        };

        loadZipBuffer().then((zipBuffer)=> {
                const instance = unzipper.newInstance(zipBuffer);
                instance.unzipAll(onEachFile, onDone);
            })
            .then(done).catch(done);
    });

    function loadZipBuffer() {
        return testUtil.loadBuffer('base/resources/zip/file.zip');
    }
});
