describe('Service: updateService#setMeta', function () {

    var updateService;
    var historyListener;


    beforeEach(function () {
        updateService = infoModel.updateService({metaService: mockMetaService()});
        historyListener = {
            entries: [],
            newEntry: function (entry) {
                this.entries.push(entry);
            },
            newEntries: function (entries) {
                this.entries = entries;
            },
            clearEntries: function () {
                this.entries = [];
            }
        };
    });

    it('should add history entry after meta was set', function (done) {
        updateService.addHistoryListener(historyListener);

        updateService.setMeta(new ArrayBuffer(0));
        updateService.addHistoryEntry(createEmptyEntry());
        updateService.addHistoryEntry(createEmptyEntry());

        updateService.notifyHistoryListeners()
            .then(function () {

                assert.lengthOf(historyListener.entries, 2);
                assert.ok(historyListener.entries[0].metaUid);
                assert.equal(historyListener.entries[0].metaUid, historyListener.entries[1].metaUid);

                done();
            });
    });

    it('should add history entry after meta was set multiple times', function (done) {
        updateService.addHistoryListener(historyListener);

        updateService.setMeta(new ArrayBuffer(0));
        updateService.addHistoryEntry(createEmptyEntry());

        updateService.setMeta(new ArrayBuffer(0));
        updateService.addHistoryEntry(createEmptyEntry());

        updateService.notifyHistoryListeners()
            .then(function () {

                assert.lengthOf(historyListener.entries, 2);
                assert.ok(historyListener.entries[0].metaUid);
                assert.notEqual(historyListener.entries[0].metaUid, historyListener.entries[1].metaUid);

                done();
            });
    });


    function createEmptyEntry() {
        return updateService.createUpdateEntry([], []);
    }

    function mockMetaService() {
        var metaService = {};
        metaService.metaUid = null;
        metaService.getLastMetaUid = function () {
            return metaService.metaUid;
        };
        metaService.setMeta = function () {
            return new Promise(function (resolve) {
                setTimeout(function () {
                    metaService.metaUid = Math.random();
                    resolve(metaService.metaUid);
                }, 200);
            });
        };
        return metaService;
    }

});
