describe('imCompatibilityService', function () {

    const imCompatibilityService = require('../../src/utils/imCompatibilityService');
    const ImTree = require('../../src/imtree');
    var imTree;

    beforeEach(function () {
        imTree = new ImTree();
    });

    it('get hardware distname - BTS_L exists', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1', {});
        assert.equal(imCompatibilityService.getHardwareDistName(imTree), '/MRBTS-1/RAT-1/BTS_L-1');
    });

    it('get hardware distname - no BTS_L', function () {
        assert.equal(imCompatibilityService.getHardwareDistName(imTree), '/MRBTS-1/RAT-1');
    });

    it('should not find node "A" - no NP node', function () {
        const rmodL = imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1', {});
        assert.isNull(imCompatibilityService.findNodeA(rmodL));
    });

    it('should not find node "A" - no "A" nodes at all', function () {
        addNpNode();
        const rmodL = imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1', {});
        assert.isUndefined(imCompatibilityService.findNodeA(rmodL));
    });

    it('should not find node "A" - no "A" node matching "L" node', function () {
        // given
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2'
        });
        const rmodL = imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1', {});
        // when then
        assert.isUndefined(imCompatibilityService.findNodeA(rmodL));
    });

    it('should find node "A"', function () {
        // given
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1/RMOD_A-1', {
            realDN: '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1'
        });
        const rmodL = imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-1', {});
        // when then
        assert.isDefined(imCompatibilityService.findNodeA(rmodL));
    });

    it('should not find current NP node - no BTS_CONF_L', function () {
        assert.isNull(imCompatibilityService.findCurrentNpNode(imTree));
    });

    it('should find current NP node', function () {
        addNpNode();
        assert.isDefined(imCompatibilityService.findCurrentNpNode(imTree));
    });

    it('should get EQM_L node', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1', {});
        assert.isDefined(imCompatibilityService.findEqmLNode(imTree));
    });

    it('should get HWTOP_L node', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/HWTOP_L-1', {});
        assert.isDefined(imCompatibilityService.findHwtopLNode(imTree));
    });

    it('should get BBTOP_L node', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BBTOP_L-1', {});
        assert.isDefined(imCompatibilityService.findBbtopLNode(imTree));
    });

    it('should get LNBTS node', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1', {});
        assert.isDefined(imCompatibilityService.findLnbtsNode(imTree));
    });

    it('should not find MRBTS_A node - no NP node', function () {
        assert.isNull(imCompatibilityService.findMrbtsANode(imTree));
    });

    it('should not find MRBTS_A node - no SCF node', function () {
        addNpNode();
        assert.isUndefined(imCompatibilityService.findMrbtsANode(imTree));
    });

    it('should not find MRBTS_A node - no MRBTS_A in SCF', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1', {});
        assert.isUndefined(imCompatibilityService.findMrbtsANode(imTree));
    });

    it('should find MRBTS_A node', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1', {});
        assert.isDefined(imCompatibilityService.findMrbtsANode(imTree));
    });

    it('should not find LNBTS_A node - no MRBTS_A node', function () {
        assert.isNull(imCompatibilityService.findLnbtsANode(imTree));
    });

    it('should find LNBTS_A node', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/LNBTS_A-1', {});
        assert.isDefined(imCompatibilityService.findLnbtsANode(imTree));
    });

    it('should not find EQM_A node - no MRBTS_A node', function () {
        assert.isNull(imCompatibilityService.findEqmANode(imTree));
    });

    it('should find EQM_A node', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/EQM_A-1', {});
        assert.isDefined(imCompatibilityService.findEqmANode(imTree));
    });

    it('should not find HWTOP_A node - no MRBTS_A node', function () {
        assert.isNull(imCompatibilityService.findHwtopANode(imTree));
    });

    it('should find HWTOP_A node', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/HWTOP_A-1', {});
        assert.isDefined(imCompatibilityService.findHwtopANode(imTree));
    });

    it('should not find LNCEL_A nodes - no MRBTS_A node', function () {
        assert.lengthOf(imCompatibilityService.findLncelANodes(imTree), 0);
    });

    it('should not find LNCEL_A nodes - no LNBTS_A node', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1', {});
        assert.lengthOf(imCompatibilityService.findLncelANodes(imTree), 0);
    });

    it('should find LNCEL_A nodes', function () {
        addNpNode();
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/LNBTS_A-1/LNCEL_A-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1/SCF-1/MRBTS_A-1/LNBTS_A-1/LNCEL_A-2', {});
        assert.lengthOf(imCompatibilityService.findLncelANodes(imTree), 2);
    });

    it('should not find LNCEL nodes - no LNBTS node', function () {
        assert.lengthOf(imCompatibilityService.findLncelNodes(imTree), 0);
    });

    it('should find LNCEL nodes', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/LNCEL-1', {});
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/LNCEL-2', {});
        assert.lengthOf(imCompatibilityService.findLncelNodes(imTree), 2);
    });

    it('should not find FTM node', function () {
        assert.isUndefined(imCompatibilityService.findFtmNode(imTree));
    });

    it('should find FTM node', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/LNBTS-1/FTM-1', {});
        assert.isDefined(imCompatibilityService.findFtmNode(imTree));
    });

    it('should not find CRAN_CONFIG - no FUNC_L node', function () {
        assert.isUndefined(imCompatibilityService.findCranConfig(imTree));
    });

    it('should find CRAN_CONFIG', function () {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/FUNC_L-1/CRAN_CONFIG-3', {});
        assert.isDefined(imCompatibilityService.findCranConfig(imTree));
    });

    it('should not get lnmme primary ip address - no object in params', function () {
        const lnmmeParams = {
            ipAddrPrimFromFaultReq: '1'
        };
        const addr = imCompatibilityService.getLnmmePrimaryIpAddress(lnmmeParams);
        assert.isUndefined(addr.address);
        assert.equal(addr.state, '1');
    });

    it('should get lnmme primary ip address', function () {
        const lnmmeParams = {
            ipAddrPrim: {
                ip_value: '1',
                faultyIPAddress: '2'
            }
        };
        const addr = imCompatibilityService.getLnmmePrimaryIpAddress(lnmmeParams);
        assert.equal(addr.address, '1');
        assert.equal(addr.state, '2');
    });

    it('should not get lnmme secondary ip address - no object in params', function () {
        const lnmmeParams = {
            ipAddrSecFromFaultReq: '1'
        };
        const addr = imCompatibilityService.getLnmmeSecondaryIpAddress(lnmmeParams);
        assert.isUndefined(addr.address);
        assert.equal(addr.state, '1');
    });

    it('should get lnmme primary ip address', function () {
        const lnmmeParams = {
            ipAddrSec: {
                ip_value: '1',
                faultyIPAddress: '2'
            }
        };
        const addr = imCompatibilityService.getLnmmeSecondaryIpAddress(lnmmeParams);
        assert.equal(addr.address, '1');
        assert.equal(addr.state, '2');
    });

    it('should adapt cablnk mode to new version', function () {
        const linkMode1 = 'Configuration';
        const linkMode2 = 'Smth else';
        const adaptedLinkMode1 = imCompatibilityService.adaptCablnkLinkMode(linkMode1);
        const adaptedLinkMode2 = imCompatibilityService.adaptCablnkLinkMode(linkMode2);
        assert.isTrue(adaptedLinkMode1.configuration);
        assert.isFalse(adaptedLinkMode2.configuration);
    });

    it('should return link mode as it is', function () {
        const linkMode = {
            configuration: false
        };
        const adaptedLinkMode = imCompatibilityService.adaptCablnkLinkMode(linkMode);
        assert.equal(adaptedLinkMode, linkMode);
    });


    it('should return hwlinks with name HWLINK_L', function () {
        const HWTOP_DISTNAME = '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/HWTOP_L-1';
        const HWLINK_DISTNAME_1 = HWTOP_DISTNAME + '/HWLINK_L-117';
        const HWLINK_DISTNAME_2 = HWTOP_DISTNAME + '/HWLINK_NG_L-1';

        const hwtop = imTree.mergeNode(HWTOP_DISTNAME);
        imTree.mergeNode(HWLINK_DISTNAME_1);
        imTree.mergeNode(HWLINK_DISTNAME_2);

        const hwLinks = imCompatibilityService.findHwLinks(hwtop);

        assert.equal(hwLinks[0].distName, HWLINK_DISTNAME_1);
        assert.equal(hwLinks[1].distName, HWLINK_DISTNAME_2);
    });

    it('should return undefined if there is no CABINET_L object', () => {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1', {});
        assert.isUndefined(imCompatibilityService.findCabinetLNode(imTree));
    });

    it('should find CABINET_L object if there is one', () => {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/CABINET_L-2', {});
        assert.isDefined(imCompatibilityService.findCabinetLNode(imTree));
    });

    describe('getActiveBuildData', () => {

        it('should not get build data when it doesn\'t exist under storage', () => {
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/SOC_L-1/CPU_L-1/STORAGE_L-1', {});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/SOC_L-1/CPU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1', {});
            const storage = imTree.findByDistName('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/SOC_L-1/CPU_L-1/STORAGE_L-1');
            assert.isFalse(!!imCompatibilityService.getActiveBuildData(storage));
        });

        it('should get active build data when there\'s only one file system', () => {
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2/RU_L-1/STORAGE_L-1', {});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2/RU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1', {});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2/RU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1/BUILD_DATA_L-1', {});
            const storage = imTree.findByDistName('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2/RU_L-1/STORAGE_L-1');

            const activeBuildData = imCompatibilityService.getActiveBuildData(storage);

            assert.isDefined(activeBuildData);
            assert.deepEqual(activeBuildData.distName, '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/RMOD_L-2/RU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1/BUILD_DATA_L-1');
        });

        it('should get active build data when there are two file systems', () => {
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1', {});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1', {bootPurpose: 'Active'});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1/FILE_SYSTEM_L-2', {bootPurpose: 'Passive'});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1/BUILD_DATA_L-1', {});
            imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1/FILE_SYSTEM_L-2/BUILD_DATA_L-1', {});
            const storage = imTree.findByDistName('/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1');

            const activeBuildData = imCompatibilityService.getActiveBuildData(storage);

            assert.isDefined(activeBuildData);
            assert.deepEqual(activeBuildData.distName, '/MRBTS-1/RAT-1/BTS_L-1/EQM_L-1/SMOD_L-1/CCU_L-1/STORAGE_L-1/FILE_SYSTEM_L-1/BUILD_DATA_L-1');
        });

    });

    function addNpNode() {
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1', {
            currentNp: '/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1'
        });
        imTree.mergeNode('/MRBTS-1/RAT-1/BTS_L-1/BTS_CONF-1/NP-1', {});
    }
});
