const objects = require('../../src/utils/objects');

describe('objects utils', () => {

    describe('filterObject', () => {
        const parametersFixture = {
            foo: 123,
            bar: {
                x: 1,
                y: {
                    ffoobar: 'something'
                }
            },
            foobar: [1, 2, 3]
        };
        const filterObject = objects.filterObject;

        it('should return empty object for an empty input object', () => {
            //when
            const result =  filterObject({}, '', 0);
            //then
            assert.deepEqual(result, {});
        });

        it('should return same object as an input', () => {
            //given
            const regexp = new RegExp(''); // /(?:)/
            //when
            const result1 =  filterObject(parametersFixture, regexp, -2);
            const result2 =  filterObject(parametersFixture, regexp, -1);
            const result3 =  filterObject(parametersFixture, regexp, 0);
            const result4 =  filterObject(parametersFixture, regexp, 1);
            const result5 =  filterObject(parametersFixture, regexp, 2);
            const result6 =  filterObject(parametersFixture);
            //then
            assert.deepEqual(result1, parametersFixture);
            assert.deepEqual(result2, parametersFixture);
            assert.deepEqual(result3, parametersFixture);
            assert.deepEqual(result4, parametersFixture);
            assert.deepEqual(result5, parametersFixture);
            assert.deepEqual(result6, parametersFixture);
        });

        it('should return filtered object for different patterns and depths', () => {
            //given
            const regexp1 = new RegExp('foo');
            const regexp2 = new RegExp('^foo');
            const regexp3 = new RegExp('x');
            const regexp4 = new RegExp('^ffoobar$');
            //when
            const result1 =  filterObject(parametersFixture, regexp1, -1);
            const result2 =  filterObject(parametersFixture, regexp1, 0);
            const result3 =  filterObject(parametersFixture, regexp2, -1);
            const result4 =  filterObject(parametersFixture, regexp3, -1);
            const result5 =  filterObject(parametersFixture, regexp4, 1);
            //then
            assert.deepEqual(result1,
                {
                    foo: 123,
                    bar: {
                        y: {
                            ffoobar: 'something'
                        }
                    },
                    foobar: [1, 2, 3]
                }
            );
            assert.deepEqual(result2,
                {
                    foo: 123,
                    foobar: [1, 2, 3]
                }
            );
            assert.deepEqual(result3,
                {
                    foo: 123,
                    foobar: [1, 2, 3]
                }
            );
            assert.deepEqual(result4,
                {
                    bar: {
                        x: 1
                    }
                }
            );
            assert.deepEqual(result5, {});
        });

    });
});
