const xml2json = require('../src/xml2json');

describe('xml2json', function () {

    function convert(xmlString) {
        return xml2json.toJson(xmlString);
    }

    it('should covert empty element', function () {
        var obj = convert("<a></a>");

        assert.deepEqual(obj, {a: null});
    });

    it('should covert closed element', function () {
        var obj = convert("<a/>");

        assert.deepEqual(obj, {a: null});
    });

    it('should covert text element', function () {
        var obj = convert("<a>text</a>");

        assert.deepEqual(obj, {a: 'text'});
    });

    it('should covert nested elements', function () {
        var obj = convert("<a><b></b><c></c></a>");

        assert.deepEqual(obj, {a: {b: null, c: null}});
    });

    it('should covert nested elements with text', function () {
        var obj = convert("<a> text <b></b></a>");

        assert.deepEqual(obj, {a: {'#text': ' text ', b: null}});
    });

    it('should covert multiple elements', function () {
        var obj = convert("<a><b>1</b><b>2</b><b>3</b></a>");

        assert.deepEqual(obj, {a: {b: ['1', '2', '3']}});
    });

    it('should covert nested multiple elements', function () {
        var obj = convert("<a><b><c>1</c></b><b><d>2</d></b></a>");

        assert.deepEqual(obj, {a: {b: [{c: '1'}, {d: '2'}]}});
    });

    it('should covert attribute', function () {
        var obj = convert("<a name='x'></a>");

        assert.deepEqual(obj, {a: {'@name': 'x'}});
    });

    it('should covert attribute and text', function () {
        var obj = convert("<a name='x'>ala</a>");

        assert.deepEqual(obj, {a: {'@name': 'x', '#text': 'ala'}});
    });

    it('should skip comment', function () {
        var obj = convert("<a><!-- comment --></a>");

        assert.deepEqual(obj, {a: null});
    });

});
