const testUtil = require('../test.util');

describe('xz decoder', function () {
    const xzDecoder = infoModel.xz;
    const dummyListener = {
        onComplete: function (decodedXz) {
        },
        onReadBytes: function (loaded, total) {
        }
    };

    it('should decode xz format', function (done) {
        var bytesBuffer = new Int8Array([-3, 55, 122, 88, 90, 0, 0, 1, 105, 34, -34, 54, 2, 0,
            33, 1, 22, 0, 0, 0, 116, 47, -27, -93, 1, 0, 10, 120, 122, 95, 100, 101, 99, 111,
            100, 101, 100, 10, 0, 0, -27, -25, -41, 96, 0, 1, 31, 11, 61, 98, 14, 122, -112, 66,
            -103, 13, 1, 0, 0, 0, 0, 1, 89, 90]).buffer;

        xzDecoder.decode(bytesBuffer, assertDecoded(function (decodedXz) {
            assert.equal(arrayBuffer2String(decodedXz), 'xz_decoded\n');
            done();
        }));
    });

    it('should not decode xz format - invalid format id', function () {
        var bytesBuffer = new Int8Array([0, 55, 122, 88, 90, 0]).buffer;

        expect(xzDecoder.decode.bind(this, bytesBuffer, dummyListener)).to.throw('XZFormatException');
    });

    it('should not decode xz format - no all bytes at the end', function () {
        var bytesBuffer = new Int8Array([-3, 55, 122, 88, 90, 0, 0, 1, 105, 34, -34, 54, 2, 0,
            33, 1, 22, 0, 0, 0, 116, 47, -27, -93, 1, 0, 10, 120, 122, 95, 100, 101, 99, 111,
            100, 101, 100, 10, 0, 0, -27, -25, -41, 96, 0, 1, 31]).buffer;

        expect(xzDecoder.decode.bind(this, bytesBuffer, dummyListener)).to.throw('EOFException');
    });

    it('should decode xz file 1', function (done) {
        assertEqualTwoFiles(
            'base/resources/xz/BTS6249_1011_pm_im_snapshot.ims2.xz',
            'base/resources/xz/BTS6249_1011_pm_im_snapshot.dat',
            done
        );
    });

    it('should decode xz file 2', function (done) {
        assertEqualTwoFiles(
            'base/resources/xz/snapshot.ims2.xz',
            'base/resources/xz/snapshot.dat',
            done
        );
    });

    function assertEqualTwoFiles(path1, path2, done) {
        loadBuffer(path1).then(decodeXz);

        function decodeXz(xzArrayBuffer) {
            xzDecoder.decode(xzArrayBuffer, assertDecoded(function (decodedXz) {
                loadBuffer(path2).then(compareMD5With(decodedXz));
            }));
        }

        function compareMD5With(decodedXz) {
            return function (imsArrayBuffer) {
                assert.equal(decodedXz.byteLength, imsArrayBuffer.byteLength);
                assert.equal(
                    SparkMD5.ArrayBuffer.hash(decodedXz),
                    SparkMD5.ArrayBuffer.hash(imsArrayBuffer)
                );

                done();
            }
        }
    }

    function assertDecoded(fn) {
        return {
            onComplete: function (decodedXz) {
                fn(decodedXz);
            },
            onReadBytes: function (loaded, total) {
            }
        }
    }

    function arrayBuffer2String(ab) {
        return String.fromCharCode.apply(null, new Uint8Array(ab));
    }

    function loadBuffer(file) {
        return testUtil.loadBuffer(file);
    }
});
